@@ -1,3 +1,32 @@
+/**
+ * Loads history and gameplay data of all civs.
+ *
+ * @param selectableOnly {boolean} - Only load civs that can be selected
+ *        in the gamesetup. Scenario maps might set non-selectable civs.
+ */
+function loadCivFiles(selectableOnly)
+{
+	let propertyNames = [
+		"Code", "Culture", "Name", "Emblem", "History", "Music", "Factions", "CivBonuses", "TeamBonuses",
+		"Structures", "StartEntities", "Formations", "AINames", "SkirmishReplacements", "SelectableInGameSetup"];
+
+	let civData = {};
+
+	for (let filename of Engine.ListDirectoryFiles("simulation/data/civs/", "*.json", false))
+	{
+		let data = Engine.ReadJSONFile(filename);
+
+		for (let prop of propertyNames)
+			if (data[prop] === undefined)
+				throw new Error(filename + " doesn't contain " + prop);
+
+		if (!selectableOnly || data.SelectableInGameSetup)
+			civData[data.Code] = data;
+	}
+
+	return civData;
+}
+
 /**
  * Gets an array of all classes for this identity template
  */
@@ -27,34 +56,35 @@ function GetVisibleIdentityClasses(template)
 }
 
 /**
- * Check if the classes given in the identity template
- * match a list of classes
- * @param classes List of the classes to check against
- * @param match Either a string in the form
+ * Check if a given list of classes matches another list of classes.
+ * Useful f.e. for checking identity classes.
+ *
+ * @param classes - List of the classes to check against.
+ * @param match - Either a string in the form
  *     "Class1 Class2+Class3"
  * where spaces are handled as OR and '+'-signs as AND,
- * and ! is handled as NOT, thus Class1+!Class2 = Class1 AND NOT Class2
+ * and ! is handled as NOT, thus Class1+!Class2 = Class1 AND NOT Class2.
  * Or a list in the form
  *     [["Class1"], ["Class2", "Class3"]]
- * where the outer list is combined as OR, and the inner lists are AND-ed
+ * where the outer list is combined as OR, and the inner lists are AND-ed.
  * Or a hybrid format containing a list of strings, where the list is
- * combined as OR, and the strings are split by space and '+' and AND-ed
+ * combined as OR, and the strings are split by space and '+' and AND-ed.
  *
  * @return undefined if there are no classes or no match object
  * true if the the logical combination in the match object matches the classes
- * false otherwise
+ * false otherwise.
  */
 function MatchesClassList(classes, match)
 {
 	if (!match || !classes)
 		return undefined;
-	// transform the string to an array
+	// Transform the string to an array
 	if (typeof match == "string")
 		match = match.split(/\s+/);
 
-	for (var sublist of match)
+	for (let sublist of match)
 	{
-		// if the elements are still strings, split them by space or by '+'
+		// If the elements are still strings, split them by space or by '+'
 		if (typeof sublist == "string")
 			sublist = sublist.split(/[+\s]+/);
 		if (sublist.every(c => (c[0] == "!" && classes.indexOf(c.substr(1)) == -1)
@@ -116,11 +146,12 @@ function GetModifiedTemplateDataValue(template, value_path, mod_key, player, mod
  *                          of properties.
  * @param {object} auraTemplates - In the form of { key: { "auraName": "", "auraDescription": "" } }.
  * @param {object} resources - An instance of the Resources prototype.
+ * @param {object} damageTypes - An instance of the DamageTypes prototype.
  * @param {object} modifiers - Modifications from auto-researched techs, unit upgrades
  *                             etc. Optional as only used if there's no player
  *                             id provided.
  */
-function GetTemplateDataHelper(template, player, auraTemplates, resources, modifiers={})
+function GetTemplateDataHelper(template, player, auraTemplates, resources, damageTypes, modifiers={})
 {
 	// Return data either from template (in tech tree) or sim state (ingame).
 	// @param {string} value_path - Route to the value within the template.
@@ -132,11 +163,11 @@ function GetTemplateDataHelper(template, player, auraTemplates, resources, modif
 	let ret = {};
 
 	if (template.Armour)
-		ret.armour = {
-			"hack": getEntityValue("Armour/Hack"),
-			"pierce": getEntityValue("Armour/Pierce"),
-			"crush": getEntityValue("Armour/Crush")
-		};
+	{
+		ret.armour = {};
+		for (let damageType of damageTypes.GetTypes())
+			ret.armour[damageType] = getEntityValue("Armour/" + damageType);
+	}
 
 	if (template.Attack)
 	{
@@ -154,31 +185,41 @@ function GetTemplateDataHelper(template, player, auraTemplates, resources, modif
 			else
 			{
 				ret.attack[type] = {
-					"hack": getAttackStat("Hack"),
-					"pierce": getAttackStat("Pierce"),
-					"crush": getAttackStat("Crush"),
 					"minRange": getAttackStat("MinRange"),
 					"maxRange": getAttackStat("MaxRange"),
 					"elevationBonus": getAttackStat("ElevationBonus")
 				};
+				for (let damageType of damageTypes.GetTypes())
+					ret.attack[type][damageType] = getAttackStat(damageType);
+
 				ret.attack[type].elevationAdaptedRange = Math.sqrt(ret.attack[type].maxRange *
 					(2 * ret.attack[type].elevationBonus + ret.attack[type].maxRange));
 			}
 			ret.attack[type].repeatTime = getAttackStat("RepeatTime");
 
 			if (template.Attack[type].Splash)
+			{
 				ret.attack[type].splash = {
-					"hack": getAttackStat("Splash/Hack"),
-					"pierce": getAttackStat("Splash/Pierce"),
-					"crush": getAttackStat("Splash/Crush"),
 					// true if undefined
 					"friendlyFire": template.Attack[type].Splash.FriendlyFire != "false",
 					"shape": template.Attack[type].Splash.Shape
 				};
+				for (let damageType of damageTypes.GetTypes())
+					ret.attack[type].splash[damageType] = getAttackStat("Splash/" + damageType);
+			}
 		}
 	}
 
-	if (template.Auras)
+	if (template.DeathDamage)
+	{
+		ret.deathDamage = {
+			"friendlyFire": template.DeathDamage.FriendlyFire != "false"
+		};
+		for (let damageType of damageTypes.GetTypes())
+			ret.deathDamage[damageType] = getEntityValue("DeathDamage/" + damageType);
+	}
+
+	if (template.Auras && auraTemplates)
 	{
 		ret.auras = {};
 		for (let auraID of template.Auras._string.split(/\s+/))
@@ -216,10 +257,10 @@ function GetTemplateDataHelper(template, player, auraTemplates, resources, modif
 			};
 
 			if (template.BuildRestrictions.Distance.MinDistance)
-				ret.buildRestrictions.distance.min = +template.BuildRestrictions.Distance.MinDistance;
+				ret.buildRestrictions.distance.min = getEntityValue("BuildRestrictions/Distance/MinDistance");
 
 			if (template.BuildRestrictions.Distance.MaxDistance)
-				ret.buildRestrictions.distance.max = +template.BuildRestrictions.Distance.MaxDistance;
+				ret.buildRestrictions.distance.max = getEntityValue("BuildRestrictions/Distance/MaxDistance");
 		}
 	}
 
@@ -349,6 +390,7 @@ function GetTemplateDataHelper(template, player, auraTemplates, resources, modif
 		ret.tooltip = template.Identity.Tooltip;
 		ret.requiredTechnology = template.Identity.RequiredTechnology;
 		ret.visibleIdentityClasses = GetVisibleIdentityClasses(template.Identity);
+		ret.nativeCiv = template.Identity.Civ;
 	}
 
 	if (template.UnitMotion)
@@ -397,20 +439,32 @@ function GetTemplateDataHelper(template, player, auraTemplates, resources, modif
 		};
 
 	if (template.WallSet)
+	{
 		ret.wallSet = {
 			"templates": {
 				"tower": template.WallSet.Templates.Tower,
 				"gate": template.WallSet.Templates.Gate,
+				"fort": template.WallSet.Templates.Fort || "structures/" + template.Identity.Civ + "_fortress",
 				"long": template.WallSet.Templates.WallLong,
 				"medium": template.WallSet.Templates.WallMedium,
-				"short": template.WallSet.Templates.WallShort,
+				"short": template.WallSet.Templates.WallShort
 			},
 			"maxTowerOverlap": +template.WallSet.MaxTowerOverlap,
-			"minTowerOverlap": +template.WallSet.MinTowerOverlap,
+			"minTowerOverlap": +template.WallSet.MinTowerOverlap
 		};
+		if (template.WallSet.Templates.WallEnd)
+			ret.wallSet.templates.end = template.WallSet.Templates.WallEnd;
+		if (template.WallSet.Templates.WallCurves)
+			ret.wallSet.templates.curves = template.WallSet.Templates.WallCurves.split(" ");
+	}
 
 	if (template.WallPiece)
-		ret.wallPiece = { "length": +template.WallPiece.Length };
+		ret.wallPiece = {
+			"length": +template.WallPiece.Length,
+			"angle": +(template.WallPiece.Orientation || 1) * Math.PI,
+			"indent": +(template.WallPiece.Indent || 0),
+			"bend": +(template.WallPiece.Bend || 0) * Math.PI
+		};
 
 	return ret;
 }
@@ -430,7 +484,8 @@ function GetTechnologyBasicDataHelper(template, civ)
 		"description": template.description,
 		"reqs": DeriveTechnologyRequirements(template, civ),
 		"modifications": template.modifications,
-		"affects": template.affects
+		"affects": template.affects,
+		"replaces": template.replaces
 	};
 }
 
@@ -474,3 +529,12 @@ function calculateCarriedResources(carriedResources, tradingGoods)
 	return resources;
 }
 
+/**
+ * Remove filter prefix (mirage, corpse, etc) from template name.
+ *
+ * ie. filter|dir/to/template -> dir/to/template
+ */
+function removeFiltersFromTemplateName(templateName)
+{
+	return templateName.split("|").pop();
+}
