@@ -7,14 +7,12 @@ m.SharedScript = function(settings)
 	if (!settings)
 		return;
 
-	this._players = settings.players;
+	this._players = Object.keys(settings.players).map(key => settings.players[key]); // TODO SM55 Object.values(settings.players)
 	this._templates = settings.templates;
-	this._derivedTemplates = {};
-	this._techTemplates = settings.techTemplates;
 
 	this._entityMetadata = {};
-	for (let i in this._players)
-		this._entityMetadata[this._players[i]] = {};
+	for (let player of this._players)
+		this._entityMetadata[player] = {};
 
 	// array of entity collections
 	this._entityCollections = new Map();
@@ -31,7 +29,6 @@ m.SharedScript.prototype.Serialize = function()
 {
 	return {
 		"players": this._players,
-		"techTemplates": this._techTemplates,
 		"templatesModifications": this._templatesModifications,
 		"entitiesModifications": this._entitiesModifications,
 		"metadata": this._entityMetadata
@@ -45,92 +42,19 @@ m.SharedScript.prototype.Serialize = function()
 m.SharedScript.prototype.Deserialize = function(data)
 {
 	this._players = data.players;
-	this._techTemplates = data.techTemplates;
 	this._templatesModifications = data.templatesModifications;
 	this._entitiesModifications = data.entitiesModifications;
 	this._entityMetadata = data.metadata;
-	this._derivedTemplates = {};
 
 	this.isDeserialized = true;
 };
 
-/**
- * Components that will be disabled in foundation entity templates.
- * (This is a bit yucky and fragile since it's the inverse of
- * CCmpTemplateManager::CopyFoundationSubset and only includes components
- * that our Template class currently uses.)
- */
-m.g_FoundationForbiddenComponents = {
-	"ProductionQueue": 1,
-	"ResourceSupply": 1,
-	"ResourceDropsite": 1,
-	"GarrisonHolder": 1,
-	"Capturable": 1
-};
-
-/**
- * Components that will be disabled in resource entity templates.
- * Roughly the inverse of CCmpTemplateManager::CopyResourceSubset.
- */
-m.g_ResourceForbiddenComponents = {
-	"Cost": 1,
-	"Decay": 1,
-	"Health": 1,
-	"UnitAI": 1,
-	"UnitMotion": 1,
-	"Vision": 1
-};
-
 m.SharedScript.prototype.GetTemplate = function(name)
 {
-	if (this._templates[name])
-		return this._templates[name];
+	if (this._templates[name] === undefined)
+		this._templates[name] = Engine.GetTemplate(name) || null;
 
-	if (this._derivedTemplates[name])
-		return this._derivedTemplates[name];
-
-	// If this is a foundation template, construct it automatically
-	if (name.indexOf("foundation|") !== -1)
-	{
-		let base = this.GetTemplate(name.substr(11));
-
-		let foundation = {};
-		for (let key in base)
-			if (!m.g_FoundationForbiddenComponents[key])
-				foundation[key] = base[key];
-
-		this._derivedTemplates[name] = foundation;
-		return foundation;
-	}
-	else if (name.indexOf("resource|") !== -1)
-	{
-		let base = this.GetTemplate(name.substr(9));
-
-		let resource = {};
-		for (let key in base)
-			if (!m.g_ResourceForbiddenComponents[key])
-				resource[key] = base[key];
-
-		this._derivedTemplates[name] = resource;
-		return resource;
-	}
-	else if (name.indexOf("ungarrisonable|") !== -1)
-	{
-		let base = this.GetTemplate(name.substr(15));
-
-		let ent = {};
-		for (let key in base)
-			if (key !== "Garrisonable")
-				ent[key] = base[key];
-			else
-				ent[key] = "false";
-
-		this._derivedTemplates[name] = ent;
-		return ent;
-	}
-
-	error("Tried to retrieve invalid template '"+name+"'");
-	return null;
+	return this._templates[name];
 };
 
 /**
@@ -141,23 +65,19 @@ m.SharedScript.prototype.GetTemplate = function(name)
 m.SharedScript.prototype.init = function(state, deserialization)
 {
 	if (!deserialization)
-	{
 		this._entitiesModifications = new Map();
-		for (let i = 0; i < state.players.length; ++i)
-			this._templatesModifications[i] = {};
-	}
 
 	this.ApplyTemplatesDelta(state);
 
 	this.passabilityClasses = state.passabilityClasses;
-	this.players = this._players;
 	this.playersData = state.players;
 	this.timeElapsed = state.timeElapsed;
 	this.circularMap = state.circularMap;
 	this.mapSize = state.mapSize;
-	this.gameType = state.gameType;
+	this.victoryConditions = new Set(state.victoryConditions);
 	this.alliedVictory = state.alliedVictory;
 	this.ceasefireActive = state.ceasefireActive;
+	this.ceasefireTimeRemaining = state.ceasefireTimeRemaining / 1000;
 
 	this.passabilityMap = state.passabilityMap;
 	if (this.mapSize % this.passabilityMap.width !== 0)
@@ -195,9 +115,6 @@ m.SharedScript.prototype.init = function(state, deserialization)
 	this.accessibility = new m.Accessibility();
 	this.accessibility.init(state, this.terrainAnalyzer);
 
-	// Setup resources
-	this.resourceInfo = state.resources;
-	m.Resources.prototype.types = state.resources.codes;
 	// Resource types: ignore = not used for resource maps
 	//                 abundant = abundant resource with small amount each
 	//                 sparse = sparse resource, but huge amount each
@@ -211,10 +128,10 @@ m.SharedScript.prototype.init = function(state, deserialization)
 	this.createResourceMaps();
 
 	this.gameState = {};
-	for (let i in this._players)
+	for (let player of this._players)
 	{
-		this.gameState[this._players[i]] = new m.GameState();
-		this.gameState[this._players[i]].init(this,state, this._players[i]);
+		this.gameState[player] = new m.GameState();
+		this.gameState[player].init(this, state, player);
 	}
 };
 
@@ -243,6 +160,7 @@ m.SharedScript.prototype.onUpdate = function(state)
 	this.timeElapsed = state.timeElapsed;
 	this.barterPrices = state.barterPrices;
 	this.ceasefireActive = state.ceasefireActive;
+	this.ceasefireTimeRemaining = state.ceasefireTimeRemaining / 1000;
 
 	this.passabilityMap = state.passabilityMap;
 	this.passabilityMap.cellSize = this.mapSize / this.passabilityMap.width;
@@ -267,10 +185,8 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 	// by order of updating:
 	// we "Destroy" last because we want to be able to switch Metadata first.
 
-	let CreateEvents = state.events.Create;
-	for (let i = 0; i < CreateEvents.length; ++i)
+	for (let evt of state.events.Create)
 	{
-		let evt = CreateEvents[i];
 		if (!state.entities[evt.entity])
 			continue; // Sometimes there are things like foundations which get destroyed too fast
 
@@ -285,49 +201,39 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 
 	for (let evt of state.events.EntityRenamed)
 	{	// Switch the metadata: TODO entityCollections are updated only because of the owner change. Should be done properly
-		for (let p in this._players)
+		for (let player of this._players)
 		{
-			this._entityMetadata[this._players[p]][evt.newentity] = this._entityMetadata[this._players[p]][evt.entity];
-			this._entityMetadata[this._players[p]][evt.entity] = {};
+			this._entityMetadata[player][evt.newentity] = this._entityMetadata[player][evt.entity];
+			this._entityMetadata[player][evt.entity] = {};
 		}
 	}
 
 	for (let evt of state.events.TrainingFinished)
 	{	// Apply metadata stored in training queues
 		for (let entId of evt.entities)
-			for (let key in evt.metadata)
-				this.setMetadata(evt.owner, this._entities.get(entId), key, evt.metadata[key]);
+			if (this._entities.has(entId))
+				for (let key in evt.metadata)
+					this.setMetadata(evt.owner, this._entities.get(entId), key, evt.metadata[key]);
 	}
 
 	for (let evt of state.events.ConstructionFinished)
-	{	// we'll move metadata.
-		if (!this._entities.has(evt.entity))
-			continue;
-		let ent = this._entities.get(evt.entity);
-		let newEnt = this._entities.get(evt.newentity);
-		if (this._entityMetadata[ent.owner()] && this._entityMetadata[ent.owner()][evt.entity] !== undefined)
-			for (let key in this._entityMetadata[ent.owner()][evt.entity])
-				this.setMetadata(ent.owner(), newEnt, key, this._entityMetadata[ent.owner()][evt.entity][key]);
-		foundationFinished[evt.entity] = true;
+	{
+		// metada are already moved by EntityRenamed when needed (i.e. construction, not repair)
+		if (evt.entity != evt.newentity)
+			foundationFinished[evt.entity] = true;
 	}
 
 	for (let evt of state.events.AIMetadata)
 	{
 		if (!this._entities.has(evt.id))
 			continue;	// might happen in some rare cases of foundations getting destroyed, perhaps.
-						// Apply metadata (here for buildings for example)
+		// Apply metadata (here for buildings for example)
 		for (let key in evt.metadata)
 			this.setMetadata(evt.owner, this._entities.get(evt.id), key, evt.metadata[key]);
 	}
 
-	let DestroyEvents = state.events.Destroy;
-	for (let i = 0; i < DestroyEvents.length; ++i)
+	for (let evt of state.events.Destroy)
 	{
-		let evt = DestroyEvents[i];
-		// A small warning: javascript "delete" does not actually delete, it only removes the reference in this object.
-		// the "deleted" object remains in memory, and any older reference to it will still reference it as if it were not "deleted".
-		// Worse, they might prevent it from being garbage collected, thus making it stay alive and consuming ram needlessly.
-		// So take care, and if you encounter a weird bug with deletion not appearing to work correctly, this is probably why.
 		if (!this._entities.has(evt.entity))
 			continue;// probably should remove the event.
 
@@ -338,8 +244,8 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 		// remember the entity and this AI's metadata concerning it
 		evt.metadata = {};
 		evt.entityObj = this._entities.get(evt.entity);
-		for (let j in this._players)
-			evt.metadata[this._players[j]] = this._entityMetadata[this._players[j]][evt.entity];
+		for (let player of this._players)
+			evt.metadata[player] = this._entityMetadata[player][evt.entity];
 
 		let entity = this._entities.get(evt.entity);
 		for (let entCol of this._entityCollections.values())
@@ -348,8 +254,8 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 
 		this._entities.delete(evt.entity);
 		this._entitiesModifications.delete(evt.entity);
-		for (let j in this._players)
-			delete this._entityMetadata[this._players[j]][evt.entity];
+		for (let player of this._players)
+			delete this._entityMetadata[player][evt.entity];
 	}
 
 	for (let id in state.entities)
@@ -389,9 +295,11 @@ m.SharedScript.prototype.ApplyTemplatesDelta = function(state)
 		for (let template in playerDiff)
 		{
 			let changes = playerDiff[template];
-			if (!this._templatesModifications[player][template])
-				this._templatesModifications[player][template] = new Map();
-			let modif = this._templatesModifications[player][template];
+			if (!this._templatesModifications[template])
+				this._templatesModifications[template] = {};
+			if (!this._templatesModifications[template][player])
+				this._templatesModifications[template][player] = new Map();
+			let modif = this._templatesModifications[template][player];
 			for (let change of changes)
 				modif.set(change.variable, change.value);
 		}
@@ -437,7 +345,10 @@ m.SharedScript.prototype.setMetadata = function(player, ent, key, value)
 {
 	let metadata = this._entityMetadata[player][ent.id()];
 	if (!metadata)
-		metadata = this._entityMetadata[player][ent.id()] = {};
+	{
+		this._entityMetadata[player][ent.id()] = {};
+		metadata = this._entityMetadata[player][ent.id()];
+	}
 	metadata[key] = value;
 
 	this.updateEntityCollections('metadata', ent);
@@ -469,9 +380,9 @@ m.SharedScript.prototype.deleteMetadata = function(player, ent, key)
 m.copyPrototype = function(descendant, parent)
 {
 	let sConstructor = parent.toString();
-	let aMatch = sConstructor.match( /\s*function (.*)\(/ );
+	let aMatch = sConstructor.match(/\s*function (.*)\(/);
 
-	if ( aMatch != null )
+	if (aMatch != null)
 		descendant.prototype[aMatch[1]] = parent;
 
 	for (let p in parent.prototype)
