@@ -7,7 +7,11 @@ m.DefenseManager = function(Config)
 	this.Config = Config;
 	this.targetList = [];
 	this.armyMergeSize = this.Config.Defense.armyMergeSize;
-	this.attackingArmies = {};	// stats on how many enemies are currently attacking our allies
+	// stats on how many enemies are currently attacking our allies
+	// this.attackingArmies[enemy][ally] = number of enemy armies inside allied territory
+	// this.attackingUnits[enemy][ally] = number of enemy units not in armies inside allied territory
+	// this.attackedAllies[ally] = number of enemies attacking the ally
+	this.attackingArmies = {};
 	this.attackingUnits = {};
 	this.attackedAllies = {};
 };
@@ -63,15 +67,19 @@ m.DefenseManager.prototype.update = function(gameState, events)
 	Engine.ProfileStop();
 };
 
-m.DefenseManager.prototype.makeIntoArmy = function(gameState, entityID)
+m.DefenseManager.prototype.makeIntoArmy = function(gameState, entityID, type = "default")
 {
-	// Try to add it to an existing army.
-	for (let army of this.armies)
-		if (!army.isCapturing(gameState) && army.addFoe(gameState, entityID))
-			return;	// over
+	if (type == "default")
+	{
+		// Try to add it to an existing army.
+		for (let army of this.armies)
+			if (army.getType() == type && army.addFoe(gameState, entityID))
+				return;	// over
+	}
 
 	// Create a new army for it.
-	let army = new m.DefenseArmy(gameState, [], [entityID]);
+	let army = new m.DefenseArmy(gameState, [entityID], type);
+
 	this.armies.push(army);
 };
 
@@ -79,7 +87,7 @@ m.DefenseManager.prototype.getArmy = function(partOfArmy)
 {
 	// Find the army corresponding to this ID partOfArmy
 	for (let army of this.armies)
-		if (army.ID === partOfArmy)
+		if (army.ID == partOfArmy)
 			return army;
 
 	return undefined;
@@ -91,20 +99,20 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 		return false;
 
 	let territoryOwner = this.territoryMap.getOwner(entity.position());
-	if (territoryOwner !== 0 && !gameState.isPlayerAlly(territoryOwner))
+	if (territoryOwner != 0 && !gameState.isPlayerAlly(territoryOwner))
 		return false;
 	// check if the entity is trying to build a new base near our buildings,
 	// and if yes, add this base in our target list
 	if (entity.unitAIState() && entity.unitAIState() == "INDIVIDUAL.REPAIR.REPAIRING")
 	{
 		let targetId = entity.unitAIOrderData()[0].target;
-		if (this.targetList.indexOf(targetId) !== -1)
+		if (this.targetList.indexOf(targetId) != -1)
 			return true;
 		let target = gameState.getEntityById(targetId);
 		if (target)
 		{
 			let isTargetEnemy = gameState.isPlayerEnemy(target.owner());
-			if (isTargetEnemy && territoryOwner === PlayerID)
+			if (isTargetEnemy && territoryOwner == PlayerID)
 			{
 				if (target.hasClass("Structure"))
 					this.targetList.push(targetId);
@@ -115,6 +123,8 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 				let myBuildings = gameState.getOwnStructures();
 				for (let building of myBuildings.values())
 				{
+					if (building.foundationProgress() == 0)
+						continue;
 					if (API3.SquareVectorDistance(building.position(), entity.position()) > 30000)
 						continue;
 					this.targetList.push(targetId);
@@ -128,7 +138,7 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 		return false;
 	let dist2Min = 6000;
 	// TODO the 30 is to take roughly into account the structure size in following checks. Can be improved
-	if (entity.attackTypes().indexOf("Ranged") !== -1)
+	if (entity.attackTypes().indexOf("Ranged") != -1)
 		dist2Min = (entity.attackRange("Ranged").max + 30) * (entity.attackRange("Ranged").max + 30);
 
 	for (let targetId of this.targetList)
@@ -143,9 +153,9 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 	let ccEnts = gameState.updatingGlobalCollection("allCCs", API3.Filters.byClass("CivCentre"));
 	for (let cc of ccEnts.values())
 	{
-		if (!gameState.isEntityExclusiveAlly(cc) || cc.foundationProgress() === 0)
+		if (!gameState.isEntityExclusiveAlly(cc) || cc.foundationProgress() == 0)
 			continue;
-		let cooperation = this.GetCooperationLevel(cc);
+		let cooperation = this.GetCooperationLevel(cc.owner());
 		if (cooperation < 0.6 && cc.foundationProgress() !== undefined)
 			continue;
 		if (cooperation < 0.3)
@@ -154,18 +164,33 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 			return true;
 	}
 
-	let myBuildings = gameState.getOwnStructures();
-	for (let building of myBuildings.values())
+	for (let building of gameState.getOwnStructures().values())
 	{
-		if (building.foundationProgress() === 0)
+		if (building.foundationProgress() == 0 ||
+		    API3.SquareVectorDistance(building.position(), entity.position()) > dist2Min)
 			continue;
-		if (API3.SquareVectorDistance(building.position(), entity.position()) < dist2Min)
+		if (!this.territoryMap.isBlinking(building.position()) || gameState.ai.HQ.isDefendable(building))
 			return true;
 	}
 
-	// Update the number of enemies attacking this ally
 	if (gameState.isPlayerMutualAlly(territoryOwner))
 	{
+		// If ally attacked by more than 2 enemies, help him not only for cc but also for structures
+		if (territoryOwner != PlayerID && this.attackedAllies[territoryOwner] &&
+		                                  this.attackedAllies[territoryOwner] > 1 &&
+		                                  this.GetCooperationLevel(territoryOwner) > 0.7)
+		{
+			for (let building of gameState.getAllyStructures(territoryOwner).values())
+			{
+				if (building.foundationProgress() == 0 ||
+				    API3.SquareVectorDistance(building.position(), entity.position()) > dist2Min)
+					continue;
+				if (!this.territoryMap.isBlinking(building.position()))
+					return true;
+			}
+		}
+
+		// Update the number of enemies attacking this ally
 		let enemy = entity.owner();
 		if (this.attackingUnits[enemy] === undefined)
 			this.attackingUnits[enemy] = {};
@@ -181,9 +206,9 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 {
 	const nbPlayers = gameState.sharedScript.playersData.length;
 	let i = gameState.ai.playedTurn % nbPlayers;
- 	this.attackingUnits[i] = undefined;
+	this.attackingUnits[i] = undefined;
 
-	if (i === PlayerID)
+	if (i == PlayerID)
 	{
 		if (!this.armies.length)
 		{
@@ -201,7 +226,7 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 						lost += capture[j];
 				if (lost < Math.ceil(0.25 * capture[i]))
 					continue;
-				this.makeIntoArmy(gameState, ent.id());
+				this.makeIntoArmy(gameState, ent.id(), "capturing");
 				break;
 			}
 		}
@@ -219,7 +244,7 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 		// keep animals attacking us or our allies
 		if (ent.hasClass("Animal"))
 		{
-			if (!ent.unitAIState() || ent.unitAIState().split(".")[1] !== "COMBAT")
+			if (!ent.unitAIState() || ent.unitAIState().split(".")[1] != "COMBAT")
 				continue;
 			let orders = ent.unitAIOrderData();
 			if (!orders || !orders.length || !orders[0].target)
@@ -238,7 +263,7 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 			this.makeIntoArmy(gameState, ent.id());
 	}
 
-	if (i !== 0 || this.armies.length > 1 || gameState.ai.HQ.numActiveBase() === 0)
+	if (i != 0 || this.armies.length > 1 || gameState.ai.HQ.numActiveBases() == 0)
 		return;
 	// look for possible gaia buildings inside our territory (may happen when enemy resign or after structure decay)
 	// and attack it only if useful (and capturable) or dangereous
@@ -249,8 +274,8 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 		if (!ent.capturePoints() && !ent.hasDefensiveFire())
 			continue;
 		let owner = this.territoryMap.getOwner(ent.position());
-		if (owner === PlayerID)
-			this.makeIntoArmy(gameState, ent.id());
+		if (owner == PlayerID)
+			this.makeIntoArmy(gameState, ent.id(), "capturing");
 	}
 };
 
@@ -264,11 +289,12 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 		for (let breaker of breakaways)
 			this.makeIntoArmy(gameState, breaker);		// assume dangerosity
 
-		if (army.getState() === 0)
+		if (army.getState() == 0)
 		{
-			this.switchToAttack(gameState, army);
+			if (army.getType() == "default")
+				this.switchToAttack(gameState, army);
 			army.clear(gameState);
-			this.armies.splice(i--,1);
+			this.armies.splice(i--, 1);
 			continue;
 		}
 	}
@@ -276,24 +302,24 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 	for (let i = 0; i < this.armies.length - 1; ++i)
 	{
 		let army = this.armies[i];
-		if (army.isCapturing(gameState))
+		if (army.getType() != "default")
 			continue;
 		for (let j = i+1; j < this.armies.length; ++j)
 		{
 			let otherArmy = this.armies[j];
-			if (otherArmy.isCapturing(gameState) ||
+			if (otherArmy.getType() != "default" ||
 				API3.SquareVectorDistance(army.foePosition, otherArmy.foePosition) > this.armyMergeSize)
 				continue;
 			// no need to clear here.
 			army.merge(gameState, otherArmy);
-			this.armies.splice(j--,1);
+			this.armies.splice(j--, 1);
 		}
 	}
 
-	if (gameState.ai.playedTurn % 5 !== 0)
+	if (gameState.ai.playedTurn % 5 != 0)
 		return;
 	// Check if any army is no more dangerous (possibly because it has defeated us and destroyed our base)
- 	this.attackingArmies = {};
+	this.attackingArmies = {};
 	for (let i = 0; i < this.armies.length; ++i)
 	{
 		let army = this.armies[i];
@@ -320,10 +346,10 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 			}
 			continue;
 		}
-		else if (owner !== 0)   // enemy army back in its territory
+		else if (owner != 0)   // enemy army back in its territory
 		{
 			army.clear(gameState);
-			this.armies.splice(i--,1);
+			this.armies.splice(i--, 1);
 			continue;
 		}
 
@@ -331,11 +357,11 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 		// TODO check smaller distance with all our buildings instead of only ccs with big distance
 		let stillDangerous = false;
 		let bases = gameState.updatingGlobalCollection("allCCs", API3.Filters.byClass("CivCentre"));
-	 	for (let base of bases.values())
+		for (let base of bases.values())
 		{
 			if (!gameState.isEntityAlly(base))
 				continue;
-			let cooperation = this.GetCooperationLevel(base);
+			let cooperation = this.GetCooperationLevel(base.owner());
 			if (cooperation < 0.3 && !gameState.isEntityOwn(base))
 				continue;
 			if (API3.SquareVectorDistance(base.position(), army.foePosition) > 40000)
@@ -345,18 +371,29 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 			stillDangerous = true;
 			break;
 		}
+		if (stillDangerous)
+			continue;
+		// Need to also check docks because of oversea bases
+		for (let dock of gameState.getOwnStructures().filter(API3.Filters.byClass("Dock")).values())
+		{
+			if (API3.SquareVectorDistance(dock.position(), army.foePosition) > 10000)
+				continue;
+			stillDangerous = true;
+			break;
+		}
 		if (stillDangerous)
 			continue;
 
-		this.switchToAttack(gameState, army);
+		if (army.getType() == "default")
+			this.switchToAttack(gameState, army);
 		army.clear(gameState);
-		this.armies.splice(i--,1);
+		this.armies.splice(i--, 1);
 	}
 };
 
 m.DefenseManager.prototype.assignDefenders = function(gameState)
 {
-	if (this.armies.length === 0)
+	if (!this.armies.length)
 		return;
 
 	let armiesNeeding = [];
@@ -367,12 +404,22 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 		if (needsDef === false)
 			continue;
 
-		// Okay for now needsDef is the total needed strength.
-		// we're dumb so we don't choose if we have a defender shortage.
-		armiesNeeding.push( {"army": army, "need": needsDef} );
+		let armyAccess;
+		for (let entId of army.foeEntities)
+		{
+			let ent = gameState.getEntityById(entId);
+			if (!ent || !ent.position())
+				continue;
+			armyAccess = m.getLandAccess(gameState, ent);
+			break;
+		}
+		if (!armyAccess)
+			API3.warn(" Petra error: attacking army " + army.ID + " without access");
+		army.recalculatePosition(gameState);
+		armiesNeeding.push({ "army": army, "access": armyAccess, "need": needsDef });
 	}
 
-	if (armiesNeeding.length === 0)
+	if (!armiesNeeding.length)
 		return;
 
 	// let's get our potential units
@@ -380,62 +427,77 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 	gameState.getOwnUnits().forEach(function(ent) {
 		if (!ent.position())
 			return;
-		if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
+		if (ent.getMetadata(PlayerID, "plan") == -2 || ent.getMetadata(PlayerID, "plan") == -3)
 			return;
 		if (ent.hasClass("Support") || ent.attackTypes() === undefined)
 			return;
-		if (ent.hasClass("Siege") && !ent.hasClass("Melee"))
+		if (ent.hasClass("Catapult"))
 			return;
 		if (ent.hasClass("FishingBoat") || ent.hasClass("Trader"))
 			return;
-		if (ent.getMetadata(PlayerID, "transport") !== undefined || ent.getMetadata(PlayerID, "transporter") !== undefined)
+		if (ent.getMetadata(PlayerID, "transport") !== undefined ||
+		    ent.getMetadata(PlayerID, "transporter") !== undefined)
 			return;
-		if (gameState.ai.HQ.gameTypeManager.criticalEnts.has(ent.id()))
+		if (gameState.ai.HQ.victoryManager.criticalEnts.has(ent.id()))
 			return;
-		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") !== -1)
+		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") != -1)
 		{
 			let subrole = ent.getMetadata(PlayerID, "subrole");
-			if (subrole && (subrole === "completing" || subrole === "walking" || subrole === "attacking"))
+			if (subrole && (subrole == "completing" || subrole == "walking" || subrole == "attacking"))
 				return;
 		}
 		potentialDefenders.push(ent.id());
 	});
 
-	for (let a = 0; a < armiesNeeding.length; ++a)
-		armiesNeeding[a].army.recalculatePosition(gameState);
-
-	for (let i = 0; i < potentialDefenders.length; ++i)
+	for (let ipass = 0; ipass < 2; ++ipass)
 	{
-		let ent = gameState.getEntityById(potentialDefenders[i]);
-		if (!ent.position())
-			continue;
-		let aMin;
-		let distMin;
-		for (let a = 0; a < armiesNeeding.length; ++a)
+		// First pass only assign defenders with the right access
+		// Second pass assign all defenders
+		// TODO could sort them by distance
+		let backup = 0;
+		for (let i = 0; i < potentialDefenders.length; ++i)
 		{
-			let dist = API3.SquareVectorDistance(ent.position(), armiesNeeding[a].army.foePosition);
-			if (aMin !== undefined && dist > distMin)
+			let ent = gameState.getEntityById(potentialDefenders[i]);
+			if (!ent || !ent.position())
 				continue;
-			aMin = a;
-			distMin = dist;
+			let aMin;
+			let distMin;
+			let access = ipass == 0 ? m.getLandAccess(gameState, ent) : undefined;
+			for (let a = 0; a < armiesNeeding.length; ++a)
+			{
+				if (access && armiesNeeding[a].access != access)
+					continue;
+				let dist = API3.SquareVectorDistance(ent.position(), armiesNeeding[a].army.foePosition);
+				if (aMin !== undefined && dist > distMin)
+					continue;
+				aMin = a;
+				distMin = dist;
+			}
+
+			// If outside our territory (helping an ally or attacking a cc foundation)
+			// or if in another access, keep some troops in backup
+			if (backup < 12 && (aMin == undefined || distMin > 40000 &&
+			        this.territoryMap.getOwner(armiesNeeding[aMin].army.foePosition) != PlayerID))
+			{
+				++backup;
+				potentialDefenders[i] = undefined;
+				continue;
+			}
+			else if (aMin === undefined)
+				continue;
+
+			armiesNeeding[aMin].need -= m.getMaxStrength(ent);
+			armiesNeeding[aMin].army.addOwn(gameState, potentialDefenders[i]);
+			armiesNeeding[aMin].army.assignUnit(gameState, potentialDefenders[i]);
+			potentialDefenders[i] = undefined;
+
+			if (armiesNeeding[aMin].need <= 0)
+				armiesNeeding.splice(aMin, 1);
+			if (!armiesNeeding.length)
+				return;
 		}
-
-		// if outside our territory (helping an ally or attacking a cc foundation), keep some troops in backup
-		if (i < 12 && this.territoryMap.getOwner(armiesNeeding[aMin].army.foePosition) !== PlayerID)
-			continue;
-
-		armiesNeeding[aMin].need -= m.getMaxStrength(ent);
-		armiesNeeding[aMin].army.addOwn(gameState, potentialDefenders[i]);
-		armiesNeeding[aMin].army.assignUnit(gameState, potentialDefenders[i]);
-
-		if (armiesNeeding[aMin].need <= 0)
-			armiesNeeding.splice(aMin, 1);
-		if (!armiesNeeding.length)
-			break;
 	}
 
-	if (!armiesNeeding.length)
-		return;
 	// If shortage of defenders, produce infantry garrisoned in nearest civil centre
 	let armiesPos = [];
 	for (let a = 0; a < armiesNeeding.length; ++a)
@@ -448,7 +510,7 @@ m.DefenseManager.prototype.abortArmy = function(gameState, army)
 	army.clear(gameState);
 	for (let i = 0; i < this.armies.length; ++i)
 	{
-		if (this.armies[i].ID !== army.ID)
+		if (this.armies[i].ID != army.ID)
 			continue;
 		this.armies.splice(i, 1);
 		break;
@@ -459,7 +521,7 @@ m.DefenseManager.prototype.abortArmy = function(gameState, army)
  * If our defense structures are attacked, garrison soldiers inside when possible
  * and if a support unit is attacked and has less than 55% health, garrison it inside the nearest healing structure
  * and if a ranged siege unit (not used for defense) is attacked, garrison it in the nearest fortress
- * If our hero is attacked in regicide game mode, the gameTypeManager will handle it
+ * If our hero is attacked with regicide victory condition, the victoryManager will handle it
  */
 m.DefenseManager.prototype.checkEvents = function(gameState, events)
 {
@@ -477,6 +539,10 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 		}
 	}
 
+	let allAttacked = {};
+	for (let evt of events.Attacked)
+		allAttacked[evt.target] = evt.attacker;
+
 	for (let evt of events.Attacked)
 	{
 		let target = gameState.getEntityById(evt.target);
@@ -489,8 +555,10 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 		{
 			// If enemies are in range of one of our defensive structures, garrison it for arrow multiplier
 			// (enemy non-defensive structure are not considered to stay in sync with garrisonManager)
-			if (attacker.position() && attacker.isGarrisonHolder() && attacker.getArrowMultiplier())
-				this.garrisonUnitsInside(gameState, attacker, {"attacker": target});
+			if (attacker.position() && attacker.isGarrisonHolder() && attacker.getArrowMultiplier() &&
+			    (target.owner() != 0 || !target.hasClass("Unit") ||
+			     target.unitAIState() && target.unitAIState().split(".")[1] == "COMBAT"))
+				this.garrisonUnitsInside(gameState, attacker, { "attacker": target });
 		}
 
 		if (!gameState.isEntityOwn(target))
@@ -500,15 +568,48 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 		if (attacker && gameState.isEntityAlly(attacker))
 			continue;
 
-		if (target.hasClass("Ship"))    // TODO integrate ships later   need to be sure it is accessible
+		if (attacker && attacker.position() && target.hasClass("FishingBoat"))
+		{
+			let unitAIState = target.unitAIState();
+			let unitAIStateOrder = unitAIState ? unitAIState.split(".")[1] : "";
+			if (target.isIdle() || unitAIStateOrder == "GATHER")
+			{
+				let pos = attacker.position();
+				let range = attacker.attackRange("Ranged") ? attacker.attackRange("Ranged").max + 15 : 25;
+				if (range * range > API3.SquareVectorDistance(pos, target.position()))
+					target.moveToRange(pos[0], pos[1], range, range);
+			}
 			continue;
+		}
+
+		if (target.hasClass("Ship"))    // TODO integrate other ships later, need to be sure it is accessible
+			continue;
+
+		// If a building on a blinking tile is attacked, check if it can be defended.
+		// Same thing for a building in an isolated base (not connected to a base with anchor).
+		if (target.hasClass("Structure"))
+		{
+			let base = gameState.ai.HQ.getBaseByID(target.getMetadata(PlayerID, "base"));
+			if (this.territoryMap.isBlinking(target.position()) && !gameState.ai.HQ.isDefendable(target) ||
+			    !base || gameState.ai.HQ.baseManagers.every(b => !b.anchor || b.accessIndex != base.accessIndex))
+			{
+				let capture = target.capturePoints();
+				if (!capture)
+					continue;
+				let captureRatio = capture[PlayerID] / capture.reduce((a, b) => a + b);
+				if (captureRatio > 0.50 && captureRatio < 0.70)
+					target.destroy();
+				continue;
+			}
+		}
+
 
 		// If inside a started attack plan, let the plan deal with this unit
 		let plan = target.getMetadata(PlayerID, "plan");
 		if (plan !== undefined && plan >= 0)
 		{
 			let attack = gameState.ai.HQ.attackManager.getPlan(plan);
-			if (attack && attack.state !== "unexecuted")
+			if (attack && attack.state != "unexecuted")
 				continue;
 		}
 
@@ -518,28 +619,28 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 			!attacker.hasClass("Structure") && !attacker.hasClass("Ship"))
 		{
 			let territoryOwner = this.territoryMap.getOwner(attacker.position());
-			if (territoryOwner === 0 || gameState.isPlayerAlly(territoryOwner))
+			if (territoryOwner == 0 || gameState.isPlayerAlly(territoryOwner))
 				this.makeIntoArmy(gameState, attacker.id());
 		}
 
 		if (target.getMetadata(PlayerID, "PartOfArmy") !== undefined)
 		{
 			let army = this.getArmy(target.getMetadata(PlayerID, "PartOfArmy"));
-			if (army.isCapturing(gameState))
+			if (army.getType() == "capturing")
 			{
 				let abort = false;
 				// if one of the units trying to capture a structure is attacked,
 				// abort the army so that the unit can defend itself
-				if (army.ownEntities.indexOf(target.id()) !== -1)
+				if (army.ownEntities.indexOf(target.id()) != -1)
 					abort = true;
-				else if (army.foeEntities[0] === target.id() && target.owner() === PlayerID)
+				else if (army.foeEntities[0] == target.id() && target.owner() == PlayerID)
 				{
 					// else we may be trying to regain some capture point from one of our structure
 					abort = true;
 					let capture = target.capturePoints();
 					for (let j = 0; j < capture.length; ++j)
 					{
-						if (!gameState.isPlayerEnemy(j) || capture[j] === 0)
+						if (!gameState.isPlayerEnemy(j) || capture[j] == 0)
 							continue;
 						abort = false;
 						break;
@@ -553,15 +654,15 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 
 		// try to garrison any attacked support unit if low healthlevel
 		if (target.hasClass("Support") && target.healthLevel() < this.Config.garrisonHealthLevel.medium &&
-			!target.getMetadata(PlayerID, "transport") && plan !== -2 && plan !== -3)
+			!target.getMetadata(PlayerID, "transport") && plan != -2 && plan != -3)
 		{
 			this.garrisonAttackedUnit(gameState, target);
 			continue;
 		}
 
-		// try to garrison any attacked range siege unit
-		if (target.hasClass("Siege") && !target.hasClass("Melee") &&
-			!target.getMetadata(PlayerID, "transport") && plan !== -2 && plan !== -3)
+		// try to garrison any attacked catapult
+		if (target.hasClass("Catapult") &&
+			!target.getMetadata(PlayerID, "transport") && plan != -2 && plan != -3)
 		{
 			this.garrisonSiegeUnit(gameState, target);
 			continue;
@@ -571,32 +672,88 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 			continue;
 
 		if (target.isGarrisonHolder() && target.getArrowMultiplier())
-			this.garrisonUnitsInside(gameState, target, {"attacker": attacker});
+			this.garrisonUnitsInside(gameState, target, { "attacker": attacker });
+
+		if (target.hasClass("Unit") && attacker.hasClass("Unit"))
+		{
+			// Consider if we should retaliate or continue our task
+			if (target.hasClass("Support") || target.attackTypes() === undefined)
+				continue;
+			let orderData = target.unitAIOrderData();
+			let currentTarget = orderData && orderData.length && orderData[0].target ?
+				gameState.getEntityById(orderData[0].target) : undefined;
+			if (currentTarget)
+			{
+				let unitAIState = target.unitAIState();
+				let unitAIStateOrder = unitAIState ? unitAIState.split(".")[1] : "";
+				if (unitAIStateOrder == "COMBAT" && (currentTarget == attacker.id() ||
+					!currentTarget.hasClass("Structure") && !currentTarget.hasClass("Support")))
+					continue;
+				if (unitAIStateOrder == "REPAIR" && currentTarget.hasDefensiveFire())
+					continue;
+				if (unitAIStateOrder == "COMBAT" && !m.isSiegeUnit(currentTarget) &&
+				    gameState.ai.HQ.capturableTargets.has(orderData[0].target))
+				{
+					// take the nearest unit also attacking this structure to help us
+					let capturableTarget = gameState.ai.HQ.capturableTargets.get(orderData[0].target);
+					let minDist;
+					let minEnt;
+					let pos = attacker.position();
+					capturableTarget.ents.delete(target.id());
+					for (let entId of capturableTarget.ents)
+					{
+						if (allAttacked[entId])
+							continue;
+						let ent = gameState.getEntityById(entId);
+						if (!ent || !ent.position())
+							continue;
+						// Check that the unit is still attacking the structure (since the last played turn)
+						let state = ent.unitAIState();
+						if (!state || !state.split(".")[1] || state.split(".")[1] != "COMBAT")
+							continue;
+						let entOrderData = ent.unitAIOrderData();
+						if (!entOrderData || !entOrderData.length || !entOrderData[0].target ||
+						     entOrderData[0].target != orderData[0].target)
+							continue;
+						let dist = API3.SquareVectorDistance(pos, ent.position());
+						if (minEnt && dist > minDist)
+							continue;
+						minDist = dist;
+						minEnt = ent;
+					}
+					if (minEnt)
+					{
+						capturableTarget.ents.delete(minEnt.id());
+						minEnt.attack(attacker.id(), m.allowCapture(gameState, minEnt, attacker));
+					}
+				}
+			}
+			target.attack(attacker.id(), m.allowCapture(gameState, target, attacker));
+		}
 	}
 };
 
 m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, data)
 {
-	let minGarrison = data.min ? data.min : target.garrisonMax();
-	let typeGarrison = data.type ? data.type : "protection";
-	if (gameState.ai.HQ.garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
-		return false;
 	if (target.hitpoints() < target.garrisonEjectHealth() * target.maxHitpoints())
 		return false;
+	let minGarrison = data.min || target.garrisonMax();
+	if (gameState.ai.HQ.garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
+		return false;
 	if (data.attacker)
 	{
 		let attackTypes = target.attackTypes();
-		if (!attackTypes || attackTypes.indexOf("Ranged") === -1)
+		if (!attackTypes || attackTypes.indexOf("Ranged") == -1)
 			return false;
 		let dist = API3.SquareVectorDistance(data.attacker.position(), target.position());
 		let range = target.attackRange("Ranged").max;
 		if (dist >= range*range)
 			return false;
 	}
-	let index = gameState.ai.accessibility.getAccessValue(target.position());
+	let access = m.getLandAccess(gameState, target);
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	let garrisonArrowClasses = target.getGarrisonArrowClasses();
-	let units = gameState.getOwnUnits().filter(ent => MatchesClassList(ent.classes(), garrisonArrowClasses)).filterNearest(target.position());
+	let typeGarrison = data.type || "protection";
 	let allowMelee = gameState.ai.HQ.garrisonManager.allowMelee(target);
 	if (allowMelee === undefined)
 	{
@@ -606,35 +763,42 @@ m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, dat
 		else
 			allowMelee = true;
 	}
+	let units = gameState.getOwnUnits().filter(ent => {
+		if (!ent.position())
+			return false;
+		if (!MatchesClassList(ent.classes(), garrisonArrowClasses))
+			return false;
+		if (typeGarrison != "decay" && !allowMelee && ent.attackTypes().indexOf("Melee") != -1)
+			return false;
+		if (ent.getMetadata(PlayerID, "transport") !== undefined)
+			return false;
+		let army = ent.getMetadata(PlayerID, "PartOfArmy") ? this.getArmy(ent.getMetadata(PlayerID, "PartOfArmy")) : undefined;
+		if (!army && (ent.getMetadata(PlayerID, "plan") == -2 || ent.getMetadata(PlayerID, "plan") == -3))
+			return false;
+		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") >= 0)
+		{
+			let subrole = ent.getMetadata(PlayerID, "subrole");
+			// when structure decaying (usually because we've just captured it in enemy territory), also allow units from an attack plan
+			if (typeGarrison != "decay" && subrole && (subrole == "completing" || subrole == "walking" || subrole == "attacking"))
+				return false;
+		}
+		if (m.getLandAccess(gameState, ent) != access)
+			return false;
+		return true;
+	}).filterNearest(target.position());
+
 	let ret = false;
 	for (let ent of units.values())
 	{
 		if (garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
 			break;
-		if (!ent.position())
-			continue;
-		if (typeGarrison !== "decay" && !allowMelee && ent.attackTypes().indexOf("Melee") !== -1)
-			continue;
-		if (ent.getMetadata(PlayerID, "transport") !== undefined)
-			continue;
-		let army = ent.getMetadata(PlayerID, "PartOfArmy") ? this.getArmy(ent.getMetadata(PlayerID, "PartOfArmy")) : undefined;
-		if (!army && (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3))
-			continue;
-		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") >= 0)
-		{
-			let subrole = ent.getMetadata(PlayerID, "subrole");
-			// when structure decaying (usually because we've just captured it in enemy territory), also allow units from an attack plan
-			if (typeGarrison !== "decay" && subrole && (subrole === "completing" || subrole === "walking" || subrole === "attacking"))
-				continue;
-		}
-		if (gameState.ai.accessibility.getAccessValue(ent.position()) !== index)
-			continue;
 		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") >= 0)
 		{
 			let attackPlan = gameState.ai.HQ.attackManager.getPlan(ent.getMetadata(PlayerID, "plan"));
 			if (attackPlan)
 				attackPlan.removeUnit(ent, true);
 		}
+		let army = ent.getMetadata(PlayerID, "PartOfArmy") ? this.getArmy(ent.getMetadata(PlayerID, "PartOfArmy")) : undefined;
 		if (army)
 			army.removeOwn(gameState, ent.id());
 		garrisonManager.garrison(gameState, ent, target, typeGarrison);
@@ -648,23 +812,26 @@ m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 {
 	let distmin = Math.min();
 	let nearest;
-	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
+	let unitAccess = m.getLandAccess(gameState, unit);
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
-	gameState.getAllyStructures().forEach(function(ent) {
+	for (let ent of gameState.getAllyStructures().values())
+	{
+		if (!ent.isGarrisonHolder())
+			continue;
 		if (!MatchesClassList(unit.classes(), ent.garrisonableClasses()))
-			return;
+			continue;
 		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax())
-			return;
+			continue;
 		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
-			return;
-		if (m.getLandAccess(gameState, ent) !== unitAccess)
-			return;
+			continue;
+		if (m.getLandAccess(gameState, ent) != unitAccess)
+			continue;
 		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
-			return;
+			continue;
 		distmin = dist;
 		nearest = ent;
-	});
+	}
 	if (nearest)
 		garrisonManager.garrison(gameState, unit, nearest, "protection");
 	return nearest !== undefined;
@@ -679,10 +846,12 @@ m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emer
 {
 	let distmin = Math.min();
 	let nearest;
-	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
+	let unitAccess = m.getLandAccess(gameState, unit);
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	for (let ent of gameState.getAllyStructures().values())
 	{
+		if (!ent.isGarrisonHolder())
+			continue;
 		if (!emergency && !ent.buffHeal())
 			continue;
 		if (!MatchesClassList(unit.classes(), ent.garrisonableClasses()))
@@ -692,7 +861,7 @@ m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emer
 			continue;
 		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
 			continue;
-		if (m.getLandAccess(gameState, ent) !== unitAccess)
+		if (m.getLandAccess(gameState, ent) != unitAccess)
 			continue;
 		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
@@ -718,9 +887,8 @@ m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emer
 /**
  * Be more inclined to help an ally attacked by several enemies
  */
-m.DefenseManager.prototype.GetCooperationLevel = function(ent)
+m.DefenseManager.prototype.GetCooperationLevel = function(ally)
 {
-	let ally = ent.owner();
 	let cooperation = this.Config.personality.cooperative;
 	if (this.attackedAllies[ally] && this.attackedAllies[ally] > 1)
 		cooperation += 0.2 * (this.attackedAllies[ally] - 1);
@@ -744,12 +912,9 @@ m.DefenseManager.prototype.switchToAttack = function(gameState, army)
 		for (let entId of army.ownEntities)
 		{
 			let ent = gameState.getEntityById(entId);
-			if (!ent)
+			if (!ent || !ent.position() || m.getLandAccess(gameState, ent) != targetAccess)
 				continue;
-			let pos = ent.position();
-			if (!pos || gameState.ai.accessibility.getAccessValue(pos) !== targetAccess)
-				continue;
-			if (API3.SquareVectorDistance(targetPos, pos) > 14400)
+			if (API3.SquareVectorDistance(targetPos, ent.position()) > 14400)
 				continue;
 			gameState.ai.HQ.attackManager.switchDefenseToAttack(gameState, target, { "armyID": army.ID, "uniqueTarget": true });
 			return;
@@ -760,7 +925,7 @@ m.DefenseManager.prototype.switchToAttack = function(gameState, army)
 m.DefenseManager.prototype.Serialize = function()
 {
 	let properties = {
-		"targetList" : this.targetList,
+		"targetList": this.targetList,
 		"armyMergeSize": this.armyMergeSize,
 		"attackingUnits": this.attackingUnits,
 		"attackingArmies": this.attackingArmies,
@@ -782,7 +947,7 @@ m.DefenseManager.prototype.Deserialize = function(gameState, data)
 	this.armies = [];
 	for (let dataArmy of data.armies)
 	{
-		let army = new m.DefenseArmy(gameState, [], []);
+		let army = new m.DefenseArmy(gameState, []);
 		army.Deserialize(dataArmy);
 		this.armies.push(army);
 	}
