@@ -26,7 +26,7 @@ var PETRA = function(m)
 m.DiplomacyManager = function(Config)
 {
 	this.Config = Config;
-	this.nextTributeUpdate = -1;
+	this.nextTributeUpdate = 90;
 	this.nextTributeRequest = new Map();
 	this.nextTributeRequest.set("all", 240);
 	this.betrayLapseTime = -1;
@@ -122,7 +122,7 @@ m.DiplomacyManager.prototype.tributes = function(gameState)
 	}
 };
 
-m.DiplomacyManager.prototype.checkEvents = function (gameState, events)
+m.DiplomacyManager.prototype.checkEvents = function(gameState, events)
 {
 	// Increase slowly the cooperative personality trait either when we receive tribute from our allies
 	// or if our allies attack enemies inside our territory
@@ -193,7 +193,7 @@ m.DiplomacyManager.prototype.checkEvents = function (gameState, events)
 			    gameState.sharedScript.playersData[evt.player].isNeutral[PlayerID] && sentRequest.requestType === "neutral")
 				this.changePlayerDiplomacy(gameState, evt.player, sentRequest.requestType);
 
-			// Just remove the request if the other player switched their stance to a different and/or more negative state 
+			// Just remove the request if the other player switched their stance to a different and/or more negative state
 			// TODO: Keep this send request and take it into account for later diplomacy changes (maybe be less inclined to offer to this player)
 			this.sentDiplomacyRequests.delete(evt.player);
 			continue;
@@ -315,7 +315,7 @@ m.DiplomacyManager.prototype.lastManStandingCheck = function(gameState)
 		if (gameState.isPlayerNeutral(i)) // be more inclined to turn against neutral players
 			turnFactor += this.betrayWeighting;
 
-		if (gameState.getGameType() === "wonder")
+		if (gameState.getVictoryConditions().has("wonder"))
 		{
 			let wonder = gameState.getEnemyStructures(i).filter(API3.Filters.byClass("Wonder"))[0];
 			if (wonder)
@@ -330,7 +330,7 @@ m.DiplomacyManager.prototype.lastManStandingCheck = function(gameState)
 			}
 		}
 
-		if (gameState.getGameType() === "capture_the_relic")
+		if (gameState.getVictoryConditions().has("capture_the_relic"))
 		{
 			let relicsCount = gameState.updatingGlobalCollection("allRelics", API3.Filters.byClass("Relic"))
 				.filter(relic => relic.owner() === i).length;
