@@ -4,7 +4,7 @@ var PETRA = function(m)
 /** returns true if this unit should be considered as a siege unit */
 m.isSiegeUnit = function(ent)
 {
-	return ent.hasClass("Siege") || (ent.hasClass("Elephant") && ent.hasClass("Champion"));
+	return ent.hasClass("Siege") || ent.hasClass("Elephant") && ent.hasClass("Melee") && ent.hasClass("Champion");
 };
 
 /** returns some sort of DPS * health factor. If you specify a class, it'll use the modifiers against that class too. */
@@ -28,13 +28,13 @@ m.getMaxStrength = function(ent, againstClass)
 				val *= ent.getMultiplierAgainst(type, againstClass);
 			switch (str)
 			{
-			case "crush":
+			case "Crush":
 				strength += val * 0.085 / 3;
 				break;
-			case "hack":
+			case "Hack":
 				strength += val * 0.075 / 3;
 				break;
-			case "pierce":
+			case "Pierce":
 				strength += val * 0.065 / 3;
 				break;
 			default:
@@ -70,13 +70,13 @@ m.getMaxStrength = function(ent, againstClass)
 		let val = parseFloat(armourStrength[str]);
 		switch (str)
 		{
-		case "crush":
+		case "Crush":
 			strength += val * 0.085 / 3;
 			break;
-		case "hack":
+		case "Hack":
 			strength += val * 0.075 / 3;
 			break;
-		case "pierce":
+		case "Pierce":
 			strength += val * 0.065 / 3;
 			break;
 		default:
@@ -87,49 +87,80 @@ m.getMaxStrength = function(ent, againstClass)
 	return strength * ent.maxHitpoints() / 100.0;
 };
 
-/** Get access and cache it in metadata if not already done */
+/** Get access and cache it (except for units as it can change) in metadata if not already done */
 m.getLandAccess = function(gameState, ent)
 {
+	if (ent.hasClass("Unit"))
+		return gameState.ai.accessibility.getAccessValue(ent.position());
+
 	let access = ent.getMetadata(PlayerID, "access");
 	if (!access)
 	{
 		access = gameState.ai.accessibility.getAccessValue(ent.position());
+		// Docks are sometimes not as expected
+		if (access < 2 && ent.buildPlacementType() == "shore")
+		{
+			let halfDepth = 0;
+			if (ent.get("Footprint/Square"))
+				halfDepth = +ent.get("Footprint/Square/@depth") / 2;
+			else if (ent.get("Footprint/Circle"))
+				halfDepth = +ent.get("Footprint/Circle/@radius");
+			let entPos = ent.position();
+			let cosa = Math.cos(ent.angle());
+			let sina = Math.sin(ent.angle());
+			for (let d = 3; d < halfDepth; d += 3)
+			{
+				let pos = [ entPos[0] - d * sina,
+				            entPos[1] - d * cosa];
+				access = gameState.ai.accessibility.getAccessValue(pos);
+				if (access > 1)
+					break;
+			}
+		}
 		ent.setMetadata(PlayerID, "access", access);
 	}
 	return access;
 };
 
+/** Sea access always cached as it never changes */
 m.getSeaAccess = function(gameState, ent)
 {
 	let sea = ent.getMetadata(PlayerID, "sea");
 	if (!sea)
 	{
 		sea = gameState.ai.accessibility.getAccessValue(ent.position(), true);
-		if (sea < 2)	// pre-positioned docks are sometimes not well positionned
+		// Docks are sometimes not as expected
+		if (sea < 2 && ent.buildPlacementType() == "shore")
 		{
 			let entPos = ent.position();
-			let radius = ent.footprintRadius();
-			for (let i = 0; i < 16; ++i)
+			let cosa = Math.cos(ent.angle());
+			let sina = Math.sin(ent.angle());
+			for (let d = 3; d < 15; d += 3)
 			{
-				let pos = [ entPos[0] + radius*Math.cos(i*Math.PI/8),
-				            entPos[1] + radius*Math.sin(i*Math.PI/8) ];
+				let pos = [ entPos[0] + d * sina,
+				            entPos[1] + d * cosa];
 				sea = gameState.ai.accessibility.getAccessValue(pos, true);
-				if (sea >= 2)
+				if (sea > 1)
 					break;
 			}
 		}
-		if (sea < 2)
-			API3.warn("ERROR in Petra getSeaAccess because of position with sea index " + sea);
 		ent.setMetadata(PlayerID, "sea", sea);
 	}
 	return sea;
 };
 
+m.setSeaAccess = function(gameState, ent)
+{
+	m.getSeaAccess(gameState, ent);
+};
+
 /** Decide if we should try to capture (returns true) or destroy (return false) */
 m.allowCapture = function(gameState, ent, target)
 {
 	if (!target.isCapturable() || !ent.canCapture(target))
 		return false;
+	if (target.isInvulnerable())
+		return true;
 	// always try to recapture cp from an allied, except if it's decaying
 	if (gameState.isPlayerAlly(target.owner()))
 		return !target.decaying();
@@ -157,14 +188,13 @@ m.allowCapture = function(gameState, ent, target)
 		}
 		capture = capturable.strength;
 	}
-	capture *= 1 / ( 0.1 + 0.9*target.healthLevel());
+	capture *= 1 / (0.1 + 0.9*target.healthLevel());
 	let sumCapturePoints = target.capturePoints().reduce((a, b) => a + b);
 	if (target.hasDefensiveFire() && target.isGarrisonHolder() && target.garrisoned())
 		return capture > antiCapture + sumCapturePoints/50;
 	return capture > antiCapture + sumCapturePoints/80;
 };
 
-/** copy of GetAttackBonus from Attack.js */
 m.getAttackBonus = function(ent, target, type)
 {
 	let attackBonus = 1;
@@ -193,8 +223,9 @@ m.returnResources = function(gameState, ent)
 
 	let closestDropsite;
 	let distmin = Math.min();
-	let access = gameState.ai.accessibility.getAccessValue(ent.position());
-	let dropsiteCollection = gameState.playerData.hasSharedDropsites ? gameState.getAnyDropsites(resource) : gameState.getOwnDropsites(resource);
+	let access = m.getLandAccess(gameState, ent);
+	let dropsiteCollection = gameState.playerData.hasSharedDropsites ?
+	                         gameState.getAnyDropsites(resource) : gameState.getOwnDropsites(resource);
 	for (let dropsite of dropsiteCollection.values())
 	{
 		if (!dropsite.position())
@@ -203,13 +234,7 @@ m.returnResources = function(gameState, ent)
 		// owner !== PlayerID can only happen when hasSharedDropsites === true, so no need to test it again
 		if (owner !== PlayerID && (!dropsite.isSharedDropsite() || !gameState.isPlayerMutualAlly(owner)))
 			continue;
-		let dropsiteAccess = dropsite.getMetadata(PlayerID, "access");
-		if (!dropsiteAccess)
-		{
-			dropsiteAccess = gameState.ai.accessibility.getAccessValue(dropsite.position());
-			dropsite.setMetadata(PlayerID, "access", dropsiteAccess);
-		}
-		if (dropsiteAccess !== access)
+		if (m.getLandAccess(gameState, dropsite) != access)
 			continue;
 		let dist = API3.SquareVectorDistance(ent.position(), dropsite.position());
 		if (dist > distmin)
@@ -227,23 +252,20 @@ m.returnResources = function(gameState, ent)
 /** is supply full taking into account gatherers affected during this turn */
 m.IsSupplyFull = function(gameState, ent)
 {
-	if (ent.isFull() === true)
-	    return true;
-	let turnCache = gameState.ai.HQ.turnCache;
-	let count = ent.resourceSupplyNumGatherers();
-	if (turnCache.resourceGatherer && turnCache.resourceGatherer[ent.id()])
-		count += turnCache.resourceGatherer[ent.id()];
-	if (count >= ent.maxGatherers())
-		return true;
-	return false;
+	return ent.isFull() === true ||
+		ent.resourceSupplyNumGatherers() + gameState.ai.HQ.GetTCGatherer(ent.id()) >= ent.maxGatherers();
 };
 
 /**
- * get the best base (in terms of distance and accessIndex) for an entity
+ * Get the best base (in terms of distance and accessIndex) for an entity.
+ * It should be on the same accessIndex for structures.
+ * If nothing found, return the base[0] for units and undefined for structures.
+ * If exclude is given, we exclude the base with ID = exclude.
  */
-m.getBestBase = function(gameState, ent, onlyConstructedBase = false)
+m.getBestBase = function(gameState, ent, onlyConstructedBase = false, exclude = false)
 {
 	let pos = ent.position();
+	let accessIndex;
 	if (!pos)
 	{
 		let holder = m.getHolder(gameState, ent);
@@ -254,23 +276,48 @@ m.getBestBase = function(gameState, ent, onlyConstructedBase = false)
 			return gameState.ai.HQ.baseManagers[0];
 		}
 		pos = holder.position();
+		accessIndex = m.getLandAccess(gameState, holder);
 	}
+	else
+		accessIndex = m.getLandAccess(gameState, ent);
+
 	let distmin = Math.min();
+	let dist;
 	let bestbase;
-	let accessIndex = gameState.ai.accessibility.getAccessValue(pos);
 	for (let base of gameState.ai.HQ.baseManagers)
 	{
-		if (!base.anchor || onlyConstructedBase && base.anchor.foundationProgress() !== undefined)
+		if (base.ID == gameState.ai.HQ.baseManagers[0].ID || exclude && base.ID == exclude)
 			continue;
-		let dist = API3.SquareVectorDistance(base.anchor.position(), pos);
-		if (base.accessIndex !== accessIndex)
-			dist += 100000000;
+		if (onlyConstructedBase && (!base.anchor || base.anchor.foundationProgress() !== undefined))
+			continue;
+		if (ent.hasClass("Structure") && base.accessIndex != accessIndex)
+			continue;
+		if (base.anchor && base.anchor.position())
+			dist = API3.SquareVectorDistance(base.anchor.position(), pos);
+		else
+		{
+			let found = false;
+			for (let structure of base.buildings.values())
+			{
+				if (!structure.position())
+					continue;
+				dist = API3.SquareVectorDistance(structure.position(), pos);
+				found = true;
+				break;
+			}
+			if (!found)
+				continue;
+		}
+		if (base.accessIndex != accessIndex)
+			dist += 50000000;
+		if (!base.anchor)
+			dist += 50000000;
 		if (dist > distmin)
 			continue;
 		distmin = dist;
 		bestbase = base;
 	}
-	if (!bestbase)
+	if (!bestbase && !ent.hasClass("Structure"))
 		bestbase = gameState.ai.HQ.baseManagers[0];
 	return bestbase;
 };
@@ -285,6 +332,15 @@ m.getHolder = function(gameState, ent)
 	return undefined;
 };
 
+/** return the template of the built foundation if a foundation, otherwise return the entity itself */
+m.getBuiltEntity = function(gameState, ent)
+{
+	if (ent.foundationProgress() !== undefined)
+		return gameState.getBuiltTemplate(ent.templateName());
+
+	return ent;
+};
+
 /**
  * return true if it is not worth finishing this building (it would surely decay)
  * TODO implement the other conditions
@@ -294,7 +350,7 @@ m.isNotWorthBuilding = function(gameState, ent)
 	if (gameState.ai.HQ.territoryMap.getOwner(ent.position()) !== PlayerID)
 	{
 		let buildTerritories = ent.buildTerritories();
-		if (buildTerritories && (!buildTerritories.length || (buildTerritories.length === 1 && buildTerritories[0] === "own")))
+		if (buildTerritories && (!buildTerritories.length || buildTerritories.length === 1 && buildTerritories[0] === "own"))
 			return true;
 	}
 	return false;
@@ -318,6 +374,50 @@ m.isLineInsideEnemyTerritory = function(gameState, pos1, pos2, step=70)
 	return false;
 };
 
+m.gatherTreasure = function(gameState, ent, water = false)
+{
+	if (!gameState.ai.HQ.treasures.hasEntities())
+		return false;
+	if (!ent || !ent.position())
+		return false;
+	let rates = ent.resourceGatherRates();
+	if (!rates || !rates.treasure || rates.treasure <= 0)
+		return false;
+	let treasureFound;
+	let distmin = Math.min();
+	let access = water ? m.getSeaAccess(gameState, ent) : m.getLandAccess(gameState, ent);
+	for (let treasure of gameState.ai.HQ.treasures.values())
+	{
+		if (m.IsSupplyFull(gameState, treasure))
+			continue;
+		// let some time for the previous gatherer to reach the treasure before trying again
+		let lastGathered = treasure.getMetadata(PlayerID, "lastGathered");
+		if (lastGathered && gameState.ai.elapsedTime - lastGathered < 20)
+			continue;
+		if (!water && access != m.getLandAccess(gameState, treasure))
+			continue;
+		if (water && access != m.getSeaAccess(gameState, treasure))
+			continue;
+		let territoryOwner = gameState.ai.HQ.territoryMap.getOwner(treasure.position());
+		if (territoryOwner != 0 && !gameState.isPlayerAlly(territoryOwner))
+			continue;
+		let dist = API3.SquareVectorDistance(ent.position(), treasure.position());
+		if (dist > 120000 || territoryOwner != PlayerID && dist > 14000) // AI has no LOS, so restrict it a bit
+			continue;
+		if (dist > distmin)
+			continue;
+		distmin = dist;
+		treasureFound = treasure;
+	}
+	if (!treasureFound)
+		return false;
+	treasureFound.setMetadata(PlayerID, "lastGathered", gameState.ai.elapsedTime);
+	ent.gather(treasureFound);
+	gameState.ai.HQ.AddTCGatherer(treasureFound.id());
+	ent.setMetadata(PlayerID, "supply", treasureFound.id());
+	return true;
+};
+
 m.dumpEntity = function(ent)
 {
 	if (!ent)
@@ -328,9 +428,12 @@ m.dumpEntity = function(ent)
 		  " subrole " + ent.getMetadata(PlayerID, "subrole"));
 	API3.warn("owner " + ent.owner() + " health " + ent.hitpoints() + " healthMax " + ent.maxHitpoints() +
 	          " foundationProgress " + ent.foundationProgress());
-	API3.warn(" garrisoning " + ent.getMetadata(PlayerID, "garrisoning") + " garrisonHolder " + ent.getMetadata(PlayerID, "garrisonHolder") +
-		  " plan " + ent.getMetadata(PlayerID, "plan")	+ " transport " + ent.getMetadata(PlayerID, "transport") +
-		  " gather-type " + ent.getMetadata(PlayerID, "gather-type") + " target-foundation " + ent.getMetadata(PlayerID, "target-foundation") +
+	API3.warn(" garrisoning " + ent.getMetadata(PlayerID, "garrisoning") +
+		  " garrisonHolder " + ent.getMetadata(PlayerID, "garrisonHolder") +
+		  " plan " + ent.getMetadata(PlayerID, "plan")	+ " transport " + ent.getMetadata(PlayerID, "transport"));
+	API3.warn(" stance " + ent.getStance() + " transporter " + ent.getMetadata(PlayerID, "transporter") +
+		  " gather-type " + ent.getMetadata(PlayerID, "gather-type") +
+		  " target-foundation " + ent.getMetadata(PlayerID, "target-foundation") +
 		  " PartOfArmy " + ent.getMetadata(PlayerID, "PartOfArmy"));
 };
 
