@@ -40,7 +40,9 @@ m.Queue.prototype.check= function(gameState)
 	{
 		if (!this.plans[0].isInvalid(gameState))
 			return;
-		this.plans.shift();
+		let plan = this.plans.shift();
+		if (plan.queueToReset)
+			gameState.ai.queueManager.changePriority(plan.queueToReset, gameState.ai.Config.priorities[plan.queueToReset]);
 	}
 };
 
@@ -145,12 +147,12 @@ m.Queue.prototype.Deserialize = function(gameState, data)
 	for (let dataPlan of data.plans)
 	{
 		let plan;
-		if (dataPlan.prop.category == "unit")
-			plan = new m.TrainingPlan(gameState, dataPlan.prop.type);
-		else if (dataPlan.prop.category == "building")
-			plan = new m.ConstructionPlan(gameState, dataPlan.prop.type);
-		else if (dataPlan.prop.category == "technology")
-			plan = new m.ResearchPlan(gameState, dataPlan.prop.type);
+		if (dataPlan.category == "unit")
+			plan = new m.TrainingPlan(gameState, dataPlan.type);
+		else if (dataPlan.category == "building")
+			plan = new m.ConstructionPlan(gameState, dataPlan.type);
+		else if (dataPlan.category == "technology")
+			plan = new m.ResearchPlan(gameState, dataPlan.type);
 		else
 		{
 			API3.warn("Petra deserialization error: plan unknown " + uneval(dataPlan));
