@@ -46,23 +46,14 @@ function sortNameIgnoreCase(x, y)
 
 /**
  * Escape tag start and escape characters, so users cannot use special formatting.
- * Also limit string length to 256 characters (not counting escape characters).
  */
-function escapeText(text, limitLength = true)
+function escapeText(text)
 {
-	if (!text)
-		return text;
-
-	if (limitLength)
-		text = text.substr(0, 255);
-
 	return text.replace(/\\/g, "\\\\").replace(/\[/g, "\\[");
 }
 
 function unescapeText(text)
 {
-	if (!text)
-		return text;
 	return text.replace(/\\\\/g, "\\").replace(/\\\[/g, "\[");
 }
 
@@ -82,12 +73,18 @@ function playerDataToStringifiedTeamList(playerData)
 		delete teamList[team].Team;
 	}
 
-	return escapeText(JSON.stringify(teamList), false);
+	return escapeText(JSON.stringify(teamList));
 }
 
 function stringifiedTeamListToPlayerData(stringifiedTeamList)
 {
-	let teamList = JSON.parse(unescapeText(stringifiedTeamList));
+	let teamList = {};
+	try
+	{
+		teamList = JSON.parse(unescapeText(stringifiedTeamList));
+	}
+	catch (e) {}
+
 	let playerData = [];
 
 	for (let team in teamList)
@@ -171,12 +168,13 @@ function clearChatMessages()
 	g_ChatMessages.length = 0;
 	Engine.GetGUIObjectByName("chatText").caption = "";
 
-	try {
+	try
+	{
 		for (let timer of g_ChatTimers)
 			clearTimeout(timer);
 		g_ChatTimers.length = 0;
-	} catch (e) {
 	}
+	catch (e) {}
 }
 
 /**
