@@ -7,27 +7,11 @@ Armour.prototype.Schema =
 		"<Pierce>0.0</Pierce>" +
 		"<Crush>5.0</Crush>" +
 	"</a:example>" +
-	"<element name='Hack' a:help='Hack damage protection'>" +
-		"<ref name='nonNegativeDecimal'/>" +
-	"</element>" +
-	"<element name='Pierce' a:help='Pierce damage protection'>" +
-		"<ref name='nonNegativeDecimal'/>" +
-	"</element>" +
-	"<element name='Crush' a:help='Crush damage protection'>" +
-		"<ref name='nonNegativeDecimal'/>" +
-	"</element>" +
+	DamageTypes.BuildSchema("damage protection") +
 	"<optional>" +
 		"<element name='Foundation' a:help='Armour given to building foundations'>" +
 			"<interleave>" +
-				"<element name='Hack' a:help='Hack damage protection'>" +
-					"<ref name='nonNegativeDecimal'/>" +
-				"</element>" +
-				"<element name='Pierce' a:help='Pierce damage protection'>" +
-					"<ref name='nonNegativeDecimal'/>" +
-				"</element>" +
-				"<element name='Crush' a:help='Crush damage protection'>" +
-					"<ref name='nonNegativeDecimal'/>" +
-				"</element>" +
+				DamageTypes.BuildSchema("damage protection") +
 			"</interleave>" +
 		"</element>" +
 	"</optional>";
@@ -37,29 +21,36 @@ Armour.prototype.Init = function()
 	this.invulnerable = false;
 };
 
+Armour.prototype.IsInvulnerable = function()
+{
+	return this.invulnerable;
+};
+
 Armour.prototype.SetInvulnerability = function(invulnerability)
 {
 	this.invulnerable = invulnerability;
+	Engine.PostMessage(this.entity, MT_InvulnerabilityChanged, { "entity": this.entity, "invulnerability": invulnerability });
 };
 
 /**
  * Take damage according to the entity's armor.
- * Returns object of the form { "killed": false, "change": -12 }
+ * @param {Object} strengths - { "hack": number, "pierce": number, "crush": number } or something like that.
+ * @param {number} multiplier - the damage multiplier.
+ * Returns object of the form { "killed": false, "change": -12 }.
  */
-Armour.prototype.TakeDamage = function(hack, pierce, crush)
+Armour.prototype.TakeDamage = function(strengths, multiplier = 1)
 {
 	if (this.invulnerable)
 		return { "killed": false, "change": 0 };
 
 	// Adjust damage values based on armour; exponential armour: damage = attack * 0.9^armour
 	var armourStrengths = this.GetArmourStrengths();
-	var adjHack = hack * Math.pow(0.9, armourStrengths.hack);
-	var adjPierce = pierce * Math.pow(0.9, armourStrengths.pierce);
-	var adjCrush = crush * Math.pow(0.9, armourStrengths.crush);
 
 	// Total is sum of individual damages
 	// Don't bother rounding, since HP is no longer integral.
-	var total = adjHack + adjPierce + adjCrush;
+	var total = 0;
+	for (let type in strengths)
+		total += strengths[type] * multiplier * Math.pow(0.9, armourStrengths[type] || 0);
 
 	// Reduce health
 	var cmpHealth = Engine.QueryInterface(this.entity, IID_Health);
@@ -84,11 +75,11 @@ Armour.prototype.GetArmourStrengths = function()
 
 	var foundation = Engine.QueryInterface(this.entity, IID_Foundation) && this.template.Foundation;
 
-	return {
-		"hack": applyMods("Hack", foundation),
-		"pierce": applyMods("Pierce", foundation),
-		"crush": applyMods("Crush", foundation)
-	};
+	let ret = {};
+	for (let damageType of DamageTypes.GetTypes())
+		ret[damageType] = applyMods(damageType, foundation);
+
+	return ret;
 };
 
 Engine.RegisterComponentType(IID_DamageReceiver, "Armour", Armour);
