@@ -74,12 +74,18 @@ Attack.prototype.Schema =
 			"<RepeatTime>1600</RepeatTime>" +
 			"<ProjectileSpeed>50.0</ProjectileSpeed>" +
 			"<Spread>2.5</Spread>" +
+			"<Delay>1000</Delay>" +
 			"<Bonuses>" +
 				"<Bonus1>" +
 					"<Classes>Cavalry</Classes>" +
 					"<Multiplier>2</Multiplier>" +
 				"</Bonus1>" +
 			"</Bonuses>" +
+			"<Projectile>" +
+				"<ActorName>props/units/weapons/rock_flaming.xml</ActorName>" +
+				"<ImpactActorName>props/units/weapons/rock_explosion.xml</ImpactActorName>" +
+				"<ImpactAnimationLifetime>0.1</ImpactAnimationLifetime>" +
+			"</Projectile>" +
 			"<RestrictedClasses datatype=\"tokens\">Champion</RestrictedClasses>" +
 			"<Splash>" +
 				"<Shape>Circular</Shape>" +
@@ -100,9 +106,7 @@ Attack.prototype.Schema =
 	"<optional>" +
 		"<element name='Melee'>" +
 			"<interleave>" +
-				"<element name='Hack' a:help='Hack damage strength'><ref name='nonNegativeDecimal'/></element>" +
-				"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
-				"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
+				DamageTypes.BuildSchema("damage strength") +
 				"<element name='MaxRange' a:help='Maximum attack range (in metres)'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='PrepareTime' a:help='Time from the start of the attack command until the attack actually occurs (in milliseconds). This value relative to RepeatTime should closely match the \"event\" point in the actor&apos;s attack animation'>" +
 					"<data type='nonNegativeInteger'/>" +
@@ -119,14 +123,21 @@ Attack.prototype.Schema =
 	"<optional>" +
 		"<element name='Ranged'>" +
 			"<interleave>" +
-				"<element name='Hack' a:help='Hack damage strength'><ref name='nonNegativeDecimal'/></element>" +
-				"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
-				"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
+				DamageTypes.BuildSchema("damage strength") +
 				"<element name='MaxRange' a:help='Maximum attack range (in metres)'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='MinRange' a:help='Minimum attack range (in metres)'><ref name='nonNegativeDecimal'/></element>" +
 				"<optional>"+
 					"<element name='ElevationBonus' a:help='give an elevation advantage (in meters)'><ref name='nonNegativeDecimal'/></element>" +
 				"</optional>" +
+				"<optional>" +
+					"<element name='RangeOverlay'>" +
+						"<interleave>" +
+							"<element name='LineTexture'><text/></element>" +
+							"<element name='LineTextureMask'><text/></element>" +
+							"<element name='LineThickness'><ref name='nonNegativeDecimal'/></element>" +
+						"</interleave>" +
+					"</element>" +
+				"</optional>" +
 				"<element name='PrepareTime' a:help='Time from the start of the attack command until the attack actually occurs (in milliseconds). This value relative to RepeatTime should closely match the \"event\" point in the actor&apos;s attack animation'>" +
 					"<data type='nonNegativeInteger'/>" +
 				"</element>" +
@@ -136,19 +147,42 @@ Attack.prototype.Schema =
 				"<element name='ProjectileSpeed' a:help='Speed of projectiles (in metres per second)'>" +
 					"<ref name='positiveDecimal'/>" +
 				"</element>" +
+				"<element name='Gravity' a:help='The gravity affecting the projectile. This affects the shape of the flight curve.'>" +
+					"<ref name='nonNegativeDecimal'/>" +
+				"</element>" +
 				"<element name='Spread' a:help='Standard deviation of the bivariate normal distribution of hits at 100 meters. A disk at 100 meters from the attacker with this radius (2x this radius, 3x this radius) is expected to include the landing points of 39.3% (86.5%, 98.9%) of the rounds.'><ref name='nonNegativeDecimal'/></element>" +
+				"<element name='Delay' a:help='Delay of the damage in milliseconds'><ref name='nonNegativeDecimal'/></element>" +
 				Attack.prototype.bonusesSchema +
 				Attack.prototype.preferredClassesSchema +
 				Attack.prototype.restrictedClassesSchema +
+				"<optional>" +
+					"<element name='Projectile'>" +
+						"<interleave>" +
+							"<oneOrMore>" +
+								"<choice>" +
+									"<element name='ActorName' a:help='actor of the projectile animation'>" +
+										"<text/>" +
+									"</element>" +
+									"<interleave>" +
+										"<element name='ImpactActorName' a:help='actor of the projectile impact animation'>" +
+											"<text/>" +
+										"</element>" +
+										"<element name='ImpactAnimationLifetime' a:help='length of the projectile impact animation'>" +
+											"<ref name='positiveDecimal'/>" +
+										"</element>" +
+									"</interleave>" +
+								"</choice>" +
+							"</oneOrMore>" +
+						"</interleave>" +
+					"</element>" +
+				"</optional>" +
 				"<optional>" +
 					"<element name='Splash'>" +
 						"<interleave>" +
 							"<element name='Shape' a:help='Shape of the splash damage, can be circular or linear'><text/></element>" +
 							"<element name='Range' a:help='Size of the area affected by the splash'><ref name='nonNegativeDecimal'/></element>" +
 							"<element name='FriendlyFire' a:help='Whether the splash damage can hurt non enemy units'><data type='boolean'/></element>" +
-							"<element name='Hack' a:help='Hack damage strength'><ref name='nonNegativeDecimal'/></element>" +
-							"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
-							"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
+							DamageTypes.BuildSchema("damage strength") +
 							Attack.prototype.bonusesSchema +
 						"</interleave>" +
 					"</element>" +
@@ -173,9 +207,7 @@ Attack.prototype.Schema =
 	"<optional>" +
 		"<element name='Slaughter' a:help='A special attack to kill domestic animals'>" +
 			"<interleave>" +
-				"<element name='Hack' a:help='Hack damage strength'><ref name='nonNegativeDecimal'/></element>" +
-				"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
-				"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
+				DamageTypes.BuildSchema("damage strength") +
 				"<element name='MaxRange'><ref name='nonNegativeDecimal'/></element>" + // TODO: how do these work?
 				Attack.prototype.bonusesSchema +
 				Attack.prototype.preferredClassesSchema +
@@ -230,12 +262,13 @@ Attack.prototype.CanAttack = function(target, wantedTypes)
 	if (!cmpThisPosition || !cmpTargetPosition || !cmpThisPosition.IsInWorld() || !cmpTargetPosition.IsInWorld())
 		return false;
 
-	let cmpIdentity = Engine.QueryInterface(target, IID_Identity);
+	let cmpIdentity = QueryMiragedInterface(target, IID_Identity);
 	if (!cmpIdentity)
 		return false;
 
+	let cmpHealth = QueryMiragedInterface(target, IID_Health);
 	let targetClasses = cmpIdentity.GetClassesList();
-	if (targetClasses.indexOf("Domestic") != -1 && this.template.Slaughter &&
+	if (targetClasses.indexOf("Domestic") != -1 && this.template.Slaughter && cmpHealth && cmpHealth.GetHitpoints() &&
 	   (!wantedTypes || !wantedTypes.filter(wType => wType.indexOf("!") != 0).length))
 		return true;
 
@@ -256,7 +289,7 @@ Attack.prototype.CanAttack = function(target, wantedTypes)
 
 	for (let type of types)
 	{
-		if (type != "Capture" && !cmpEntityPlayer.IsEnemy(targetOwner))
+		if (type != "Capture" && (!cmpEntityPlayer.IsEnemy(targetOwner) || !cmpHealth || !cmpHealth.GetHitpoints()))
 			continue;
 
 		if (type == "Capture" && (!cmpCapturable || !cmpCapturable.CanCapture(entityOwner)))
@@ -281,11 +314,11 @@ Attack.prototype.CanAttack = function(target, wantedTypes)
  */
 Attack.prototype.GetPreference = function(target)
 {
-	const cmpIdentity = Engine.QueryInterface(target, IID_Identity);
+	let cmpIdentity = Engine.QueryInterface(target, IID_Identity);
 	if (!cmpIdentity)
 		return undefined;
 
-	const targetClasses = cmpIdentity.GetClassesList();
+	let targetClasses = cmpIdentity.GetClassesList();
 
 	let minPref = null;
 	for (let type of this.GetAttackTypes())
@@ -400,11 +433,11 @@ Attack.prototype.GetAttackStrengths = function(type)
 	if (type == "Capture")
 		return { "value": applyMods("Value") };
 
-	return {
-		"hack": applyMods("Hack"),
-		"pierce": applyMods("Pierce"),
-		"crush": applyMods("Crush")
-	};
+	let ret = {};
+	for (let damageType of DamageTypes.GetTypes())
+		ret[damageType] = applyMods(damageType);
+
+	return ret;
 };
 
 Attack.prototype.GetSplashDamage = function(type)
@@ -432,33 +465,13 @@ Attack.prototype.GetRange = function(type)
 	return { "max": max, "min": min, "elevationBonus": elevationBonus };
 };
 
-// Calculate the attack damage multiplier against a target
-Attack.prototype.GetAttackBonus = function(type, target)
+Attack.prototype.GetBonusTemplate = function(type)
 {
-	let attackBonus = 1;
 	let template = this.template[type];
 	if (!template)
 		template = this.template[type.split(".")[0]].Splash;
 
-	if (template.Bonuses)
-	{
-		let cmpIdentity = Engine.QueryInterface(target, IID_Identity);
-		if (!cmpIdentity)
-			return 1;
-
-		// Multiply the bonuses for all matching classes
-		for (let key in template.Bonuses)
-		{
-			let bonus = template.Bonuses[key];
-			if (bonus.Civ && bonus.Civ !== cmpIdentity.GetCiv())
-				continue;
-			if (bonus.Classes && bonus.Classes.split(/\s+/).some(cls => !cmpIdentity.HasClass(cls)))
-				continue;
-			attackBonus *= bonus.Multiplier;
-		}
-	}
-
-	return attackBonus;
+	return template.Bonuses || null;
 };
 
 /**
@@ -481,7 +494,7 @@ Attack.prototype.PerformAttack = function(type, target)
 		//  * Obstacles like walls should block projectiles entirely
 
 		let horizSpeed = +this.template[type].ProjectileSpeed;
-		let gravity = 9.81; // this affects the shape of the curve; assume it's constant for now
+		let gravity = +this.template[type].Gravity;
 		//horizSpeed /= 2; gravity /= 2; // slow it down for testing
 
 		let cmpPosition = Engine.QueryInterface(this.entity, IID_Position);
@@ -501,7 +514,7 @@ Attack.prototype.PerformAttack = function(type, target)
 
 		// Add inaccuracy based on spread.
 		let distanceModifiedSpread = ApplyValueModificationsToEntity("Attack/Ranged/Spread", +this.template.Ranged.Spread, this.entity) *
-			targetPosition.horizDistanceTo(selfPosition) / 100;
+			predictedPosition.horizDistanceTo(selfPosition) / 100;
 
 		let randNorm = randomNormal2D();
 		let offsetX = randNorm[0] * distanceModifiedSpread;
@@ -517,9 +530,41 @@ Attack.prototype.PerformAttack = function(type, target)
 
 		// Launch the graphical projectile.
 		let cmpProjectileManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_ProjectileManager);
-		let id = cmpProjectileManager.LaunchProjectileAtPoint(this.entity, realTargetPosition, horizSpeed, gravity);
 
-		cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+		let actorName = "";
+		let impactActorName = "";
+		let impactAnimationLifetime = 0;
+		if (this.template.Ranged.Projectile)
+		{
+			actorName = this.template.Ranged.Projectile.ActorName || "";
+			impactActorName = this.template.Ranged.Projectile.ImpactActorName || "";
+			impactAnimationLifetime = this.template.Ranged.Projectile.ImpactAnimationLifetime || 0;
+		}
+
+		let launchPoint = selfPosition.clone();
+		// TODO: remove this when all the ranged unit templates are updated with Projectile/Launchpoint
+		launchPoint.y += 3;
+		
+		let cmpVisual = Engine.QueryInterface(this.entity, IID_Visual);
+		if (cmpVisual)
+		{
+			// if the projectile definition is missing from the template
+			// then fallback to the projectile name and launchpoint in the visual actor
+			if (!actorName)
+				actorName = cmpVisual.GetProjectileActor();
+
+			let visualActorLaunchPoint = cmpVisual.GetProjectileLaunchPoint();
+			if (visualActorLaunchPoint.length() > 0)
+				launchPoint = visualActorLaunchPoint;
+		}
+
+		let id = cmpProjectileManager.LaunchProjectileAtPoint(launchPoint, realTargetPosition, horizSpeed, gravity, actorName, impactActorName, impactAnimationLifetime);
+
+		let attackImpactSound = "";
+		let cmpSound = Engine.QueryInterface(this.entity, IID_Sound);
+		if (cmpSound)
+			attackImpactSound = cmpSound.GetSoundGroup("attack_impact_" + type.toLowerCase());
+
 		let data = {
 			"type": type,
 			"attacker": this.entity,
@@ -528,9 +573,10 @@ Attack.prototype.PerformAttack = function(type, target)
 			"position": realTargetPosition,
 			"direction": missileDirection,
 			"projectileId": id,
-			"multiplier": this.GetAttackBonus(type, target),
+			"bonus": this.GetBonusTemplate(type),
 			"isSplash": false,
-			"attackerOwner": attackerOwner
+			"attackerOwner": attackerOwner,
+			"attackImpactSound": attackImpactSound
 		};
 		if (this.template.Ranged.Splash)
 		{
@@ -538,16 +584,17 @@ Attack.prototype.PerformAttack = function(type, target)
 			data.radius = +this.template.Ranged.Splash.Range;
 			data.shape = this.template.Ranged.Splash.Shape;
 			data.isSplash = true;
-			data.splashStrengths = this.GetAttackStrengths(type+".Splash");
+			data.splashStrengths = this.GetAttackStrengths(type + ".Splash");
+			data.splashBonus = this.GetBonusTemplate(type + ".Splash");
 		}
-		cmpTimer.SetTimeout(SYSTEM_ENTITY, IID_Damage, "MissileHit", timeToTarget * 1000, data);
+		cmpTimer.SetTimeout(SYSTEM_ENTITY, IID_Damage, "MissileHit", timeToTarget * 1000 + +this.template.Ranged.Delay, data);
 	}
 	else if (type == "Capture")
 	{
-		if (attackerOwner == -1)
+		if (attackerOwner == INVALID_PLAYER)
 			return;
 
-		let multiplier = this.GetAttackBonus(type, target);
+		let multiplier = GetDamageBonus(target, this.GetBonusTemplate(type));
 		let cmpHealth = Engine.QueryInterface(target, IID_Health);
 		if (!cmpHealth || cmpHealth.GetHitpoints() == 0)
 			return;
@@ -574,7 +621,7 @@ Attack.prototype.PerformAttack = function(type, target)
 			"strengths": this.GetAttackStrengths(type),
 			"target": target,
 			"attacker": this.entity,
-			"multiplier": this.GetAttackBonus(type, target),
+			"multiplier": GetDamageBonus(target, this.GetBonusTemplate(type)),
 			"type": type,
 			"attackerOwner": attackerOwner
 		});
@@ -609,7 +656,7 @@ Attack.prototype.PredictTimeToTarget = function(selfPosition, horizSpeed, target
 		return c / (Math.sqrt(disc) - b);
 
 	return false;
-}
+};
 
 Attack.prototype.OnValueModification = function(msg)
 {
@@ -625,4 +672,22 @@ Attack.prototype.OnValueModification = function(msg)
 		cmpUnitAI.UpdateRangeQueries();
 };
 
+Attack.prototype.GetRangeOverlays = function()
+{
+	if (!this.template.Ranged || !this.template.Ranged.RangeOverlay)
+		return [];
+
+	let range = this.GetRange("Ranged");
+	let rangeOverlays = [];
+	for (let i in range)
+		if ((i == "min" || i == "max") && range[i])
+			rangeOverlays.push({
+				"radius": range[i],
+				"texture": this.template.Ranged.RangeOverlay.LineTexture,
+				"textureMask": this.template.Ranged.RangeOverlay.LineTextureMask,
+				"thickness": +this.template.Ranged.RangeOverlay.LineThickness,
+			});
+	return rangeOverlays;
+};
+
 Engine.RegisterComponentType(IID_Attack, "Attack", Attack);
