@@ -5,13 +5,13 @@ Builder.prototype.Schema =
 	"<a:example>" +
 		"<Rate>1.0</Rate>" +
 		"<Entities datatype='tokens'>" +
-			"\n    structures/{civ}_barracks\n    structures/{civ}_civil_centre\n    structures/pers_apadana\n  " +
+			"\n    structures/{civ}_barracks\n    structures/{native}_civil_centre\n    structures/pers_apadana\n  " +
 		"</Entities>" +
 	"</a:example>" +
 	"<element name='Rate' a:help='Construction speed multiplier (1.0 is normal speed, higher values are faster).'>" +
 		"<ref name='positiveDecimal'/>" +
 	"</element>" +
-	"<element name='Entities' a:help='Space-separated list of entity template names that this unit can build. The special string \"{civ}\" will be automatically replaced by the unit&apos;s four-character civ code. This element can also be empty, in which case no new foundations may be placed by the unit, but they can still repair existing buildings.'>" +
+	"<element name='Entities' a:help='Space-separated list of entity template names that this unit can build. The special string \"{civ}\" will be automatically replaced by the civ code of the unit&apos;s owner, while the string \"{native}\" will be automatically replaced by the unit&apos;s civ code. This element can also be empty, in which case no new foundations may be placed by the unit, but they can still repair existing buildings.'>" +
 		"<attribute name='datatype'>" +
 			"<value>tokens</value>" +
 		"</attribute>" +
@@ -27,22 +27,24 @@ Builder.prototype.Serialize = null; // we have no dynamic state to save
 Builder.prototype.GetEntitiesList = function()
 {
 	let string = this.template.Entities._string;
-
 	if (!string)
 		return [];
 
+	let cmpPlayer = QueryOwnerInterface(this.entity);
+	if (!cmpPlayer)
+		return [];
+
 	let cmpIdentity = Engine.QueryInterface(this.entity, IID_Identity);
 	if (cmpIdentity)
-		string = string.replace(/\{civ\}/g, cmpIdentity.GetCiv());
+		string = string.replace(/\{native\}/g, cmpIdentity.GetCiv());
 
-	let entities = string.split(/\s+/);
-
-	let cmpPlayer = QueryOwnerInterface(this.entity);
-	if (!cmpPlayer)
-		return entities;
+	let entities = string.replace(/\{civ\}/g, cmpPlayer.GetCiv()).split(/\s+/);
 
 	let disabledTemplates = cmpPlayer.GetDisabledTemplates();
-	return entities.filter(ent => !disabledTemplates[ent]);
+
+	let cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+
+	return entities.filter(ent => !disabledTemplates[ent] && cmpTemplateManager.TemplateExists(ent));
 };
 
 Builder.prototype.GetRange = function()
@@ -55,13 +57,18 @@ Builder.prototype.GetRange = function()
 	return { "max": max, "min": 0 };
 };
 
+Builder.prototype.GetRate = function()
+{
+	return ApplyValueModificationsToEntity("Builder/Rate", +this.template.Rate, this.entity);
+};
+
 /**
  * Build/repair the target entity. This should only be called after a successful range check.
  * It should be called at a rate of once per second.
  */
 Builder.prototype.PerformBuilding = function(target)
 {
-	let rate = ApplyValueModificationsToEntity("Builder/Rate", +this.template.Rate, this.entity);
+	let rate = this.GetRate();
 
 	let cmpFoundation = Engine.QueryInterface(target, IID_Foundation);
 	if (cmpFoundation)
