@@ -34,6 +34,7 @@ GuiInterface.prototype.Init = function()
 	this.entsRallyPointsDisplayed = [];
 	this.entsWithAuraAndStatusBars = new Set();
 	this.enabledVisualRangeOverlayTypes = {};
+	this.templateModified = {};
 };
 
 /*
@@ -55,18 +56,15 @@ GuiInterface.prototype.GetSimulationState = function()
 		"players": []
 	};
 
-	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	let numPlayers = cmpPlayerManager.GetNumPlayers();
-
+	let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
 	for (let i = 0; i < numPlayers; ++i)
 	{
-		let playerEnt = cmpPlayerManager.GetPlayerByID(i);
-		let cmpPlayerEntityLimits = Engine.QueryInterface(playerEnt, IID_EntityLimits);
-		let cmpPlayer = Engine.QueryInterface(playerEnt, IID_Player);
+		let cmpPlayer = QueryPlayerIDInterface(i);
+		let cmpPlayerEntityLimits = QueryPlayerIDInterface(i, IID_EntityLimits);
 
 		// Work out what phase we are in
 		let phase = "";
-		let cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager);
+		let cmpTechnologyManager = QueryPlayerIDInterface(i, IID_TechnologyManager);
 		if (cmpTechnologyManager)
 		{
 			if (cmpTechnologyManager.IsTechnologyResearched("phase_city"))
@@ -124,8 +122,8 @@ GuiInterface.prototype.GetSimulationState = function()
 			"researchedTechs": cmpTechnologyManager ? cmpTechnologyManager.GetResearchedTechs() : null,
 			"classCounts": cmpTechnologyManager ? cmpTechnologyManager.GetClassCounts() : null,
 			"typeCountsByClass": cmpTechnologyManager ? cmpTechnologyManager.GetTypeCountsByClass() : null,
-			"canBarter": Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter).PlayerHasMarket(playerEnt),
-			"barterPrices": Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter).GetPrices(playerEnt)
+			"canBarter": Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter).PlayerHasMarket(i),
+			"barterPrices": Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter).GetPrices(i)
 		});
 	}
 
@@ -156,23 +154,13 @@ GuiInterface.prototype.GetSimulationState = function()
 
 	// Add the game type and allied victory
 	let cmpEndGameManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_EndGameManager);
-	ret.gameType = cmpEndGameManager.GetGameType();
+	ret.victoryConditions = cmpEndGameManager.GetVictoryConditions();
 	ret.alliedVictory = cmpEndGameManager.GetAlliedVictory();
 
-	// Add Resource Codes, untranslated names and AI Analysis
-	ret.resources = {
-		"codes": Resources.GetCodes(),
-		"names": Resources.GetNames(),
-		"aiInfluenceGroups": {}
-	};
-	for (let res of ret.resources.codes)
-		ret.resources.aiInfluenceGroups[res] = Resources.GetResource(res).aiAnalysisInfluenceGroup;
-
 	// Add basic statistics to each player
 	for (let i = 0; i < numPlayers; ++i)
 	{
-		let playerEnt = cmpPlayerManager.GetPlayerByID(i);
-		let cmpPlayerStatisticsTracker = Engine.QueryInterface(playerEnt, IID_StatisticsTracker);
+		let cmpPlayerStatisticsTracker = QueryPlayerIDInterface(i, IID_StatisticsTracker);
 		if (cmpPlayerStatisticsTracker)
 			ret.players[i].statistics = cmpPlayerStatisticsTracker.GetBasicStatistics();
 	}
@@ -192,12 +180,10 @@ GuiInterface.prototype.GetExtendedSimulationState = function()
 	let ret = this.GetSimulationState();
 
 	// Add statistics to each player
-	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	let n = cmpPlayerManager.GetNumPlayers();
-	for (let i = 0; i < n; ++i)
+	let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
+	for (let i = 0; i < numPlayers; ++i)
 	{
-		let playerEnt = cmpPlayerManager.GetPlayerByID(i);
-		let cmpPlayerStatisticsTracker = Engine.QueryInterface(playerEnt, IID_StatisticsTracker);
+		let cmpPlayerStatisticsTracker = QueryPlayerIDInterface(i, IID_StatisticsTracker);
 		if (cmpPlayerStatisticsTracker)
 			ret.players[i].sequences = cmpPlayerStatisticsTracker.GetSequences();
 	}
@@ -209,8 +195,8 @@ GuiInterface.prototype.GetRenamedEntities = function(player)
 {
 	if (this.miragedEntities[player])
 		return this.renamedEntities.concat(this.miragedEntities[player]);
-	else
-		return this.renamedEntities;
+
+	return this.renamedEntities;
 };
 
 GuiInterface.prototype.ClearRenamedEntities = function()
@@ -241,30 +227,8 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 
 	let ret = {
 		"id": ent,
-		"template": template,
-
-		"alertRaiser": null,
-		"builder": null,
-		"canGarrison": null,
-		"identity": null,
-		"fogging": null,
-		"foundation": null,
-		"garrisonHolder": null,
-		"gate": null,
-		"guard": null,
-		"market": null,
-		"mirage": null,
-		"pack": null,
-		"upgrade" : null,
-		"player": -1,
-		"position": null,
-		"production": null,
-		"rallyPoint": null,
-		"resourceCarrying": null,
-		"rotation": null,
-		"trader": null,
-		"unitAI": null,
-		"visibility": null,
+		"player": INVALID_PLAYER,
+		"template": template
 	};
 
 	let cmpMirage = Engine.QueryInterface(ent, IID_Mirage);
@@ -277,15 +241,13 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 			"rank": cmpIdentity.GetRank(),
 			"classes": cmpIdentity.GetClassesList(),
 			"visibleClasses": cmpIdentity.GetVisibleClassesList(),
-			"selectionGroupName": cmpIdentity.GetSelectionGroupName()
+			"selectionGroupName": cmpIdentity.GetSelectionGroupName(),
+			"canDelete": !cmpIdentity.IsUndeletable()
 		};
 
 	let cmpPosition = Engine.QueryInterface(ent, IID_Position);
 	if (cmpPosition && cmpPosition.IsInWorld())
-	{
 		ret.position = cmpPosition.GetPosition();
-		ret.rotation = cmpPosition.GetRotation();
-	}
 
 	let cmpHealth = QueryMiragedInterface(ent, IID_Health);
 	if (cmpHealth)
@@ -294,7 +256,6 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 		ret.maxHitpoints = cmpHealth.GetMaxHitpoints();
 		ret.needsRepair = cmpHealth.IsRepairable() && cmpHealth.GetHitpoints() < cmpHealth.GetMaxHitpoints();
 		ret.needsHeal = !cmpHealth.IsUnhealable();
-		ret.canDelete = !cmpHealth.IsUndeletable();
 	}
 
 	let cmpCapturable = QueryMiragedInterface(ent, IID_Capturable);
@@ -325,7 +286,7 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 	var cmpUpgrade = Engine.QueryInterface(ent, IID_Upgrade);
 	if (cmpUpgrade)
 		ret.upgrade = {
-			"upgrades" : cmpUpgrade.GetUpgrades(),
+			"upgrades": cmpUpgrade.GetUpgrades(),
 			"progress": cmpUpgrade.GetProgress(),
 			"template": cmpUpgrade.GetUpgradingTo()
 		};
@@ -345,22 +306,19 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 			"goods": cmpTrader.GetGoods()
 		};
 
-	let cmpFogging = Engine.QueryInterface(ent, IID_Fogging);
-	if (cmpFogging)
-		ret.fogging = {
-			"mirage": cmpFogging.IsMiraged(player) ? cmpFogging.GetMirage(player) : null
-		};
-
 	let cmpFoundation = QueryMiragedInterface(ent, IID_Foundation);
 	if (cmpFoundation)
 		ret.foundation = {
-			"progress": cmpFoundation.GetBuildPercentage(),
-			"numBuilders": cmpFoundation.GetNumBuilders()
+			"numBuilders": cmpFoundation.GetNumBuilders(),
+			"buildTime": cmpFoundation.GetBuildTime()
 		};
 
 	let cmpRepairable = QueryMiragedInterface(ent, IID_Repairable);
 	if (cmpRepairable)
-		ret.repairable = { "numBuilders": cmpRepairable.GetNumBuilders() };
+		ret.repairable = {
+			"numBuilders": cmpRepairable.GetNumBuilders(),
+			"buildTime": cmpRepairable.GetBuildTime()
+		};
 
 	let cmpOwnership = Engine.QueryInterface(ent, IID_Ownership);
 	if (cmpOwnership)
@@ -391,8 +349,8 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 			"canGuard": cmpUnitAI.CanGuard(),
 			"isGuarding": cmpUnitAI.IsGuardOf(),
 			"canPatrol": cmpUnitAI.CanPatrol(),
-			"possibleStances": cmpUnitAI.GetPossibleStances(),
-			"isIdle":cmpUnitAI.IsIdle(),
+			"selectableStances": cmpUnitAI.GetSelectableStances(),
+			"isIdle": cmpUnitAI.IsIdle(),
 		};
 
 	let cmpGuard = Engine.QueryInterface(ent, IID_Guard);
@@ -403,7 +361,10 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 
 	let cmpResourceGatherer = Engine.QueryInterface(ent, IID_ResourceGatherer);
 	if (cmpResourceGatherer)
+	{
 		ret.resourceCarrying = cmpResourceGatherer.GetCarryingStatus();
+		ret.resourceGatherRates = cmpResourceGatherer.GetGatherRates();
+	}
 
 	let cmpGate = Engine.QueryInterface(ent, IID_Gate);
 	if (cmpGate)
@@ -413,44 +374,11 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 
 	let cmpAlertRaiser = Engine.QueryInterface(ent, IID_AlertRaiser);
 	if (cmpAlertRaiser)
-		ret.alertRaiser = {
-			"level": cmpAlertRaiser.GetLevel(),
-			"canIncreaseLevel": cmpAlertRaiser.CanIncreaseLevel(),
-			"hasRaisedAlert": cmpAlertRaiser.HasRaisedAlert(),
-		};
+		ret.alertRaiser = true;
 
 	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	ret.visibility = cmpRangeManager.GetLosVisibility(ent, player);
 
-	return ret;
-};
-
-/**
- * Get additionnal entity info, rarely used in the gui
- */
-GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
-{
-	let ret = {
-		"armour": null,
-		"attack": null,
-		"buildingAI": null,
-		"heal": null,
-		"isBarterMarket": null,
-		"loot": null,
-		"obstruction": null,
-		"turretParent":null,
-		"promotion": null,
-		"repairRate": null,
-		"buildRate": null,
-		"resourceDropsite": null,
-		"resourceGatherRates": null,
-		"resourceSupply": null,
-		"resourceTrickle": null,
-		"speed": null,
-	};
-
-	let cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
-
 	let cmpAttack = Engine.QueryInterface(ent, IID_Attack);
 	if (cmpAttack)
 	{
@@ -481,10 +409,6 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 
 			ret.attack[type].elevationBonus = range.elevationBonus;
 
-			let cmpPosition = Engine.QueryInterface(ent, IID_Position);
-			let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-			let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-
 			if (cmpUnitAI && cmpPosition && cmpPosition.IsInWorld())
 			{
 				// For units, take the range in front of it, no spread. So angle = 0
@@ -507,10 +431,6 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 	if (cmpArmour)
 		ret.armour = cmpArmour.GetArmourStrengths();
 
-	let cmpAuras = Engine.QueryInterface(ent, IID_Auras);
-	if (cmpAuras)
-		ret.auras = cmpAuras.GetDescriptions();
-
 	let cmpBuildingAI = Engine.QueryInterface(ent, IID_BuildingAI);
 	if (cmpBuildingAI)
 		ret.buildingAI = {
@@ -521,25 +441,9 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 			"arrowCount": cmpBuildingAI.GetArrowCount()
 		};
 
-	let cmpObstruction = Engine.QueryInterface(ent, IID_Obstruction);
-	if (cmpObstruction)
-		ret.obstruction = {
-			"controlGroup": cmpObstruction.GetControlGroup(),
-			"controlGroup2": cmpObstruction.GetControlGroup2(),
-		};
-
-	let cmpPosition = Engine.QueryInterface(ent, IID_Position);
 	if (cmpPosition && cmpPosition.GetTurretParent() != INVALID_ENTITY)
 		ret.turretParent = cmpPosition.GetTurretParent();
 
-	let cmpRepairable = Engine.QueryInterface(ent, IID_Repairable);
-	if (cmpRepairable)
-		ret.repairRate = cmpRepairable.GetRepairRate();
-
-	let cmpFoundation = Engine.QueryInterface(ent, IID_Foundation);
-	if (cmpFoundation)
-		ret.buildRate = cmpFoundation.GetBuildRate();
-
 	let cmpResourceSupply = QueryMiragedInterface(ent, IID_ResourceSupply);
 	if (cmpResourceSupply)
 		ret.resourceSupply = {
@@ -552,10 +456,6 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 			"numGatherers": cmpResourceSupply.GetNumGatherers()
 		};
 
-	let cmpResourceGatherer = Engine.QueryInterface(ent, IID_ResourceGatherer);
-	if (cmpResourceGatherer)
-		ret.resourceGatherRates = cmpResourceGatherer.GetGatherRates();
-
 	let cmpResourceDropsite = Engine.QueryInterface(ent, IID_ResourceDropsite);
 	if (cmpResourceDropsite)
 		ret.resourceDropsite = {
@@ -587,27 +487,17 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 	let cmpLoot = Engine.QueryInterface(ent, IID_Loot);
 	if (cmpLoot)
 	{
-		let resources = cmpLoot.GetResources();
-		ret.loot = {
-			"xp": cmpLoot.GetXp()
-		};
-		for (let res of Resources.GetCodes())
-			ret.loot[res] = resources[res];
+		ret.loot = cmpLoot.GetResources();
+		ret.loot.xp = cmpLoot.GetXp();
 	}
 
 	let cmpResourceTrickle = Engine.QueryInterface(ent, IID_ResourceTrickle);
 	if (cmpResourceTrickle)
-	{
 		ret.resourceTrickle = {
 			"interval": cmpResourceTrickle.GetTimer(),
-			"rates": {}
+			"rates": cmpResourceTrickle.GetRates()
 		};
 
-		let rates = cmpResourceTrickle.GetRates();
-		for (let res in rates)
-			ret.resourceTrickle.rates[res] = rates[res];
-	}
-
 	let cmpUnitMotion = Engine.QueryInterface(ent, IID_UnitMotion);
 	if (cmpUnitMotion)
 		ret.speed = {
@@ -618,6 +508,11 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 	return ret;
 };
 
+GuiInterface.prototype.GetMultipleEntityStates = function(player, ents)
+{
+	return ents.map(ent => ({ "entId": ent, "state": this.GetEntityState(player, ent) }));
+};
+
 GuiInterface.prototype.GetAverageRangeForBuildings = function(player, cmd)
 {
 	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
@@ -636,10 +531,10 @@ GuiInterface.prototype.GetAverageRangeForBuildings = function(player, cmd)
 	return cmpRangeManager.GetElevationAdaptedRange(pos, rot, range, elevationBonus, 2*Math.PI);
 };
 
-GuiInterface.prototype.GetTemplateData = function(player, name)
+GuiInterface.prototype.GetTemplateData = function(player, templateName)
 {
 	let cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	let template = cmpTemplateManager.GetTemplate(name);
+	let template = cmpTemplateManager.GetTemplate(templateName);
 
 	if (!template)
 		return null;
@@ -647,29 +542,14 @@ GuiInterface.prototype.GetTemplateData = function(player, name)
 	let aurasTemplate = {};
 
 	if (!template.Auras)
-		return GetTemplateDataHelper(template, player, aurasTemplate, Resources);
+		return GetTemplateDataHelper(template, player, aurasTemplate, Resources, DamageTypes);
 
-	// Add aura name and description loaded from JSON file
 	let auraNames = template.Auras._string.split(/\s+/);
-	let cmpDataTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager);
+
 	for (let name of auraNames)
-		aurasTemplate[name] = cmpDataTemplateManager.GetAuraTemplate(name);
-	return GetTemplateDataHelper(template, player, aurasTemplate, Resources);
-};
+		aurasTemplate[name] = AuraTemplates.Get(name);
 
-GuiInterface.prototype.GetTechnologyData = function(player, data)
-{
-	let cmpDataTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager);
-	let template = cmpDataTemplateManager.GetTechnologyTemplate(data.name);
-
-	if (!template)
-	{
-		warn("Tried to get data for invalid technology: " + data.name);
-		return null;
-	}
-
-	let cmpPlayer = QueryPlayerIDInterface(player, IID_Player);
-	return GetTechnologyDataHelper(template, data.civ || cmpPlayer.GetCiv(), Resources);
+	return GetTemplateDataHelper(template, player, aurasTemplate, Resources, DamageTypes);
 };
 
 GuiInterface.prototype.IsTechnologyResearched = function(player, data)
@@ -705,14 +585,20 @@ GuiInterface.prototype.GetStartedResearch = function(player)
 		return {};
 
 	let ret = {};
-	for (let tech in cmpTechnologyManager.GetStartedTechs())
+	for (let tech of cmpTechnologyManager.GetStartedTechs())
 	{
 		ret[tech] = { "researcher": cmpTechnologyManager.GetResearcher(tech) };
 		let cmpProductionQueue = Engine.QueryInterface(ret[tech].researcher, IID_ProductionQueue);
 		if (cmpProductionQueue)
+		{
 			ret[tech].progress = cmpProductionQueue.GetQueue()[0].progress;
+			ret[tech].timeRemaining = cmpProductionQueue.GetQueue()[0].timeRemaining;
+		}
 		else
+		{
 			ret[tech].progress = 0;
+			ret[tech].timeRemaining = 0;
+		}
 	}
 	return ret;
 };
@@ -740,6 +626,25 @@ GuiInterface.prototype.GetNeededResources = function(player, data)
 	return QueryPlayerIDInterface(data.player || player).GetNeededResources(data.cost);
 };
 
+/**
+ * State of the templateData (player dependent): true when some template values have been modified
+ * and need to be reloaded by the gui.
+ */
+GuiInterface.prototype.OnTemplateModification = function(msg)
+{
+	this.templateModified[msg.player] = true;
+};
+
+GuiInterface.prototype.IsTemplateModified = function(player)
+{
+	return this.templateModified[player] || false;
+};
+
+GuiInterface.prototype.ResetTemplateModified = function()
+{
+	this.templateModified = {};
+};
+
 /**
  * Add a timed notification.
  * Warning: timed notifacations are serialised
@@ -753,14 +658,12 @@ GuiInterface.prototype.AddTimeNotification = function(notification, duration = 1
 	notification.id = ++this.timeNotificationID;
 
 	// Let all players and observers receive the notification by default
-	if (notification.players == undefined)
+	if (!notification.players)
 	{
-		let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-		let numPlayers = cmpPlayerManager.GetNumPlayers();
-		notification.players = [-1];
-		for (let i = 1; i < numPlayers; ++i)
-			notification.players.push(i);
+		notification.players = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetAllPlayers();
+		notification.players[0] = -1;
 	}
+
 	this.timeNotifications.push(notification);
 	this.timeNotifications.sort((n1, n2) => n2.endTime - n1.endTime);
 
@@ -828,15 +731,10 @@ GuiInterface.prototype.GetFormationInfoFromTemplate = function(player, data)
 
 GuiInterface.prototype.IsFormationSelected = function(player, data)
 {
-	for (let ent of data.ents)
-	{
+	return data.ents.some(ent => {
 		let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-		// GetLastFormationName is named in a strange way as it (also) is
-		// the value of the current formation (see Formation.js LoadFormation)
-		if (cmpUnitAI && cmpUnitAI.GetLastFormationTemplate() == data.formationTemplate)
-			return true;
-	}
-	return false;
+		return cmpUnitAI && cmpUnitAI.GetFormationTemplate() == data.formationTemplate;
+	});
 };
 
 GuiInterface.prototype.IsStanceSelected = function(player, data)
@@ -866,6 +764,39 @@ GuiInterface.prototype.GetAllBuildableEntities = function(player, cmd)
 	return buildableEnts;
 };
 
+GuiInterface.prototype.UpdateDisplayedPlayerColors = function(player, data)
+{
+	let updateEntityColor = (iids, entities) => {
+		for (let ent of entities)
+			for (let iid of iids)
+			{
+				let cmp = Engine.QueryInterface(ent, iid);
+				if (cmp)
+					cmp.UpdateColor();
+			}
+	};
+
+	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+	let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
+	for (let i = 1; i < numPlayers; ++i)
+	{
+		let cmpPlayer = QueryPlayerIDInterface(i, IID_Player);
+		if (!cmpPlayer)
+			continue;
+
+		cmpPlayer.SetDisplayDiplomacyColor(data.displayDiplomacyColors);
+		if (data.displayDiplomacyColors)
+			cmpPlayer.SetDiplomacyColor(data.displayedPlayerColors[i]);
+
+		updateEntityColor(data.showAllStatusBars && (i == player || player == -1) ?
+			[IID_Minimap, IID_RangeOverlayRenderer, IID_RallyPointRenderer, IID_StatusBars] :
+			[IID_Minimap, IID_RangeOverlayRenderer, IID_RallyPointRenderer],
+			cmpRangeManager.GetEntitiesByPlayer(i));
+	}
+	updateEntityColor([IID_Selectable, IID_StatusBars], data.selected);
+	Engine.QueryInterface(SYSTEM_ENTITY, IID_TerritoryManager).UpdateColors();
+};
+
 GuiInterface.prototype.SetSelectionHighlight = function(player, cmd)
 {
 	let playerColors = {}; // cache of owner -> color map
@@ -877,7 +808,7 @@ GuiInterface.prototype.SetSelectionHighlight = function(player, cmd)
 			continue;
 
 		// Find the entity's owner's color:
-		let owner = -1;
+		let owner = INVALID_PLAYER;
 		let cmpOwnership = Engine.QueryInterface(ent, IID_Ownership);
 		if (cmpOwnership)
 			owner = cmpOwnership.GetOwner();
@@ -885,20 +816,20 @@ GuiInterface.prototype.SetSelectionHighlight = function(player, cmd)
 		let color = playerColors[owner];
 		if (!color)
 		{
-			color = { "r":1, "g":1, "b":1 };
+			color = { "r": 1, "g": 1, "b": 1 };
 			let cmpPlayer = QueryPlayerIDInterface(owner);
 			if (cmpPlayer)
-				color = cmpPlayer.GetColor();
+				color = cmpPlayer.GetDisplayedColor();
 			playerColors[owner] = color;
 		}
 
 		cmpSelectable.SetSelectionHighlight({ "r": color.r, "g": color.g, "b": color.b, "a": cmd.alpha }, cmd.selected);
 
-		let cmpRangeVisualization = Engine.QueryInterface(ent, IID_RangeVisualization);
-		if (!cmpRangeVisualization || player != owner && player != -1)
+		let cmpRangeOverlayManager = Engine.QueryInterface(ent, IID_RangeOverlayManager);
+		if (!cmpRangeOverlayManager || player != owner && player != INVALID_PLAYER)
 			continue;
 
-		cmpRangeVisualization.SetEnabled(cmd.selected, this.enabledVisualRangeOverlayTypes, false);
+		cmpRangeOverlayManager.SetEnabled(cmd.selected, this.enabledVisualRangeOverlayTypes, false);
 	}
 };
 
@@ -909,7 +840,7 @@ GuiInterface.prototype.EnableVisualRangeOverlayType = function(player, data)
 
 GuiInterface.prototype.GetEntitiesWithStatusBars = function()
 {
-	return [...this.entsWithAuraAndStatusBars];
+	return Array.from(this.entsWithAuraAndStatusBars);
 };
 
 GuiInterface.prototype.SetStatusBars = function(player, cmd)
@@ -920,7 +851,7 @@ GuiInterface.prototype.SetStatusBars = function(player, cmd)
 		let cmpStatusBars = Engine.QueryInterface(ent, IID_StatusBars);
 		if (!cmpStatusBars)
 			continue;
-		cmpStatusBars.SetEnabled(cmd.enabled);
+		cmpStatusBars.SetEnabled(cmd.enabled, cmd.showRank);
 
 		let cmpAuras = Engine.QueryInterface(ent, IID_Auras);
 		if (!cmpAuras)
@@ -951,9 +882,9 @@ GuiInterface.prototype.SetRangeOverlays = function(player, cmd)
 {
 	for (let ent of cmd.entities)
 	{
-		let cmpRangeVisualization = Engine.QueryInterface(ent, IID_RangeVisualization);
-		if (cmpRangeVisualization)
-			cmpRangeVisualization.SetEnabled(cmd.enabled, this.enabledVisualRangeOverlayTypes, true);
+		let cmpRangeOverlayManager = Engine.QueryInterface(ent, IID_RangeOverlayManager);
+		if (cmpRangeOverlayManager)
+			cmpRangeOverlayManager.SetEnabled(cmd.enabled, this.enabledVisualRangeOverlayTypes, true);
 	}
 };
 
@@ -964,7 +895,7 @@ GuiInterface.prototype.GetPlayerEntities = function(player)
 
 GuiInterface.prototype.GetNonGaiaEntities = function()
 {
-    return Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager).GetNonGaiaEntities();
+	return Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager).GetNonGaiaEntities();
 };
 
 /**
@@ -1039,6 +970,16 @@ GuiInterface.prototype.DisplayRallyPoint = function(player, cmd)
 	}
 };
 
+GuiInterface.prototype.AddTargetMarker = function(player, cmd)
+{
+	let ent = Engine.AddLocalEntity(cmd.template);
+	if (!ent)
+		return;
+
+	let cmpPosition = Engine.QueryInterface(ent, IID_Position);
+	cmpPosition.JumpTo(cmd.x, cmd.z);
+};
+
 /**
  * Display the building placement preview.
  * cmd.template is the name of the entity template, or "" to disable the preview.
@@ -1101,6 +1042,10 @@ GuiInterface.prototype.SetBuildingPlacementPreview = function(player, cmd)
 		else
 			result = cmpBuildRestrictions.CheckPlacement();
 
+		let cmpRangeOverlayManager = Engine.QueryInterface(ent, IID_RangeOverlayManager);
+		if (cmpRangeOverlayManager)
+			cmpRangeOverlayManager.SetEnabled(true, this.enabledVisualRangeOverlayTypes);
+
 		// Set it to a red shade if this is an invalid location
 		let cmpVisual = Engine.QueryInterface(ent, IID_Visual);
 		if (cmpVisual)
@@ -1218,39 +1163,40 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 
 		return false;
 	}
-	else
-	{
-		// Move all existing cached entities outside of the world and reset their use count
-		for (let tpl in this.placementWallEntities)
-		{
-			for (let ent of this.placementWallEntities[tpl].entities)
-			{
-				let pos = Engine.QueryInterface(ent, IID_Position);
-				if (pos)
-					pos.MoveOutOfWorld();
-			}
 
-			this.placementWallEntities[tpl].numUsed = 0;
+	// Move all existing cached entities outside of the world and reset their use count
+	for (let tpl in this.placementWallEntities)
+	{
+		for (let ent of this.placementWallEntities[tpl].entities)
+		{
+			let pos = Engine.QueryInterface(ent, IID_Position);
+			if (pos)
+				pos.MoveOutOfWorld();
 		}
 
-		// Create cache entries for templates we haven't seen before
-		for (let type in wallSet.templates)
-		{
-			let tpl = wallSet.templates[type];
-			if (!(tpl in this.placementWallEntities))
-			{
-				this.placementWallEntities[tpl] = {
-					"numUsed": 0,
-					"entities": [],
-					"templateData": this.GetTemplateData(player, tpl),
-				};
+		this.placementWallEntities[tpl].numUsed = 0;
+	}
 
-				// ensure that the loaded template data contains a wallPiece component
-				if (!this.placementWallEntities[tpl].templateData.wallPiece)
-				{
-					error("[SetWallPlacementPreview] No WallPiece component found for wall set template '" + tpl + "'");
-					return false;
-				}
+	// Create cache entries for templates we haven't seen before
+	for (let type in wallSet.templates)
+	{
+		if (type == "curves")
+			continue;
+
+		let tpl = wallSet.templates[type];
+		if (!(tpl in this.placementWallEntities))
+		{
+			this.placementWallEntities[tpl] = {
+				"numUsed": 0,
+				"entities": [],
+				"templateData": this.GetTemplateData(player, tpl),
+			};
+
+			// ensure that the loaded template data contains a wallPiece component
+			if (!this.placementWallEntities[tpl].templateData.wallPiece)
+			{
+				error("[SetWallPlacementPreview] No WallPiece component found for wall set template '" + tpl + "'");
+				return false;
 			}
 		}
 	}
@@ -1358,7 +1304,7 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 					"template": wallSet.templates.tower,
 					"pos": start.pos,
 					"angle": cmpPosition.GetRotation().y,
-					"controlGroups": [(startEntObstruction ? startEntObstruction.GetControlGroup() : undefined)],
+					"controlGroups": [startEntObstruction ? startEntObstruction.GetControlGroup() : undefined],
 					"excludeFromResult": true, // preview only, must not appear in the result
 				});
 		}
@@ -1383,7 +1329,7 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 		previewEntities.unshift({
 			"template": wallSet.templates.tower,
 			"pos": start.pos,
-			"angle": (previewEntities.length > 0 ? previewEntities[0].angle : this.placementWallLastAngle)
+			"angle": previewEntities.length > 0 ? previewEntities[0].angle : this.placementWallLastAngle
 		});
 	}
 
@@ -1401,7 +1347,7 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 			// '.controlGroup' property. Note that this array can only ever have 0, 1 or 2 elements (checked at a later time).
 			if (previewEntities.length > 0 && endEntObstruction)
 			{
-				previewEntities[previewEntities.length-1].controlGroups = (previewEntities[previewEntities.length-1].controlGroups || []);
+				previewEntities[previewEntities.length-1].controlGroups = previewEntities[previewEntities.length-1].controlGroups || [];
 				previewEntities[previewEntities.length-1].controlGroups.push(endEntObstruction.GetControlGroup());
 			}
 
@@ -1415,7 +1361,7 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 						"template": wallSet.templates.tower,
 						"pos": end.pos,
 						"angle": cmpPosition.GetRotation().y,
-						"controlGroups": [(endEntObstruction ? endEntObstruction.GetControlGroup() : undefined)],
+						"controlGroups": [endEntObstruction ? endEntObstruction.GetControlGroup() : undefined],
 						"excludeFromResult": true
 					});
 			}
@@ -1424,7 +1370,7 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 			previewEntities.push({
 				"template": wallSet.templates.tower,
 				"pos": end.pos,
-				"angle": (previewEntities.length > 0 ? previewEntities[previewEntities.length-1].angle : this.placementWallLastAngle)
+				"angle": previewEntities.length > 0 ? previewEntities[previewEntities.length-1].angle : this.placementWallLastAngle
 			});
 	}
 
@@ -1546,7 +1492,7 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 
 		// Check whether it's in a visible or fogged region
 		// TODO: should definitely reuse SetBuildingPlacementPreview, this is just straight up copy/pasta
-		let visible = (cmpRangeManager.GetLosVisibility(ent, player) != "hidden");
+		let visible = cmpRangeManager.GetLosVisibility(ent, player) != "hidden";
 		if (visible)
 		{
 			let cmpBuildRestrictions = Engine.QueryInterface(ent, IID_BuildRestrictions);
@@ -1557,7 +1503,7 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 			}
 
 			// TODO: Handle results of CheckPlacement
-			validPlacement = (cmpBuildRestrictions && cmpBuildRestrictions.CheckPlacement().success);
+			validPlacement = cmpBuildRestrictions && cmpBuildRestrictions.CheckPlacement().success;
 
 			// If a wall piece has two control groups, it's likely a segment that spans
 			// between two existing towers. To avoid placing a duplicate wall segment,
@@ -1965,10 +1911,9 @@ let exposedFunctions = {
 	"GetRenamedEntities": 1,
 	"ClearRenamedEntities": 1,
 	"GetEntityState": 1,
-	"GetExtendedEntityState": 1,
+	"GetMultipleEntityStates": 1,
 	"GetAverageRangeForBuildings": 1,
 	"GetTemplateData": 1,
-	"GetTechnologyData": 1,
 	"IsTechnologyResearched": 1,
 	"CheckTechnologyRequirements": 1,
 	"GetStartedResearch": 1,
@@ -1985,12 +1930,14 @@ let exposedFunctions = {
 	"GetFormationInfoFromTemplate": 1,
 	"IsStanceSelected": 1,
 
+	"UpdateDisplayedPlayerColors": 1,
 	"SetSelectionHighlight": 1,
 	"GetAllBuildableEntities": 1,
 	"SetStatusBars": 1,
 	"GetPlayerEntities": 1,
 	"GetNonGaiaEntities": 1,
 	"DisplayRallyPoint": 1,
+	"AddTargetMarker": 1,
 	"SetBuildingPlacementPreview": 1,
 	"SetWallPlacementPreview": 1,
 	"GetFoundationSnapData": 1,
@@ -2013,14 +1960,16 @@ let exposedFunctions = {
 
 	"GetTraderNumber": 1,
 	"GetTradingGoods": 1,
+	"IsTemplateModified": 1,
+	"ResetTemplateModified": 1
 };
 
 GuiInterface.prototype.ScriptCall = function(player, name, args)
 {
 	if (exposedFunctions[name])
 		return this[name](player, args);
-	else
-		throw new Error("Invalid GuiInterface Call name \""+name+"\"");
+
+	throw new Error("Invalid GuiInterface Call name \""+name+"\"");
 };
 
 Engine.RegisterSystemComponentType(IID_GuiInterface, "GuiInterface", GuiInterface);
