@@ -24,12 +24,13 @@ Looter.prototype.Collect = function(targetEntity)
 	);
 
 	// Loot resources as defined in the templates
-	var resources = cmpLoot.GetResources();
-	for (let type in resources)
-		resources[type] = ApplyValueModificationsToEntity("Looter/Resource/"+type, resources[type], this.entity)
-			+ (resourcesCarried[type] || 0);
-
-	// TODO: stop assuming that cmpLoot.GetResources() delivers all resource types (by defining them in a central location)
+	let lootTemplate = cmpLoot.GetResources();
+	let resources = {};
+	for (let type of Resources.GetCodes())
+		resources[type] =
+			ApplyValueModificationsToEntity(
+				"Looter/Resource/"+type, lootTemplate[type] || 0, this.entity) +
+			(resourcesCarried[type] || 0);
 
 	// Transfer resources
 	var cmpPlayer = QueryOwnerInterface(this.entity);
