@@ -21,6 +21,21 @@ Player.prototype.Schema =
 		"<ref name='nonNegativeDecimal'/>" +
 	"</element>";
 
+/**
+ * Don't serialize diplomacyColor or displayDiplomacyColor since they're modified by the GUI.
+ */
+Player.prototype.Serialize = function()
+{
+	let state = {};
+	for (let key in this)
+		if (this.hasOwnProperty(key))
+			state[key] = this[key];
+
+	state.diplomacyColor = undefined;
+	state.displayDiplomacyColor = false;
+	return state;
+};
+
 /**
  * Which units will be shown with special icons at the top.
  */
@@ -31,7 +46,9 @@ Player.prototype.Init = function()
 	this.playerID = undefined;
 	this.name = undefined;	// define defaults elsewhere (supporting other languages)
 	this.civ = undefined;
-	this.color = { "r": 0.0, "g": 0.0, "b": 0.0, "a": 1.0 };
+	this.color = undefined;
+	this.diplomacyColor = undefined;
+	this.displayDiplomacyColor = false;
 	this.popUsed = 0; // population of units owned or trained by this player
 	this.popBonuses = 0; // sum of population bonuses of player's entities
 	this.maxPop = 300; // maximum population
@@ -47,6 +64,7 @@ Player.prototype.Init = function()
 	this.startCam = undefined;
 	this.controlAllUnits = false;
 	this.isAI = false;
+	this.timeMultiplier = 1;
 	this.gatherRateMultiplier = 1;
 	this.tradeRateMultiplier = 1;
 	this.cheatsEnabled = false;
@@ -72,7 +90,7 @@ Player.prototype.Init = function()
 		this.resourceCount[res] = 300;
 		this.resourceNames[res] = Resources.GetResource(res).name;
 		this.tradingGoods.push({
-			"goods":  res,
+			"goods": res,
 			"proba": 5 * (quotient + (+i < remainder ? 1 : 0))
 		});
 	}
@@ -120,7 +138,25 @@ Player.prototype.GetCiv = function()
 
 Player.prototype.SetColor = function(r, g, b)
 {
-	this.color = { "r": r/255.0, "g": g/255.0, "b": b/255.0, "a": 1.0 };
+	var colorInitialized = !!this.color;
+
+	this.color = { "r": r / 255, "g": g / 255, "b": b / 255, "a": 1 };
+
+	// Used in Atlas
+	if (colorInitialized)
+		Engine.BroadcastMessage(MT_PlayerColorChanged, {
+			"player": this.playerID
+		});
+};
+
+Player.prototype.SetDiplomacyColor = function(color)
+{
+	this.diplomacyColor = { "r": color.r / 255, "g": color.g / 255, "b": color.b / 255, "a": 1 };
+};
+
+Player.prototype.SetDisplayDiplomacyColor = function(displayDiplomacyColor)
+{
+	this.displayDiplomacyColor = displayDiplomacyColor;
 };
 
 Player.prototype.GetColor = function()
@@ -128,6 +164,11 @@ Player.prototype.GetColor = function()
 	return this.color;
 };
 
+Player.prototype.GetDisplayedColor = function()
+{
+	return this.displayDiplomacyColor ? this.diplomacyColor : this.color;
+};
+
 // Try reserving num population slots. Returns 0 on success or number of missing slots otherwise.
 Player.prototype.TryReservePopulationSlots = function(num)
 {
@@ -175,7 +216,7 @@ Player.prototype.SetMaxPopulation = function(max)
 
 Player.prototype.GetMaxPopulation = function()
 {
-	return Math.round(ApplyValueModificationsToPlayer("Player/MaxPopulation", this.maxPop, this.entity, this.playerID));
+	return Math.round(ApplyValueModificationsToEntity("Player/MaxPopulation", this.maxPop, this.entity));
 };
 
 Player.prototype.GetBarterMultiplier = function()
@@ -183,19 +224,14 @@ Player.prototype.GetBarterMultiplier = function()
 	return this.barterMultiplier;
 };
 
-Player.prototype.SetGatherRateMultiplier = function(value)
-{
-	this.gatherRateMultiplier = value;
-};
-
 Player.prototype.GetGatherRateMultiplier = function()
 {
-	return this.gatherRateMultiplier;
+	return this.gatherRateMultiplier / this.cheatTimeMultiplier;
 };
 
-Player.prototype.SetTradeRateMultiplier = function(value)
+Player.prototype.GetTimeMultiplier = function()
 {
-	this.tradeRateMultiplier = value;
+	return this.timeMultiplier * this.cheatTimeMultiplier;
 };
 
 Player.prototype.GetTradeRateMultiplier = function()
@@ -208,6 +244,26 @@ Player.prototype.GetSpyCostMultiplier = function()
 	return this.spyCostMultiplier;
 };
 
+/**
+ * Setters currently used by the AI to set the difficulty level
+ */
+Player.prototype.SetGatherRateMultiplier = function(value)
+{
+	this.gatherRateMultiplier = value;
+	Engine.BroadcastMessage(MT_MultiplierChanged, { "player": this.playerID, "type": "gather" });
+};
+
+Player.prototype.SetTimeMultiplier = function(value)
+{
+	this.timeMultiplier = value;
+	Engine.BroadcastMessage(MT_MultiplierChanged, { "player": this.playerID, "type": "time" });
+};
+
+Player.prototype.SetTradeRateMultiplier = function(value)
+{
+	this.tradeRateMultiplier = value;
+};
+
 Player.prototype.GetPanelEntities = function()
 {
 	return this.panelEntities;
@@ -395,7 +451,13 @@ Player.prototype.GetState = function()
 	return this.state;
 };
 
-Player.prototype.SetState = function(newState, resign)
+/**
+ * @param {string} newState - Either "defeated" or "won".
+ * @param {string|undefined} message - A string to be shown in chat, for example
+ *     markForTranslation("%(player)s has been defeated (failed objective).").
+ *     If it is undefined, the caller MUST send that GUI notification manually.
+ */
+Player.prototype.SetState = function(newState, message)
 {
 	if (this.state != "active")
 		return;
@@ -435,20 +497,18 @@ Player.prototype.SetState = function(newState, resign)
 			});
 	}
 
-	Engine.BroadcastMessage(won ? MT_PlayerWon : MT_PlayerDefeated, { "playerId": this.playerID });
+	Engine.PostMessage(this.entity, won ? MT_PlayerWon : MT_PlayerDefeated, { "playerId": this.playerID });
 
-	let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	if (won)
+	if (message)
+	{
+		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		cmpGUIInterface.PushNotification({
-			"type": "won",
-			"players": [this.playerID]
-		});
-	else
-		cmpGUIInterface.PushNotification({
-			"type": "defeat",
+			"type": won ? "won" : "defeat",
 			"players": [this.playerID],
-			"resign": resign
+			"allies": [this.playerID],
+			"message": message
 		});
+	}
 };
 
 Player.prototype.GetTeam = function()
@@ -464,9 +524,10 @@ Player.prototype.SetTeam = function(team)
 	this.team = team;
 
 	// Set all team members as allies
-	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	if (cmpPlayerManager && this.team != -1)
-		for (let i = 0; i < cmpPlayerManager.GetNumPlayers(); ++i)
+	if (this.team != -1)
+	{
+		let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
+		for (let i = 0; i < numPlayers; ++i)
 		{
 			let cmpPlayer = QueryPlayerIDInterface(i);
 			if (this.team != cmpPlayer.GetTeam())
@@ -475,6 +536,7 @@ Player.prototype.SetTeam = function(team)
 			this.SetAlly(i);
 			cmpPlayer.SetAlly(this.playerID);
 		}
+	}
 
 	Engine.BroadcastMessage(MT_DiplomacyChanged, {
 		"player": this.playerID,
@@ -763,7 +825,7 @@ Player.prototype.OnValueModification = function(msg)
 		return;
 
 	if (msg.valueNames.indexOf("Player/SpyCostMultiplier") != -1)
-		this.spyCostMultiplier = ApplyValueModificationsToPlayer("Player/SpyCostMultiplier", +this.template.SpyCostMultiplier, this.entity, this.playerID);
+		this.spyCostMultiplier = ApplyValueModificationsToEntity("Player/SpyCostMultiplier", +this.template.SpyCostMultiplier, this.entity);
 
 	if (msg.valueNames.some(mod => mod.startsWith("Player/BarterMultiplier/")))
 		for (let res in this.template.BarterMultiplier.Buy)
@@ -786,6 +848,7 @@ Player.prototype.GetCheatsEnabled = function()
 Player.prototype.SetCheatTimeMultiplier = function(time)
 {
 	this.cheatTimeMultiplier = time;
+	Engine.BroadcastMessage(MT_MultiplierChanged, { "player": this.playerID, "type": "cheat" });
 };
 
 Player.prototype.GetCheatTimeMultiplier = function()
