@@ -8,14 +8,14 @@ ProductionQueue.prototype.Schema =
 	"<a:example>" +
 		"<BatchTimeModifier>0.7</BatchTimeModifier>" +
 		"<Entities datatype='tokens'>" +
-			"\n    units/{civ}_support_female_citizen\n    units/{civ}_support_trader\n    units/athen_infantry_spearman_b\n  " +
+			"\n    units/{civ}_support_female_citizen\n    units/{native}_support_trader\n    units/athen_infantry_spearman_b\n  " +
 		"</Entities>" +
 	"</a:example>" +
 	"<element name='BatchTimeModifier' a:help='Modifier that influences the time benefit for batch training'>" +
 		"<ref name='nonNegativeDecimal'/>" +
 	"</element>" +
 	"<optional>" +
-		"<element name='Entities' a:help='Space-separated list of entity template names that this building can train. The special string \"{civ}\" will be automatically replaced by the building&apos;s four-character civ code'>" +
+		"<element name='Entities' a:help='Space-separated list of entity template names that this entity can train. The special string \"{civ}\" will be automatically replaced by the civ code of the entity&apos;s owner, while the string \"{native}\" will be automatically replaced by the entity&apos;s civ code.'>" +
 			"<attribute name='datatype'>" +
 				"<value>tokens</value>" +
 			"</attribute>" +
@@ -23,7 +23,7 @@ ProductionQueue.prototype.Schema =
 		"</element>" +
 	"</optional>" +
 	"<optional>" +
-		"<element name='Technologies' a:help='Space-separated list of technology names that this building can research.'>" +
+		"<element name='Technologies' a:help='Space-separated list of technology names that this building can research. When present, the special string \"{civ}\" will be automatically replaced either by the civ code of the building&apos;s owner if such a tech exists, or by \"generic\".'>" +
 			"<attribute name='datatype'>" +
 				"<value>tokens</value>" +
 			"</attribute>" +
@@ -68,18 +68,6 @@ ProductionQueue.prototype.Init = function()
 
 	this.entityCache = [];
 	this.spawnNotified = false;
-
-	this.alertRaiser = undefined;
-};
-
-ProductionQueue.prototype.PutUnderAlert = function(raiser)
-{
-	this.alertRaiser = raiser;
-};
-
-ProductionQueue.prototype.ResetAlert = function()
-{
-	this.alertRaiser = undefined;
 };
 
 /*
@@ -96,31 +84,33 @@ ProductionQueue.prototype.CalculateEntitiesList = function()
 	if (!this.template.Entities)
 		return;
 
-	var string = this.template.Entities._string;
+	let string = this.template.Entities._string;
 	if (!string)
 		return;
 
-	// Replace the "{civ}" codes with this entity's civ ID
-	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var cmpPlayer = QueryOwnerInterface(this.entity);
+	// Replace the "{civ}" and "{native}" codes with the owner's civ ID and entity's civ ID.
+	let cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+	let cmpPlayer = QueryOwnerInterface(this.entity);
 	if (!cmpPlayer)
 		return;
 
-	var entitiesList = string.replace(/\{civ\}/g, cmpPlayer.GetCiv()).split(/\s+/);
+	let cmpIdentity = Engine.QueryInterface(this.entity, IID_Identity);
+	if (cmpIdentity)
+		string = string.replace(/\{native\}/g, cmpIdentity.GetCiv());
+
+	let entitiesList = string.replace(/\{civ\}/g, cmpPlayer.GetCiv()).split(/\s+/);
 
 	// filter out disabled and invalid entities
-	var disabledEntities = cmpPlayer.GetDisabledTemplates();
-	entitiesList = entitiesList.filter(
-		function(v) { return !disabledEntities[v] && cmpTemplateManager.TemplateExists(v); }
-	);
+	let disabledEntities = cmpPlayer.GetDisabledTemplates();
+	entitiesList = entitiesList.filter(ent => !disabledEntities[ent] && cmpTemplateManager.TemplateExists(ent));
 
 	// check if some templates need to show their advanced or elite version
-	var upgradeTemplate = function(templateName)
+	let upgradeTemplate = function(templateName)
 	{
-		var template = cmpTemplateManager.GetTemplate(templateName);
+		let template = cmpTemplateManager.GetTemplate(templateName);
 		while (template && template.Promotion !== undefined)
 		{
-			var requiredXp = ApplyValueModificationsToTemplate("Promotion/RequiredXp", +template.Promotion.RequiredXp, cmpPlayer.GetPlayerID(), template);
+			let requiredXp = ApplyValueModificationsToTemplate("Promotion/RequiredXp", +template.Promotion.RequiredXp, cmpPlayer.GetPlayerID(), template);
 			if (requiredXp > 0)
 				break;
 			templateName = template.Promotion.Entity;
@@ -160,11 +150,21 @@ ProductionQueue.prototype.GetTechnologiesList = function()
 
 	var techs = string.split(/\s+/);
 
+	// Replace the civ specific technologies
+	for (let i = 0; i < techs.length; ++i)
+	{
+		let tech = techs[i];
+		if (tech.indexOf("{civ}") == -1)
+			continue;
+		let civTech = tech.replace("{civ}", cmpPlayer.GetCiv());
+		techs[i] = TechnologyTemplates.Has(civTech) ? civTech : tech.replace("{civ}", "generic");
+	}
+
 	// Remove any technologies that can't be researched by this civ
-	techs = techs.filter(tech => {
-		let reqs = DeriveTechnologyRequirements(cmpTechnologyManager.GetTechnologyTemplate(tech), cmpPlayer.GetCiv());
-		return cmpTechnologyManager.CheckTechnologyRequirements(reqs, true);
-	});
+	techs = techs.filter(tech =>
+		cmpTechnologyManager.CheckTechnologyRequirements(
+			DeriveTechnologyRequirements(TechnologyTemplates.Get(tech), cmpPlayer.GetCiv()),
+			true));
 
 	var techList = [];
 	var superseded = {}; // Stores the tech which supersedes the key
@@ -178,7 +178,8 @@ ProductionQueue.prototype.GetTechnologiesList = function()
 		var tech = techs[i];
 		if (disabledTechnologies && disabledTechnologies[tech])
 			continue;
-		var template = cmpTechnologyManager.GetTechnologyTemplate(tech);
+
+		let template = TechnologyTemplates.Get(tech);
 		if (!template.supersedes || techs.indexOf(template.supersedes) === -1)
 			techList.push(tech);
 		else
@@ -190,9 +191,7 @@ ProductionQueue.prototype.GetTechnologiesList = function()
 	{
 		var tech = techList[i];
 		while (this.IsTechnologyResearchedOrInProgress(tech))
-		{
 			tech = superseded[tech];
-		}
 
 		techList[i] = tech;
 	}
@@ -209,9 +208,9 @@ ProductionQueue.prototype.GetTechnologiesList = function()
 			continue;
 		}
 
-		var template = cmpTechnologyManager.GetTechnologyTemplate(tech);
+		let template = TechnologyTemplates.Get(tech);
 		if (template.top)
-			ret[i] = {"pair": true, "top": template.top, "bottom": template.bottom};
+			ret[i] = { "pair": true, "top": template.top, "bottom": template.bottom };
 		else
 			ret[i] = tech;
 	}
@@ -234,16 +233,12 @@ ProductionQueue.prototype.IsTechnologyResearchedOrInProgress = function(tech)
 
 	var cmpTechnologyManager = QueryOwnerInterface(this.entity, IID_TechnologyManager);
 
-	var template = cmpTechnologyManager.GetTechnologyTemplate(tech);
+	let template = TechnologyTemplates.Get(tech);
 	if (template.top)
-	{
-		return (cmpTechnologyManager.IsTechnologyResearched(template.top) || cmpTechnologyManager.IsInProgress(template.top)
-			|| cmpTechnologyManager.IsTechnologyResearched(template.bottom) || cmpTechnologyManager.IsInProgress(template.bottom));
-	}
-	else
-	{
-		return (cmpTechnologyManager.IsTechnologyResearched(tech) || cmpTechnologyManager.IsInProgress(tech));
-	}
+		return cmpTechnologyManager.IsTechnologyResearched(template.top) || cmpTechnologyManager.IsInProgress(template.top) ||
+		       cmpTechnologyManager.IsTechnologyResearched(template.bottom) || cmpTechnologyManager.IsInProgress(template.bottom);
+
+	return cmpTechnologyManager.IsTechnologyResearched(tech) || cmpTechnologyManager.IsInProgress(tech);
 };
 
 /*
@@ -299,7 +294,7 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 				totalCosts[res] = Math.floor(count * costs[res]);
 			}
 
-			var population = ApplyValueModificationsToTemplate("Cost/Population",  +template.Cost.Population, cmpPlayer.GetPlayerID(), template);
+			var population = ApplyValueModificationsToTemplate("Cost/Population", +template.Cost.Population, cmpPlayer.GetPlayerID(), template);
 
 			// TrySubtractResources should report error to player (they ran out of resources)
 			if (!cmpPlayer.TrySubtractResources(totalCosts))
@@ -328,14 +323,11 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 
 			// Call the related trigger event
 			var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
-			cmpTrigger.CallEvent("TrainingQueued", {"playerid": cmpPlayer.GetPlayerID(), "unitTemplate": templateName, "count": count, "metadata": metadata, "trainerEntity": this.entity});
+			cmpTrigger.CallEvent("TrainingQueued", { "playerid": cmpPlayer.GetPlayerID(), "unitTemplate": templateName, "count": count, "metadata": metadata, "trainerEntity": this.entity });
 		}
 		else if (type == "technology")
 		{
-			// Load the technology template
-			var cmpDataTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager);
-			var template = cmpDataTemplateManager.GetTechnologyTemplate(templateName);
-			if (!template)
+			if (!TechnologyTemplates.Has(templateName))
 				return;
 
 			if (!this.GetTechnologiesList().some(tech =>
@@ -348,8 +340,9 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 				return;
 			}
 
+			let template = TechnologyTemplates.Get(templateName);
 			let techCostMultiplier = this.GetTechCostMultiplier();
-			let time =  techCostMultiplier.time * template.researchTime * cmpPlayer.GetCheatTimeMultiplier();
+			let time = techCostMultiplier.time * template.researchTime * cmpPlayer.GetTimeMultiplier();
 
 			let cost = {};
 			for (let res in template.cost)
@@ -379,7 +372,7 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 
 			// Call the related trigger event
 			var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
-			cmpTrigger.CallEvent("ResearchQueued", {"playerid": cmpPlayer.GetPlayerID(), "technologyTemplate": templateName, "researcherEntity": this.entity});
+			cmpTrigger.CallEvent("ResearchQueued", { "playerid": cmpPlayer.GetPlayerID(), "technologyTemplate": templateName, "researcherEntity": this.entity });
 		}
 		else
 		{
@@ -398,7 +391,7 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 	}
 	else
 	{
-		var notification = {"players": [cmpPlayer.GetPlayerID()], "message": markForTranslation("The production queue is full."), "translateMessage": true };
+		var notification = { "players": [cmpPlayer.GetPlayerID()], "message": markForTranslation("The production queue is full."), "translateMessage": true };
 		var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		cmpGUIInterface.PushNotification(notification);
 	}
@@ -411,10 +404,9 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 ProductionQueue.prototype.RemoveBatch = function(id)
 {
 	// Destroy any cached entities (those which didn't spawn for some reason)
-	for (var i = 0; i < this.entityCache.length; ++i)
-	{
-		Engine.DestroyEntity(this.entityCache[i]);
-	}
+	for (let ent of this.entityCache)
+		Engine.DestroyEntity(ent);
+
 	this.entityCache = [];
 
 	for (var i = 0; i < this.queue.length; ++i)
@@ -487,7 +479,7 @@ ProductionQueue.prototype.GetQueue = function()
 			"technologyTemplate": item.technologyTemplate,
 			"count": item.count,
 			"neededSlots": item.neededSlots,
-			"progress": 1 - ( item.timeRemaining / (item.timeTotal || 1) ),
+			"progress": 1 - (item.timeRemaining / (item.timeTotal || 1)),
 			"timeRemaining": item.timeRemaining,
 			"metadata": item.metadata,
 		});
@@ -518,19 +510,19 @@ ProductionQueue.prototype.GetBatchTime = function(batchSize)
 	var batchTimeModifier = ApplyValueModificationsToEntity("ProductionQueue/BatchTimeModifier", +this.template.BatchTimeModifier, this.entity);
 
 	// TODO: work out what equation we should use here.
-	return Math.pow(batchSize, batchTimeModifier) * cmpPlayer.GetCheatTimeMultiplier();
+	return Math.pow(batchSize, batchTimeModifier) * cmpPlayer.GetTimeMultiplier();
 };
 
 ProductionQueue.prototype.OnOwnershipChanged = function(msg)
 {
-	if (msg.from != -1)
+	if (msg.from != INVALID_PLAYER)
 	{
 		// Unset flag that previous owner's training may be blocked
 		var cmpPlayer = QueryPlayerIDInterface(msg.from);
 		if (cmpPlayer && this.queue.length > 0)
 			cmpPlayer.UnBlockTraining();
 	}
-	if (msg.to != -1)
+	if (msg.to != INVALID_PLAYER)
 		this.CalculateEntitiesList();
 
 	// Reset the production queue whenever the owner changes.
@@ -539,8 +531,6 @@ ProductionQueue.prototype.OnOwnershipChanged = function(msg)
 	// created from it. Also it means we don't have to worry about
 	// updating the reserved pop slots.)
 	this.ResetQueue();
-
-	this.ResetAlert();
 };
 
 ProductionQueue.prototype.OnCivChanged = function()
@@ -591,52 +581,60 @@ ProductionQueue.prototype.SpawnUnits = function(templateName, count, metadata)
 			{
 				var unitCategory = cmpTrainingRestrictions.GetCategory();
 				var cmpPlayerEntityLimits = QueryOwnerInterface(this.entity, IID_EntityLimits);
-				cmpPlayerEntityLimits.ChangeCount(unitCategory,-1);
+				cmpPlayerEntityLimits.ChangeCount(unitCategory, -1);
 			}
 		}
 	}
 
-	var cmpAutoGarrison = undefined;
+	let cmpAutoGarrison;
 	if (cmpRallyPoint)
 	{
-		var data = cmpRallyPoint.GetData()[0];
+		let data = cmpRallyPoint.GetData()[0];
 		if (data && data.target && data.target == this.entity && data.command == "garrison")
 			cmpAutoGarrison = Engine.QueryInterface(this.entity, IID_GarrisonHolder);
 	}
 
-	for (var i = 0; i < count; ++i)
+	for (let i = 0; i < count; ++i)
 	{
-		var ent = this.entityCache[0];
+		let ent = this.entityCache[0];
+		let cmpNewOwnership = Engine.QueryInterface(ent, IID_Ownership);
+		let garrisoned = false;
 
-		if (cmpAutoGarrison && cmpAutoGarrison.PerformGarrison(ent))
+		if (cmpAutoGarrison)
+		{
+			// Temporary owner affectation needed for GarrisonHolder checks
+			cmpNewOwnership.SetOwnerQuiet(cmpOwnership.GetOwner());
+			garrisoned = cmpAutoGarrison.PerformGarrison(ent);
+			cmpNewOwnership.SetOwnerQuiet(INVALID_PLAYER);
+		}
+
+		if (garrisoned)
 		{
-			let cmpNewOwnership = Engine.QueryInterface(ent, IID_Ownership);
-			cmpNewOwnership.SetOwner(cmpOwnership.GetOwner());
 			let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-			cmpUnitAI.Autogarrison(this.entity);
+			if (cmpUnitAI)
+				cmpUnitAI.Autogarrison(this.entity);
 		}
 		else
 		{
-			var pos = cmpFootprint.PickSpawnPoint(ent);
+			let pos = cmpFootprint.PickSpawnPoint(ent);
 			if (pos.y < 0)
-			{
-				// Fail: there wasn't any space to spawn the unit
 				break;
-			}
-			else
-			{
-				// Successfully spawned
-				let cmpNewOwnership = Engine.QueryInterface(ent, IID_Ownership);
-				cmpNewOwnership.SetOwner(cmpOwnership.GetOwner());
-				let cmpNewPosition = Engine.QueryInterface(ent, IID_Position);
-				cmpNewPosition.JumpTo(pos.x, pos.z);
-				// TODO: what direction should they face in?
-				spawnedEnts.push(ent);
-			}
+
+			let cmpNewPosition = Engine.QueryInterface(ent, IID_Position);
+			cmpNewPosition.JumpTo(pos.x, pos.z);
+
+			let cmpPosition = Engine.QueryInterface(this.entity, IID_Position);
+			if (cmpPosition)
+				cmpNewPosition.SetYRotation(cmpPosition.GetPosition().horizAngleTo(pos));
+
+			spawnedEnts.push(ent);
 		}
 
-		var cmpPlayerStatisticsTracker = QueryOwnerInterface(this.entity, IID_StatisticsTracker);
-		cmpPlayerStatisticsTracker.IncreaseTrainedUnitsCounter(ent);
+		cmpNewOwnership.SetOwner(cmpOwnership.GetOwner());
+
+		let cmpPlayerStatisticsTracker = QueryOwnerInterface(this.entity, IID_StatisticsTracker);
+		if (cmpPlayerStatisticsTracker)
+			cmpPlayerStatisticsTracker.IncreaseTrainedUnitsCounter(ent);
 
 		// Play a sound, but only for the first in the batch (to avoid nasty phasing effects)
 		if (createdEnts.length == 0)
@@ -663,21 +661,12 @@ ProductionQueue.prototype.SpawnUnits = function(templateName, count, metadata)
 	}
 
 	if (createdEnts.length > 0)
-	{
 		Engine.PostMessage(this.entity, MT_TrainingFinished, {
 			"entities": createdEnts,
 			"owner": cmpOwnership.GetOwner(),
 			"metadata": metadata,
 		});
 
-		if (this.alertRaiser && spawnedEnts.length > 0)
-		{
-			var cmpAlertRaiser = Engine.QueryInterface(this.alertRaiser, IID_AlertRaiser);
-			if (cmpAlertRaiser)
-				cmpAlertRaiser.UpdateUnits(spawnedEnts);
-		}
-	}
-
 	return createdEnts.length;
 };
 
@@ -706,7 +695,7 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 			{
 				// If something change population cost
 				var template = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager).GetTemplate(item.unitTemplate);
-				item.population = ApplyValueModificationsToTemplate("Cost/Population",  +template.Cost.Population, item.player, template);
+				item.population = ApplyValueModificationsToTemplate("Cost/Population", +template.Cost.Population, item.player, template);
 
 				// Batch's training hasn't started yet.
 				// Try to reserve the necessary population slots
@@ -734,7 +723,7 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 
 			item.productionStarted = true;
 			if (item.unitTemplate)
-				Engine.PostMessage(this.entity, MT_TrainingStarted, {"entity": this.entity});
+				Engine.PostMessage(this.entity, MT_TrainingStarted, { "entity": this.entity });
 		}
 
 		// If we won't finish the batch now, just update its timer
@@ -790,7 +779,7 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 			var cmpTechnologyManager = QueryOwnerInterface(this.entity, IID_TechnologyManager);
 			cmpTechnologyManager.ResearchTechnology(item.technologyTemplate);
 
-			var template = cmpTechnologyManager.GetTechnologyTemplate(item.technologyTemplate);
+			let template = TechnologyTemplates.Get(item.technologyTemplate);
 
 			if (template && template.soundComplete)
 			{
