@@ -69,9 +69,7 @@ ResourceGatherer.prototype.GetCarryingStatus = function()
 ResourceGatherer.prototype.GiveResources = function(resources)
 {
 	for (let resource of resources)
-	{
-		this.carrying[resource.type] = +(resource.amount);
-	}
+		this.carrying[resource.type] = +resource.amount;
 
 	Engine.PostMessage(this.entity, MT_ResourceCarryingChanged, { "to": this.GetCarryingStatus() });
 };
@@ -98,8 +96,8 @@ ResourceGatherer.prototype.GetLastCarriedType = function()
 {
 	if (this.lastCarriedType && this.lastCarriedType.generic in this.carrying)
 		return this.lastCarriedType;
-	else
-		return undefined;
+
+	return undefined;
 };
 
 // Since this code is very performancecritical and applying technologies quite slow, cache it.
@@ -179,7 +177,7 @@ ResourceGatherer.prototype.TryInstantGather = function(target)
 
 	let cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
 	if (cmpTrigger && cmpPlayer)
-		cmpTrigger.CallEvent("TreasureCollected", {"player": cmpPlayer.GetPlayerID(), "type": type.specific, "amount": status.amount });
+		cmpTrigger.CallEvent("TreasureCollected", { "player": cmpPlayer.GetPlayerID(), "type": type.specific, "amount": status.amount });
 
 	return true;
 };
@@ -225,7 +223,7 @@ ResourceGatherer.prototype.PerformGather = function(target)
 	return {
 		"amount": status.amount,
 		"exhausted": status.exhausted,
-		"filled": (this.carrying[type.generic] >= this.GetCapacity(type.generic))
+		"filled": this.carrying[type.generic] >= this.GetCapacity(type.generic)
 	};
 };
 
@@ -248,10 +246,6 @@ ResourceGatherer.prototype.GetTargetGatherRate = function(target)
 	if (rate == 0 && type.generic)
 		rate = this.GetGatherRate(type.generic);
 
-	let cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
-	let cheatMultiplier = cmpPlayer ? cmpPlayer.GetCheatTimeMultiplier() : 1;
-	rate = rate / cheatMultiplier;
-
 	if ("Mirages" in cmpResourceSupply)
 		return rate;
 
@@ -272,14 +266,14 @@ ResourceGatherer.prototype.GetTargetGatherRate = function(target)
  */
 ResourceGatherer.prototype.CanCarryMore = function(type)
 {
-	let amount = (this.carrying[type] || 0);
+	let amount = this.carrying[type] || 0;
 	return amount < this.GetCapacity(type);
 };
 
 
 ResourceGatherer.prototype.IsCarrying = function(type)
 {
-	let amount = (this.carrying[type] || 0);
+	let amount = this.carrying[type] || 0;
 	return amount > 0;
 };
 
@@ -341,7 +335,7 @@ ResourceGatherer.prototype.OnValueModification = function(msg)
 
 ResourceGatherer.prototype.OnOwnershipChanged = function(msg)
 {
-	if (msg.to === -1)
+	if (msg.to == INVALID_PLAYER)
 		return;
 	this.RecalculateGatherRatesAndCapacities();
 };
@@ -351,4 +345,10 @@ ResourceGatherer.prototype.OnGlobalInitGame = function(msg)
 	this.RecalculateGatherRatesAndCapacities();
 };
 
+ResourceGatherer.prototype.OnMultiplierChanged = function(msg)
+{
+	if (msg.player == QueryOwnerInterface(this.entity, IID_Player).GetPlayerID())
+		this.RecalculateGatherRatesAndCapacities();
+};
+
 Engine.RegisterComponentType(IID_ResourceGatherer, "ResourceGatherer", ResourceGatherer);
