@@ -16,6 +16,7 @@ StatisticsTracker.prototype.Init = function()
 		"Hero",
 		"Siege",
 		"Ship",
+		"Domestic",
 		"Trader"
 	];
 	this.unitsTrained = {
@@ -28,9 +29,9 @@ StatisticsTracker.prototype.Init = function()
 		"Siege": 0,
 		"Ship": 0,
 		"Trader": 0,
+		"Domestic": 0,
 		"total": 0
 	};
-	this.domesticUnitsTrainedValue = 0;
 	this.unitsLost = {
 		"Infantry": 0,
 		"Worker": 0,
@@ -145,6 +146,8 @@ StatisticsTracker.prototype.Init = function()
 	this.lootCollected = 0;
 	this.peakPercentMapControlled = 0;
 	this.teamPeakPercentMapControlled = 0;
+	this.successfulBribes = 0;
+	this.failedBribes = 0;
 
 	let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
 	this.updateTimer = cmpTimer.SetInterval(
@@ -177,7 +180,6 @@ StatisticsTracker.prototype.GetStatistics = function()
 {
 	return {
 		"unitsTrained": this.unitsTrained,
-		"domesticUnitsTrainedValue": this.domesticUnitsTrainedValue,
 		"unitsLost": this.unitsLost,
 		"unitsLostValue": this.unitsLostValue,
 		"enemyUnitsKilled": this.enemyUnitsKilled,
@@ -205,7 +207,9 @@ StatisticsTracker.prototype.GetStatistics = function()
 		"percentMapControlled": this.GetPercentMapControlled(),
 		"teamPercentMapControlled": this.GetTeamPercentMapControlled(),
 		"peakPercentMapControlled": this.peakPercentMapControlled,
-		"teamPeakPercentMapControlled": this.teamPeakPercentMapControlled
+		"teamPeakPercentMapControlled": this.teamPeakPercentMapControlled,
+		"successfulBribes": this.successfulBribes,
+		"failedBribes": this.failedBribes
 	};
 };
 
@@ -269,11 +273,15 @@ StatisticsTracker.prototype.IncreaseTrainedUnitsCounter = function(trainedUnit)
 	for (let type of this.unitsClasses)
 		this.CounterIncrement(cmpUnitEntityIdentity, "unitsTrained", type);
 
-	++this.unitsTrained.total;
+	if (!cmpUnitEntityIdentity.HasClass("Domestic"))
+		++this.unitsTrained.total;
 
 	if (cmpUnitEntityIdentity.HasClass("Domestic") && costs)
-		for (let type in costs)
-			this.domesticUnitsTrainedValue += costs[type];
+	{
+		// Subtract costs for sheep/goats/pigs to get the net food gain/use for corralling
+		this.resourcesUsed.food -= costs.food;
+		this.resourcesGathered.food -= costs.food;
+	}
 
 };
 
@@ -303,13 +311,7 @@ StatisticsTracker.prototype.KilledEntity = function(targetEntity)
 	var cmpCost = Engine.QueryInterface(targetEntity, IID_Cost);
 	var costs = cmpCost && cmpCost.GetResourceCosts();
 
-	var cmpTargetOwnership = Engine.QueryInterface(targetEntity, IID_Ownership);
-
-	// Ignore gaia
-	if (cmpTargetOwnership.GetOwner() == 0)
-		return;
-
-	if (cmpTargetEntityIdentity.HasClass("Unit") && !cmpTargetEntityIdentity.HasClass("Domestic"))
+	if (cmpTargetEntityIdentity.HasClass("Unit") && !cmpTargetEntityIdentity.HasClass("Animal"))
 	{
 		for (let type of this.unitsClasses)
 			this.CounterIncrement(cmpTargetEntityIdentity, "enemyUnitsKilled", type);
@@ -462,11 +464,23 @@ StatisticsTracker.prototype.IncreaseTradeIncomeCounter = function(amount)
 	this.tradeIncome += amount;
 };
 
+StatisticsTracker.prototype.IncreaseSuccessfulBribesCounter = function()
+{
+	++this.successfulBribes;
+};
+
+StatisticsTracker.prototype.IncreaseFailedBribesCounter = function()
+{
+	++this.failedBribes;
+};
+
 StatisticsTracker.prototype.GetPercentMapExplored = function()
 {
-	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-	var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
-	return cmpRangeManager.GetPercentMapExplored(cmpPlayer.GetPlayerID());
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+	if (!cmpPlayer)
+		return 0;
+
+	return Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager).GetPercentMapExplored(cmpPlayer.GetPlayerID());
 };
 
 /**
@@ -475,20 +489,19 @@ StatisticsTracker.prototype.GetPercentMapExplored = function()
  */
 StatisticsTracker.prototype.GetTeamPercentMapExplored = function()
 {
-	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-
-	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
 	if (!cmpPlayer)
 		return 0;
 
-	var team = cmpPlayer.GetTeam();
+	let team = cmpPlayer.GetTeam();
+	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	// If teams are not locked, this statistic won't be displayed, so don't bother computing
 	if (team == -1 || !cmpPlayer.GetLockTeams())
 		return cmpRangeManager.GetPercentMapExplored(cmpPlayer.GetPlayerID());
 
-	var teamPlayers = [];
-	for (var i = 1; i < cmpPlayerManager.GetNumPlayers(); ++i)
+	let teamPlayers = [];
+	let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
+	for (let i = 1; i < numPlayers; ++i)
 	{
 		let cmpOtherPlayer = QueryPlayerIDInterface(i);
 		if (cmpOtherPlayer && cmpOtherPlayer.GetTeam() == team)
@@ -500,28 +513,27 @@ StatisticsTracker.prototype.GetTeamPercentMapExplored = function()
 
 StatisticsTracker.prototype.GetPercentMapControlled = function()
 {
-	var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
-	var cmpTerritoryManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TerritoryManager);
-	if (!cmpPlayer || !cmpTerritoryManager)
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+	if (!cmpPlayer)
 		return 0;
 
-	return cmpTerritoryManager.GetTerritoryPercentage(cmpPlayer.GetPlayerID());
+	return Engine.QueryInterface(SYSTEM_ENTITY, IID_TerritoryManager).GetTerritoryPercentage(cmpPlayer.GetPlayerID());
 };
 
 StatisticsTracker.prototype.GetTeamPercentMapControlled = function()
 {
-	var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
-	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	var cmpTerritoryManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TerritoryManager);
-	if (!cmpPlayer || !cmpTerritoryManager)
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+	if (!cmpPlayer)
 		return 0;
 
-	var team = cmpPlayer.GetTeam();
+	let team = cmpPlayer.GetTeam();
+	let cmpTerritoryManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TerritoryManager);
 	if (team == -1 || !cmpPlayer.GetLockTeams())
 		return cmpTerritoryManager.GetTerritoryPercentage(cmpPlayer.GetPlayerID());
 
-	var teamPercent = 0;
-	for (let i = 1; i < cmpPlayerManager.GetNumPlayers(); ++i)
+	let teamPercent = 0;
+	let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
+	for (let i = 1; i < numPlayers; ++i)
 	{
 		let cmpOtherPlayer = QueryPlayerIDInterface(i);
 		if (cmpOtherPlayer && cmpOtherPlayer.GetTeam() == team)
@@ -533,13 +545,23 @@ StatisticsTracker.prototype.GetTeamPercentMapControlled = function()
 
 StatisticsTracker.prototype.OnTerritoriesChanged = function(msg)
 {
-	var newPercent = this.GetPercentMapControlled();
-	if (newPercent > this.peakPercentMapControlled)
-		this.peakPercentMapControlled = newPercent;
+	this.UpdatePeakPercentages();
+};
 
-	newPercent = this.GetTeamPercentMapControlled();
-	if (newPercent > this.teamPeakPercentMapControlled)
-		this.teamPeakPercentMapControlled = newPercent;
+StatisticsTracker.prototype.OnGlobalPlayerDefeated = function(msg)
+{
+	this.UpdatePeakPercentages();
+};
+
+StatisticsTracker.prototype.OnGlobalPlayerWon = function(msg)
+{
+	this.UpdatePeakPercentages();
+};
+
+StatisticsTracker.prototype.UpdatePeakPercentages = function()
+{
+	this.peakPercentMapControlled = Math.max(this.peakPercentMapControlled, this.GetPercentMapControlled());
+	this.teamPeakPercentMapControlled = Math.max(this.teamPeakPercentMapControlled, this.GetTeamPercentMapControlled());
 };
 
 /**
