@@ -39,16 +39,7 @@ TriggerPoint.prototype.OnDestroy = function()
  */
 TriggerPoint.prototype.RegisterRangeTrigger = function(action, data)
 {
-
-	if (data.players)
-		var players = data.players;
-	else
-	{
-		var numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
-		var players = [];
-		for (var i = 0; i < numPlayers; i++)
-			players.push(i);
-	}
+	var players = data.players || Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetAllPlayers();
 	var minRange = data.minRange || 0;
 	var maxRange = data.maxRange || -1;
 	var cid = data.requiredComponent || -1;
@@ -56,7 +47,7 @@ TriggerPoint.prototype.RegisterRangeTrigger = function(action, data)
 	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	var tag = cmpRangeManager.CreateActiveQuery(this.entity, minRange, maxRange, players, cid, cmpRangeManager.GetEntityFlagMask("normal"));
 
-    this.currentCollections[tag] = [];
+	this.currentCollections[tag] = [];
 	this.actions[tag] = action;
 	return tag;
 };
