@@ -2,17 +2,34 @@ Engine.LoadHelperScript("Player.js");
 Engine.LoadHelperScript("ValueModification.js");
 Engine.LoadComponentScript("interfaces/Auras.js");
 Engine.LoadComponentScript("interfaces/AuraManager.js");
+Engine.LoadComponentScript("interfaces/RangeOverlayManager.js");
 Engine.LoadComponentScript("interfaces/TechnologyManager.js");
 Engine.LoadComponentScript("Auras.js");
 Engine.LoadComponentScript("AuraManager.js");
 
-let playerID = [0, 1, 2];
-let playerEnt = [10, 11, 12];
-let playerState = "active";
-let sourceEnt = 20;
-let targetEnt = 30;
-let auraRange = 40;
-let template = { "Identity" : { "Classes" : { "_string" : "CorrectClass OtherClass" } } };
+var playerID = [0, 1, 2];
+var playerEnt = [10, 11, 12];
+var playerState = ["active", "active", "active"];
+var sourceEnt = 20;
+var targetEnt = 30;
+var auraRange = 40;
+var template = { "Identity": { "Classes": { "_string": "CorrectClass OtherClass" } } };
+
+global.AuraTemplates = {
+	"Get": name => {
+		let template = {
+			"type": name,
+			"affectedPlayers": ["Ally"],
+			"affects": ["CorrectClass"],
+			"modifications": [{ "value": "Component/Value", "add": 10 }],
+			"auraName": "name",
+			"auraDescription": "description"
+		};
+		if (name == "range")
+			template.radius = auraRange;
+		return template;
+	}
+};
 
 function testAuras(name, test_function)
 {
@@ -20,7 +37,8 @@ function testAuras(name, test_function)
 
 	AddMock(SYSTEM_ENTITY, IID_PlayerManager, {
 		"GetPlayerByID": idx => playerEnt[idx],
-		"GetNumPlayers": () => 3
+		"GetNumPlayers": () => 3,
+		"GetAllPlayers": () => playerID
 	});
 
 	AddMock(SYSTEM_ENTITY, IID_RangeManager, {
@@ -33,34 +51,18 @@ function testAuras(name, test_function)
 		"GetEntitiesByPlayer": id => [30, 31, 32]
 	});
 
-	AddMock(SYSTEM_ENTITY, IID_DataTemplateManager, {
-		"GetAuraTemplate": (name) => {
-			let template = {
-				"type": name,
-				"affectedPlayers": ["Ally"],
-				"affects": ["CorrectClass"],
-				"modifications": [{ "value": "Component/Value", "add": 10 }],
-				"auraName": "name",
-				"auraDescription": "description"
-			};
-			if (name == "range")
-				template.radius = auraRange;
-			return template;
-		}
-	});
-
 	AddMock(playerEnt[1], IID_Player, {
 		"IsAlly": id => id == playerID[1] || id == playerID[2],
-		"IsEnemy": id => id !=  playerID[1] || id != playerID[2],
+		"IsEnemy": id => id != playerID[1] || id != playerID[2],
 		"GetPlayerID": () => playerID[1],
-		"GetState": () => playerState
+		"GetState": () => playerState[1]
 	});
 
 	AddMock(playerEnt[2], IID_Player, {
 		"IsAlly": id => id == playerID[1] || id == playerID[2],
 		"IsEnemy": id => id != playerID[1] || id != playerID[2],
 		"GetPlayerID": () => playerID[2],
-		"GetState": () => playerState
+		"GetState": () => playerState[2]
 	});
 
 	AddMock(targetEnt, IID_Identity, {
@@ -92,15 +94,9 @@ function testAuras(name, test_function)
 	test_function(name, cmpAuras);
 }
 
-testAuras("global", (name, cmpAuras) => {
-	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 15);
-	TS_ASSERT_EQUALS(ApplyValueModificationsToTemplate("Component/Value", 5, playerID[1], template), 15);
-});
-
 targetEnt = playerEnt[playerID[2]];
 testAuras("player", (name, cmpAuras) => {
 	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 15);
-	TS_ASSERT_EQUALS(ApplyValueModificationsToTemplate("Component/Value", 5, playerID[1], template), 15);
 });
 targetEnt = 30;
 
@@ -121,9 +117,9 @@ testAuras("range", (name, cmpAuras) => {
 });
 
 testAuras("garrisonedUnits", (name, cmpAuras) => {
-	cmpAuras.OnGarrisonedUnitsChanged({ "added" : [targetEnt], "removed": [] });
+	cmpAuras.OnGarrisonedUnitsChanged({ "added": [targetEnt], "removed": [] });
 	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 15);
-	cmpAuras.OnGarrisonedUnitsChanged({ "added" : [], "removed": [targetEnt] });
+	cmpAuras.OnGarrisonedUnitsChanged({ "added": [], "removed": [targetEnt] });
 	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 5);
 });
 
@@ -132,7 +128,7 @@ testAuras("garrison", (name, cmpAuras) => {
 	cmpAuras.ApplyGarrisonBonus(targetEnt);
 	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 15);
 	cmpAuras.RemoveGarrisonBonus(targetEnt);
-	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 1, targetEnt), 1);
+	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 5);
 });
 
 testAuras("formation", (name, cmpAuras) => {
@@ -143,7 +139,21 @@ testAuras("formation", (name, cmpAuras) => {
 	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 5);
 });
 
-playerState = "defeated";
 testAuras("global", (name, cmpAuras) => {
+	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 15);
+	TS_ASSERT_EQUALS(ApplyValueModificationsToTemplate("Component/Value", 5, playerID[1], template), 15);
+	TS_ASSERT_EQUALS(ApplyValueModificationsToTemplate("Component/Value", 5, playerID[2], template), 15);
+	AddMock(sourceEnt, IID_Ownership, {
+		"GetOwner": () => -1
+	});
+	cmpAuras.OnOwnershipChanged({ "from": sourceEnt, "to": -1 });
+	TS_ASSERT_EQUALS(ApplyValueModificationsToEntity("Component/Value", 5, targetEnt), 5);
 	TS_ASSERT_EQUALS(ApplyValueModificationsToTemplate("Component/Value", 5, playerID[1], template), 5);
+	TS_ASSERT_EQUALS(ApplyValueModificationsToTemplate("Component/Value", 5, playerID[2], template), 5);
+});
+
+playerState[1] = "defeated";
+testAuras("global", (name, cmpAuras) => {
+	cmpAuras.OnGlobalPlayerDefeated({ "playerId": playerID[1] });
+	TS_ASSERT_EQUALS(ApplyValueModificationsToTemplate("Component/Value", 5, playerID[2], template), 5);
 });
