@@ -7,6 +7,7 @@ Engine.LoadComponentScript("interfaces/Builder.js");
 Engine.LoadComponentScript("interfaces/Capturable.js");
 Engine.LoadComponentScript("interfaces/CeasefireManager.js");
 Engine.LoadComponentScript("interfaces/DamageReceiver.js");
+Engine.LoadComponentScript("interfaces/DeathDamage.js");
 Engine.LoadComponentScript("interfaces/EndGameManager.js");
 Engine.LoadComponentScript("interfaces/EntityLimits.js");
 Engine.LoadComponentScript("interfaces/Foundation.js");
@@ -21,7 +22,6 @@ Engine.LoadComponentScript("interfaces/Market.js");
 Engine.LoadComponentScript("interfaces/Pack.js");
 Engine.LoadComponentScript("interfaces/ProductionQueue.js");
 Engine.LoadComponentScript("interfaces/Promotion.js");
-Engine.LoadComponentScript("interfaces/RallyPoint.js");
 Engine.LoadComponentScript("interfaces/Repairable.js");
 Engine.LoadComponentScript("interfaces/ResourceDropsite.js");
 Engine.LoadComponentScript("interfaces/ResourceGatherer.js");
@@ -65,7 +65,7 @@ AddMock(SYSTEM_ENTITY, IID_Barter, {
 });
 
 AddMock(SYSTEM_ENTITY, IID_EndGameManager, {
-	GetGameType: function() { return "conquest"; },
+	GetVictoryConditions: () => ["conquest", "wonder"],
 	GetAlliedVictory: function() { return false; }
 });
 
@@ -123,13 +123,12 @@ AddMock(100, IID_EntityLimits, {
 });
 
 AddMock(100, IID_TechnologyManager, {
-	IsTechnologyResearched: function(tech) { if (tech == "phase_village") return true; else return false; },
-	GetQueuedResearch: function() { return {}; },
-	GetStartedTechs: function() { return {}; },
-	GetResearchedTechs: function() { return {}; },
-	GetClassCounts: function() { return {}; },
-	GetTypeCountsByClass: function() { return {}; },
-	GetTechModifications: function() { return {}; }
+	"IsTechnologyResearched": tech => tech == "phase_village",
+	"GetQueuedResearch": () => new Map(),
+	"GetStartedTechs": () => new Set(),
+	"GetResearchedTechs": () => new Set(),
+	"GetClassCounts": () => ({}),
+	"GetTypeCountsByClass": () => ({})
 });
 
 AddMock(100, IID_StatisticsTracker, {
@@ -209,13 +208,12 @@ AddMock(101, IID_EntityLimits, {
 });
 
 AddMock(101, IID_TechnologyManager, {
-		IsTechnologyResearched: function(tech) { if (tech == "phase_village") return true; else return false; },
-		GetQueuedResearch: function() { return {}; },
-		GetStartedTechs: function() { return {}; },
-		GetResearchedTechs: function() { return {}; },
-		GetClassCounts: function() { return {}; },
-		GetTypeCountsByClass: function() { return {}; },
-		GetTechModifications: function() { return {}; }
+	"IsTechnologyResearched": tech => tech == "phase_village",
+	"GetQueuedResearch": () => new Map(),
+	"GetStartedTechs": () => new Set(),
+	"GetResearchedTechs": () => new Set(),
+	"GetClassCounts": () => ({}),
+	"GetTypeCountsByClass": () => ({})
 });
 
 AddMock(101, IID_StatisticsTracker, {
@@ -294,9 +292,9 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			entityLimits: {"Foo": 10},
 			entityCounts: {"Foo": 5},
 			entityLimitChangers: {"Foo": {}},
-			researchQueued: {},
-			researchStarted: {},
-			researchedTechs: {},
+			researchQueued: new Map(),
+			researchStarted: new Set(),
+			researchedTechs: new Set(),
 			classCounts: {},
 			typeCountsByClass: {},
 			canBarter: false,
@@ -343,9 +341,9 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			entityLimits: {"Bar": 20},
 			entityCounts: {"Bar": 0},
 			entityLimitChangers: {"Bar": {}},
-			researchQueued: {},
-			researchStarted: {},
-			researchedTechs: {},
+			researchQueued: new Map(),
+			researchStarted: new Set(),
+			researchedTechs: new Set(),
 			classCounts: {},
 			typeCountsByClass: {},
 			canBarter: false,
@@ -367,23 +365,8 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 	],
 	circularMap: false,
 	timeElapsed: 0,
-	gameType: "conquest",
-	alliedVictory: false,
-	"resources": {
-		"codes": ["food", "metal", "stone", "wood"],
-		"names": {
-			"food": "Food",
-			"metal": "Metal",
-			"stone": "Stone",
-			"wood": "Wood"
-		},
-		"aiInfluenceGroups": {
-			"food": "ignore",
-			"metal": "sparse",
-			"stone": "sparse",
-			"wood": "abundant"
-		}
-	}
+	"victoryConditions": ["conquest", "wonder"],
+	alliedVictory: false
 });
 
 TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
@@ -416,9 +399,9 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 			"entityLimits": {"Foo": 10},
 			"entityCounts": {"Foo": 5},
 			"entityLimitChangers": {"Foo": {}},
-			"researchQueued": {},
-			"researchStarted": {},
-			"researchedTechs": {},
+			"researchQueued": new Map(),
+			"researchStarted": new Set(),
+			"researchedTechs": new Set(),
 			"classCounts": {},
 			"typeCountsByClass": {},
 			"canBarter": false,
@@ -488,9 +471,9 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 			"entityLimits": {"Bar": 20},
 			"entityCounts": {"Bar": 0},
 			"entityLimitChangers": {"Bar": {}},
-			"researchQueued": {},
-			"researchStarted": {},
-			"researchedTechs": {},
+			"researchQueued": new Map(),
+			"researchStarted": new Set(),
+			"researchedTechs": new Set(),
 			"classCounts": {},
 			"typeCountsByClass": {},
 			"canBarter": false,
@@ -535,23 +518,8 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 	],
 	"circularMap": false,
 	"timeElapsed": 0,
-	"gameType": "conquest",
-	"alliedVictory": false,
-	"resources": {
-		"codes": ["food", "metal", "stone", "wood"],
-		"names": {
-			"food": "Food",
-			"metal": "Metal",
-			"stone": "Stone",
-			"wood": "Wood"
-		},
-		"aiInfluenceGroups": {
-			"food": "ignore",
-			"metal": "sparse",
-			"stone": "sparse",
-			"wood": "abundant"
-		}
-	}
+	"victoryConditions": ["conquest", "wonder"],
+	"alliedVictory": false
 });
 
 
@@ -565,8 +533,7 @@ AddMock(10, IID_Health, {
 	GetHitpoints: function() { return 50; },
 	GetMaxHitpoints: function() { return 60; },
 	IsRepairable: function() { return false; },
-	IsUnhealable: function() { return false; },
-	IsUndeletable: function() { return false; }
+	IsUnhealable: function() { return false; }
 });
 
 AddMock(10, IID_Identity, {
@@ -574,7 +541,8 @@ AddMock(10, IID_Identity, {
 	GetVisibleClassesList: function() { return ["class3", "class4"]; },
 	GetRank: function() { return "foo"; },
 	GetSelectionGroupName: function() { return "Selection Group Name"; },
-	HasClass: function() { return true; }
+	HasClass: function() { return true; },
+	IsUndeletable: function() { return false; }
 });
 
 AddMock(10, IID_Position, {
@@ -582,9 +550,6 @@ AddMock(10, IID_Position, {
 	GetPosition: function() {
 		return {x:1, y:2, z:3};
 	},
-	GetRotation: function() {
-		return {x:4, y:5, z:6};
-	},
 	IsInWorld: function() {
 		return true;
 	}
@@ -592,77 +557,34 @@ AddMock(10, IID_Position, {
 
 AddMock(10, IID_ResourceTrickle, {
 	"GetTimer": () => 1250,
-	"GetRates": () => ({
-		"food": 2,
-		"wood": 3,
-		"stone": 5,
-		"metal": 9
-	})
+	"GetRates": () => ({ "food": 2, "wood": 3, "stone": 5, "metal": 9 })
 });
 
 // Note: property order matters when using TS_ASSERT_UNEVAL_EQUALS,
 //	because uneval preserves property order. So make sure this object
 //	matches the ordering in GuiInterface.
 TS_ASSERT_UNEVAL_EQUALS(cmp.GetEntityState(-1, 10), {
-	id: 10,
-	template: "example",
-	alertRaiser: null,
-	builder: true,
-	canGarrison: false,
-	identity: {
-		rank: "foo",
-		classes: ["class1", "class2"],
-		visibleClasses: ["class3", "class4"],
-		selectionGroupName: "Selection Group Name"
+	"id": 10,
+	"player": INVALID_PLAYER,
+	"template": "example",
+	"identity": {
+		"rank": "foo",
+		"classes": ["class1", "class2"],
+		"visibleClasses": ["class3", "class4"],
+		"selectionGroupName": "Selection Group Name",
+		"canDelete": true
 	},
-	fogging: null,
-	foundation: null,
-	garrisonHolder: null,
-	gate: null,
-	guard: null,
-	market: null,
-	mirage: null,
-	pack: null,
-	upgrade: null,
-	player: -1,
-	position: {x:1, y:2, z:3},
-	production: null,
-	rallyPoint: null,
-	resourceCarrying: null,
-	rotation: {x:4, y:5, z:6},
-	trader: null,
-	unitAI: null,
-	visibility: "visible",
-	hitpoints: 50,
-	maxHitpoints: 60,
-	needsRepair: false,
-	needsHeal: true,
-	canDelete: true
-});
-
-TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedEntityState(-1, 10), {
-	armour: null,
-	attack: null,
-	buildingAI: null,
-	heal: null,
-	isBarterMarket: true,
-	loot: null,
-	obstruction: null,
-	turretParent: null,
-	promotion: null,
-	repairRate: null,
-	buildRate: null,
-	resourceDropsite: null,
-	resourceGatherRates: null,
-	resourceSupply: null,
-	resourceTrickle: {
+	"position": {x:1, y:2, z:3},
+	"hitpoints": 50,
+	"maxHitpoints": 60,
+	"needsRepair": false,
+	"needsHeal": true,
+	"builder": true,
+	"canGarrison": false,
+	"visibility": "visible",
+	"isBarterMarket":true,
+	"resourceTrickle": {
 		"interval": 1250,
-		"rates": {
-			"food": 2,
-			"wood": 3,
-			"stone": 5,
-			"metal": 9
-		}
-	},
-	speed: null
+		"rates": { "food": 2, "wood": 3, "stone": 5, "metal": 9 }
+	}
 });
