@@ -277,7 +277,7 @@ var g_ReadyInit = true;
 var g_ReadyChanged = 2;
 
 /**
- * Used to prevent calling resetReadyData when starting a game.
+ * Used to prevent calling resetReadyData when starting a game or doubleclicking on the "Start Game" button.
  */
 var g_GameStarted = false;
 
@@ -780,7 +780,7 @@ var g_PlayerDropdowns = {
 			g_GameAttributes.settings.PlayerData[playerIdx].Civ = g_PlayerCivList.code[selectedIdx];
 		},
 		"enabled": () => g_GameAttributes.mapType != "scenario",
-		"autocomplete": 0,
+		"autocomplete": 90,
 	},
 	"playerColorPicker": {
 		"labels": (playerIdx) => g_PlayerColorPickerList.map(color => "■"),
@@ -1011,6 +1011,16 @@ var g_MiscControls = {
 	"startGame": {
 		"caption": () =>
 			g_IsController ? translate("Start Game!") : g_ReadyData[g_IsReady].caption,
+		"onPress": () => function() {
+			if (g_IsController)
+				launchGame();
+			else
+				toggleReady();
+		},
+		"onPressRight": () => function() {
+			if (!g_IsController && g_IsReady)
+				setReady(0, true);
+		},
 		"tooltip": (hoverIdx) =>
 			!g_IsController ?
 				g_ReadyData[g_IsReady].tooltip :
@@ -1018,14 +1028,28 @@ var g_MiscControls = {
 					g_PlayerAssignments[guid].status || g_PlayerAssignments[guid].player == -1) ?
 					translate("Start a new game with the current settings.") :
 					translate("Start a new game with the current settings (disabled until all players are ready)"),
-		"enabled": () => !g_IsController ||
-		                 Object.keys(g_PlayerAssignments).every(guid => g_PlayerAssignments[guid].status ||
-		                                                                g_PlayerAssignments[guid].player == -1 ||
-		                                                                guid == Engine.GetPlayerGUID() && g_IsController),
+		"enabled": () => !g_GameStarted && (
+		                   !g_IsController ||
+		                   Object.keys(g_PlayerAssignments).every(guid => g_PlayerAssignments[guid].status ||
+		                                                                  g_PlayerAssignments[guid].player == -1 ||
+		                                                                  guid == Engine.GetPlayerGUID() && g_IsController)),
 		"hidden": () =>
 			!g_PlayerAssignments[Engine.GetPlayerGUID()] ||
 			g_PlayerAssignments[Engine.GetPlayerGUID()].player == -1 && !g_IsController,
 	},
+	"civInfoButton": {
+		"tooltip": () => sprintf(
+			translate("%(hotkey_civinfo)s / %(hotkey_structree)s: View History / Structure Tree\nLast opened will be reopened on click."), {
+				"hotkey_civinfo": colorizeHotkey("%(hotkey)s", "civinfo"),
+				"hotkey_structree": colorizeHotkey("%(hotkey)s", "structree")
+			}),
+		"onPress": () => function() {
+			Engine.PushGuiPage(g_CivInfo.page, {
+				"civ": g_CivInfo.code,
+				"callback": "storeCivInfoPage"
+			});
+		}
+	},
 	"civResetButton": {
 		"hidden": () => g_GameAttributes.mapType == "scenario" || !g_IsController,
 	},
@@ -1099,13 +1123,6 @@ function init(attribs)
 		return;
 	}
 
-	if (["offline", "server", "client"].indexOf(attribs.type) == -1)
-	{
-		error("Unexpected 'type' in gamesetup init: " + attribs.type);
-		cancelSetup();
-		return;
-	}
-
 	g_IsTutorial = !!attribs.tutorial;
 	g_ServerName = attribs.serverName;
 	g_ServerPort = attribs.serverPort;
@@ -1127,12 +1144,6 @@ function init(attribs)
 	supplementDefaults();
 
 	setTimeout(displayGamestateNotifications, 1000);
-
-	Engine.GetGUIObjectByName("civInfoButton").tooltip = sprintf(
-		translate("%(hotkey_civinfo)s / %(hotkey_structree)s: View History / Structure Tree\nLast opened will be reopened on click."), {
-			"hotkey_civinfo": colorizeHotkey("%(hotkey)s", "civinfo"),
-			"hotkey_structree": colorizeHotkey("%(hotkey)s", "structree")
-	});
 }
 
 function initDefaults()
@@ -1658,12 +1669,9 @@ function getMapDisplayName(map)
 
 function getMapPreview(map)
 {
-	if (g_GameAttributes.settings.Biome)
-	{
-		let biomePreview = basename(map) + "_" + basename(g_GameAttributes.settings.Biome) + ".png";
-		if (Engine.FileExists("art/textures/ui/session/icons/mappreview/" + biomePreview))
-			return biomePreview;
-	}
+	let biomePreview = g_GameAttributes.settings.Biome && getBiomePreview(map, g_GameAttributes.settings.Biome);
+	if (biomePreview)
+		return biomePreview;
 
 	let mapData = loadMapData(map);
 	if (!mapData || !mapData.settings || !mapData.settings.Preview)
@@ -2179,9 +2187,13 @@ function launchGame()
 		return;
 	}
 
-	if (!g_GameAttributes.map)
+	if (!g_GameAttributes.map || g_GameStarted)
 		return;
 
+	// Prevent reseting the readystate or calling this function twice
+	g_GameStarted = true;
+	updateGUIMiscControl("startGame");
+
 	savePersistMatchSettings();
 
 	// Select random map
@@ -2201,9 +2213,6 @@ function launchGame()
 
 	g_GameAttributes.settings.TriggerScripts = g_GameAttributes.settings.VictoryScripts.concat(g_GameAttributes.settings.TriggerScripts || []);
 
-	// Prevent reseting the readystate
-	g_GameStarted = true;
-
 	g_GameAttributes.settings.mapType = g_GameAttributes.mapType;
 
 	// Get a unique array of selectable cultures
