@@ -4,34 +4,34 @@ var g_DebugCommands = false;
 
 function ProcessCommand(player, cmd)
 {
+	let cmpPlayer = QueryPlayerIDInterface(player);
+	if (!cmpPlayer)
+		return;
+
 	let data = {
-		"cmpPlayerManager": Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager)
+		"cmpPlayer": cmpPlayer,
+		"controlAllUnits": cmpPlayer.CanControlAllUnits()
 	};
 
-	if (!data.cmpPlayerManager || player < 0)
-		return;
-
-	data.playerEnt = data.cmpPlayerManager.GetPlayerByID(player);
-
-	if (data.playerEnt == INVALID_ENTITY)
-		return;
-
-	data.cmpPlayer = Engine.QueryInterface(data.playerEnt, IID_Player);
-	if (!data.cmpPlayer)
-		return;
-
-	data.controlAllUnits = data.cmpPlayer.CanControlAllUnits();
-
 	if (cmd.entities)
 		data.entities = FilterEntityList(cmd.entities, player, data.controlAllUnits);
 
 	// Allow focusing the camera on recent commands
-	let cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGuiInterface.PushNotification({
+	let commandData = {
 		"type": "playercommand",
 		"players": [player],
 		"cmd": cmd
-	});
+	};
+
+	// Save the position, since the GUI event is received after the unit died
+	if (cmd.type == "delete-entities")
+	{
+		let cmpPosition = cmd.entities[0] && Engine.QueryInterface(cmd.entities[0], IID_Position);
+		commandData.position = cmpPosition && cmpPosition.IsInWorld() && cmpPosition.GetPosition2D();
+	}
+
+	let cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+	cmpGuiInterface.PushNotification(commandData);
 
 	// Note: checks of UnitAI targets are not robust enough here, as ownership
 	//	can change after the order is issued, they should be checked by UnitAI
@@ -152,6 +152,14 @@ var g_Commands = {
 		});
 	},
 
+	"walk-custom": function(player, cmd, data)
+	{
+		for (let ent in data.entities)
+			GetFormationUnitAIs([data.entities[ent]], player).forEach(cmpUnitAI => {
+				cmpUnitAI.Walk(cmd.targetPositions[ent].x, cmd.targetPositions[ent].y, cmd.queued);
+			});
+	},
+
 	"walk-to-range": function(player, cmd, data)
 	{
 		// Only used by the AI
@@ -165,11 +173,22 @@ var g_Commands = {
 
 	"attack-walk": function(player, cmd, data)
 	{
+		let allowCapture = cmd.allowCapture || cmd.allowCapture == null;
+
 		GetFormationUnitAIs(data.entities, player).forEach(cmpUnitAI => {
-			cmpUnitAI.WalkAndFight(cmd.x, cmd.z, cmd.targetClasses, cmd.queued);
+			cmpUnitAI.WalkAndFight(cmd.x, cmd.z, cmd.targetClasses, allowCapture, cmd.queued);
 		});
 	},
 
+	"attack-walk-custom": function(player, cmd, data)
+	{
+		let allowCapture = cmd.allowCapture || cmd.allowCapture == null;
+		for (let ent in data.entities)
+			GetFormationUnitAIs([data.entities[ent]], player).forEach(cmpUnitAI => {
+				cmpUnitAI.WalkAndFight(cmd.targetPositions[ent].x, cmd.targetPositions[ent].y, cmd.targetClasses, allowCapture, cmd.queued);
+			});
+	},
+
 	"attack": function(player, cmd, data)
 	{
 		let allowCapture = cmd.allowCapture || cmd.allowCapture == null;
@@ -179,14 +198,16 @@ var g_Commands = {
 			warn("Invalid command: attack target is not owned by enemy of player "+player+": "+uneval(cmd));
 
 		GetFormationUnitAIs(data.entities, player).forEach(cmpUnitAI => {
-			cmpUnitAI.Attack(cmd.target, cmd.queued, allowCapture);
+			cmpUnitAI.Attack(cmd.target, allowCapture, cmd.queued);
 		});
 	},
 
 	"patrol": function(player, cmd, data)
 	{
+		let allowCapture = cmd.allowCapture || cmd.allowCapture == null;
+
 		GetFormationUnitAIs(data.entities, player).forEach(cmpUnitAI =>
-			cmpUnitAI.Patrol(cmd.x, cmd.z, cmd.targetClasses, cmd.queued)
+			cmpUnitAI.Patrol(cmd.x, cmd.z, cmd.targetClasses, allowCapture, cmd.queued)
 		);
 	},
 
@@ -377,10 +398,10 @@ var g_Commands = {
 	{
 		for (let ent of data.entities)
 		{
-			let cmpHealth = QueryMiragedInterface(ent, IID_Health);
 			if (!data.controlAllUnits)
 			{
-				if (cmpHealth && cmpHealth.IsUndeletable())
+				let cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
+				if (cmpIdentity && cmpIdentity.IsUndeletable())
 					continue;
 
 				let cmpCapturable = QueryMiragedInterface(ent, IID_Capturable);
@@ -403,8 +424,11 @@ var g_Commands = {
 					Engine.DestroyEntity(cmpMirage.parent);
 
 				Engine.DestroyEntity(ent);
+				continue;
 			}
-			else if (cmpHealth)
+
+			let cmpHealth = Engine.QueryInterface(ent, IID_Health);
+			if (cmpHealth)
 				cmpHealth.Kill();
 			else
 				Engine.DestroyEntity(ent);
@@ -437,11 +461,9 @@ var g_Commands = {
 		}
 	},
 
-	"defeat-player": function(player, cmd, data)
+	"resign": function(player, cmd, data)
 	{
-		let cmpPlayer = QueryPlayerIDInterface(player);
-		if (cmpPlayer)
-			cmpPlayer.SetState("defeated", !!cmd.resign);
+		data.cmpPlayer.SetState("defeated", markForTranslation("%(player)s has resigned."));
 	},
 
 	"garrison": function(player, cmd, data)
@@ -547,13 +569,13 @@ var g_Commands = {
 		}
 	},
 
-	"increase-alert-level": function(player, cmd, data)
+	"alert-raise": function(player, cmd, data)
 	{
 		for (let ent of data.entities)
 		{
 			var cmpAlertRaiser = Engine.QueryInterface(ent, IID_AlertRaiser);
-			if (!cmpAlertRaiser || !cmpAlertRaiser.IncreaseAlertLevel())
-				notifyAlertFailure(player);
+			if (cmpAlertRaiser)
+				cmpAlertRaiser.RaiseAlert();
 		}
 	},
 
@@ -635,7 +657,7 @@ var g_Commands = {
 	"barter": function(player, cmd, data)
 	{
 		var cmpBarter = Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter);
-		cmpBarter.ExchangeResources(data.playerEnt, cmd.sell, cmd.buy, cmd.amount);
+		cmpBarter.ExchangeResources(player, cmd.sell, cmd.buy, cmd.amount);
 	},
 
 	"set-shading-color": function(player, cmd, data)
@@ -696,7 +718,7 @@ var g_Commands = {
 			{
 				var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 				cmpGUIInterface.PushNotification({
-					"players": [data.cmpPlayer.GetPlayerID()],
+					"players": [player],
 					"message": markForTranslation("Cannot upgrade as distance requirements are not verified or terrain is obstructed.")
 				});
 				continue;
@@ -706,18 +728,15 @@ var g_Commands = {
 			{
 				var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 				cmpGUIInterface.PushNotification({
-					"players": [data.cmpPlayer.GetPlayerID()],
+					"players": [player],
 					"message": markForTranslation("Cannot upgrade a garrisoned entity.")
 				});
 				continue;
 			}
 
 			// Check entity limits
-			var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-			var template = cmpTemplateManager.GetTemplate(cmd.template);
 			var cmpEntityLimits = QueryPlayerIDInterface(player, IID_EntityLimits);
-			if (template.TrainingRestrictions && !cmpEntityLimits.AllowedToTrain(template.TrainingRestrictions.Category, 1) ||
-				template.BuildRestrictions && !cmpEntityLimits.AllowedToBuild(template.BuildRestrictions.Category))
+			if (cmpEntityLimits && !cmpEntityLimits.AllowedToReplace(ent, cmd.template))
 			{
 				if (g_DebugCommands)
 					warn("Invalid command: build limits check failed for player " + player + ": " + uneval(cmd));
@@ -742,7 +761,7 @@ var g_Commands = {
 		{
 			let cmpUpgrade = Engine.QueryInterface(ent, IID_Upgrade);
 			if (cmpUpgrade)
-				cmpUpgrade.CancelUpgrade(data.cmpPlayer.playerID);
+				cmpUpgrade.CancelUpgrade(player);
 		}
 	},
 
@@ -773,22 +792,29 @@ var g_Commands = {
 		}));
 
 		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		cmpGUIInterface.PushNotification({
+			"type": "spy-response",
+			"players": [player],
+			"target": cmd.player,
+			"entity": ent
+		});
 		if (ent)
-		{
 			Engine.QueryInterface(ent, IID_VisionSharing).AddSpy(cmd.source);
-			cmpGUIInterface.PushNotification({
-				"type": "spy-response",
-				"players": [player],
-				"entity": ent
-			});
-		}
 		else
+		{
+			let template = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager).GetTemplate("special/spy");
+			IncurBribeCost(template, player, cmd.player, true);
+			// update statistics for failed bribes
+			let cmpBribesStatisticsTracker = QueryPlayerIDInterface(player, IID_StatisticsTracker);
+			if (cmpBribesStatisticsTracker)
+				cmpBribesStatisticsTracker.IncreaseFailedBribesCounter();
 			cmpGUIInterface.PushNotification({
 				"type": "text",
 				"players": [player],
 				"message": markForTranslation("There are no bribable units"),
 				"translateMessage": true
 			});
+		}
 	},
 
 	"diplomacy-request": function(player, cmd, data)
@@ -850,20 +876,6 @@ function notifyBackToWorkFailure(player)
 	});
 }
 
-/**
- * Sends a GUI notification about Alerts that failed to be raised
- */
-function notifyAlertFailure(player)
-{
-	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGUIInterface.PushNotification({
-		"type": "text",
-		"players": [player],
-		"message": "You can't raise the alert to a higher level!",
-		"translateMessage": true
-	});
-}
-
 /**
  * Get some information about the formations used by entities.
  * The entities must have a UnitAI component.
@@ -1420,7 +1432,6 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 			return [];
 
 		RemoveFromFormation(ents);
-		cmpUnitAI.SetLastFormationTemplate("formations/null");
 
 		return [ cmpUnitAI ];
 	}
@@ -1440,23 +1451,21 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 		// TODO: We only check if the formation is usable by some units
 		// if we move them to it. We should check if we can use formations
 		// for the other cases.
-		var nullFormation = (formationTemplate || cmpUnitAI.GetLastFormationTemplate()) == "formations/null";
-		if (!nullFormation && cmpIdentity && cmpIdentity.CanUseFormation(formationTemplate || "formations/null"))
+		var nullFormation = (formationTemplate || cmpUnitAI.GetFormationTemplate()) == "special/formations/null";
+		if (!nullFormation && cmpIdentity && cmpIdentity.CanUseFormation(formationTemplate || "special/formations/null"))
 			formedEnts.push(ent);
 		else
 		{
 			if (nullFormation)
-			{
 				RemoveFromFormation([ent]);
-				cmpUnitAI.SetLastFormationTemplate("formations/null");
-			}
+
 			nonformedUnitAIs.push(cmpUnitAI);
 		}
 	}
 
 	if (formedEnts.length == 0)
 	{
-		// No units support the foundation - return all the others
+		// No units support the formation - return all the others
 		return nonformedUnitAIs;
 	}
 
@@ -1485,14 +1494,6 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 	{
 		// We need to give the selected units a new formation controller
 
-		// Remove selected units from their current formation
-		for (var fid in formation.members)
-		{
-			var cmpFormation = Engine.QueryInterface(+fid, IID_Formation);
-			if (cmpFormation)
-				cmpFormation.RemoveMembers(formation.members[fid]);
-		}
-
 		// TODO replace the fixed 60 with something sensible, based on vision range f.e.
 		var formationSeparation = 60;
 		var clusters = ClusterEntities(formation.entities, formationSeparation);
@@ -1501,14 +1502,14 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 		{
 			if (!formationTemplate || !CanMoveEntsIntoFormation(cluster, formationTemplate))
 			{
-				// get the most recently used formation, or default to line closed
+				// Use the last formation template if everyone was using it
 				var lastFormationTemplate = undefined;
 				for (let ent of cluster)
 				{
 					var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 					if (cmpUnitAI)
 					{
-						var template = cmpUnitAI.GetLastFormationTemplate();
+						var template = cmpUnitAI.GetFormationTemplate();
 						if (lastFormationTemplate === undefined)
 						{
 							lastFormationTemplate = template;
@@ -1523,7 +1524,17 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 				if (lastFormationTemplate && CanMoveEntsIntoFormation(cluster, lastFormationTemplate))
 					formationTemplate = lastFormationTemplate;
 				else
-					formationTemplate = "formations/null";
+					formationTemplate = "special/formations/null";
+			}
+
+			RemoveFromFormation(cluster);
+
+			if (formationTemplate == "special/formations/null")
+			{
+				for (let ent of cluster)
+					nonformedUnitAIs.push(Engine.QueryInterface(ent, IID_UnitAI));
+
+				continue;
 			}
 
 			// Create the new controller
@@ -1685,8 +1696,32 @@ function FilterEntityListWithAllies(entities, player, controlAll)
 	return entities.filter(ent => CanControlUnitOrIsAlly(ent, player, controlAll));
 }
 
+/**
+ * Incur the player with the cost of a bribe, optionally multiply the cost with
+ * the additionalMultiplier
+ */
+function IncurBribeCost(template, player, playerBribed, failedBribe)
+{
+	let cmpPlayerBribed = QueryPlayerIDInterface(playerBribed);
+	if (!cmpPlayerBribed)
+		return false;
+
+	let costs = {};
+	// Additional cost for this owner
+	let multiplier = cmpPlayerBribed.GetSpyCostMultiplier();
+	if (failedBribe)
+		multiplier *= template.VisionSharing.FailureCostRatio;
+
+	for (let res in template.Cost.Resources)
+		costs[res] = Math.floor(multiplier * ApplyValueModificationsToTemplate("Cost/Resources/" + res, +template.Cost.Resources[res], player, template));
+
+	let cmpPlayer = QueryPlayerIDInterface(player);
+	return cmpPlayer && cmpPlayer.TrySubtractResources(costs);
+}
+
 Engine.RegisterGlobal("GetFormationRequirements", GetFormationRequirements);
 Engine.RegisterGlobal("CanMoveEntsIntoFormation", CanMoveEntsIntoFormation);
 Engine.RegisterGlobal("GetDockAngle", GetDockAngle);
 Engine.RegisterGlobal("ProcessCommand", ProcessCommand);
 Engine.RegisterGlobal("g_Commands", g_Commands);
+Engine.RegisterGlobal("IncurBribeCost", IncurBribeCost);
