@@ -50,7 +50,7 @@ function ChangeEntityTemplate(oldEnt, newTemplate)
 	if (cmpHealth && cmpNewHealth)
 	{
 		var healthLevel = Math.max(0, Math.min(1, cmpHealth.GetHitpoints() / cmpHealth.GetMaxHitpoints()));
-		cmpNewHealth.SetHitpoints(Math.round(cmpNewHealth.GetMaxHitpoints() * healthLevel));
+		cmpNewHealth.SetHitpoints(cmpNewHealth.GetMaxHitpoints() * healthLevel);
 	}
 
 	var cmpUnitAI = Engine.QueryInterface(oldEnt, IID_UnitAI);
@@ -95,8 +95,10 @@ function ChangeEntityTemplate(oldEnt, newTemplate)
 
 	TransferGarrisonedUnits(oldEnt, newEnt);
 
-	Engine.BroadcastMessage(MT_EntityRenamed, { "entity": oldEnt, "newentity": newEnt });
+	Engine.PostMessage(oldEnt, MT_EntityRenamed, { "entity": oldEnt, "newentity": newEnt });
 
+	if (cmpPosition && cmpPosition.IsInWorld())
+		cmpPosition.MoveOutOfWorld();
 	Engine.DestroyEntity(oldEnt);
 
 	return newEnt;
@@ -188,8 +190,11 @@ function ObstructionsBlockingTemplateChange(ent, templateArg)
 			var cmpNewObstruction = Engine.QueryInterface(previewEntity, IID_Obstruction);
 			if (cmpNewObstruction && cmpNewObstruction.GetBlockMovementFlag())
 			{
-				// Check for units
-				var collisions = cmpNewObstruction.GetUnitCollisions();
+				// Remove all obstructions at the new entity, especially animal corpses
+				for (let ent of cmpNewObstruction.GetEntitiesDeletedUponConstruction())
+					Engine.DestroyEntity(ent);
+
+				let collisions = cmpNewObstruction.GetEntitiesBlockingConstruction();
 				if (collisions.length)
 					return DeleteEntityAndReturn(previewEntity, cmpPosition, pos, angle, cmpNewPosition, true);
 			}
@@ -216,16 +221,20 @@ function DeleteEntityAndReturn(ent, cmpPosition, position, angle, cmpNewPosition
 function TransferGarrisonedUnits(oldEnt, newEnt)
 {
 	// Transfer garrisoned units if possible, or unload them
-	var cmpOldGarrison = Engine.QueryInterface(oldEnt, IID_GarrisonHolder);
-	var cmpNewGarrison = Engine.QueryInterface(newEnt, IID_GarrisonHolder);
-	if (!cmpNewGarrison || !cmpOldGarrison || !cmpOldGarrison.GetEntities().length)
-		return;	// nothing to do as the code will by default unload all.
+	let cmpOldGarrison = Engine.QueryInterface(oldEnt, IID_GarrisonHolder);
+	if (!cmpOldGarrison || !cmpOldGarrison.GetEntities().length)
+		return;
 
-	var garrisonedEntities = cmpOldGarrison.GetEntities().slice();
-	for (let ent of garrisonedEntities)
+	let cmpNewGarrison = Engine.QueryInterface(newEnt, IID_GarrisonHolder);
+	let entities = cmpOldGarrison.GetEntities().slice();
+	for (let ent of entities)
 	{
-		let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 		cmpOldGarrison.Eject(ent);
+		if (!cmpNewGarrison)
+			continue;
+		let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
+		if (!cmpUnitAI)
+			continue;
 		cmpUnitAI.Autogarrison(newEnt);
 		cmpNewGarrison.Garrison(ent);
 	}
