@@ -1,30 +1,41 @@
-RMS.LoadLibrary("rmgen");
-RMS.LoadLibrary("rmgen2");
+Engine.LoadLibrary("rmgen");
+Engine.LoadLibrary("rmgen-common");
+Engine.LoadLibrary("rmgen2");
+Engine.LoadLibrary("rmbiome");
 
-InitMap();
+setSelectedBiome();
 
-randomizeBiome();
-initMapSettings();
-initTileClasses();
+var heightLand = 2;
 
-resetTerrain(g_Terrains.mainTerrain, g_TileClasses.land, 2);
-RMS.SetProgress(10);
+var g_Map = new RandomMap(heightLand, g_Terrains.mainTerrain);
+var mapCenter = g_Map.getCenter();
+var mapSize = g_Map.getSize();
 
-var pos = randomStartingPositionPattern();
-addBases(pos.setup, pos.distance, pos.separation);
-RMS.SetProgress(20);
+initTileClasses(["bluffsPassage", "nomadArea"]);
+createArea(
+	new MapBoundsPlacer(),
+	new TileClassPainter(g_TileClasses.land));
+
+Engine.SetProgress(10);
+
+var playerIDs;
+var playerPosition;
+if (!isNomad())
+{
+	let playerbasePattern = randomStartingPositionPattern(getTeamsArray());
+	[playerIDs, playerPosition] = createBasesByPattern(playerbasePattern.setup, playerbasePattern.distance, playerbasePattern.groupedDistance, randomAngle());
+	markPlayerAvoidanceArea(playerPosition, defaultPlayerBaseRadius());
+}
+Engine.SetProgress(20);
 
 addElements([
 	{
 		"func": addBluffs,
-		"avoid": [
-			g_TileClasses.bluff, 12,
-			g_TileClasses.hill, 5,
-			g_TileClasses.player, 35
-		],
+		"baseHeight": heightLand,
+		"avoid": [g_TileClasses.bluffIgnore, 0],
 		"sizes": ["normal", "big", "huge"],
 		"mixes": ["same"],
-		"amounts": ["tons"]
+		"amounts": ["many"]
 	},
 	{
 		"func": addHills,
@@ -38,13 +49,17 @@ addElements([
 		"amounts": ["tons"]
 	}
 ]);
-RMS.SetProgress(30);
+Engine.SetProgress(30);
+
+if (!isNomad())
+	createBluffsPassages(playerPosition);
 
 addElements([
 	{
 		"func": addLayeredPatches,
 		"avoid": [
 			g_TileClasses.bluff, 2,
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.dirt, 5,
 			g_TileClasses.forest, 2,
 			g_TileClasses.mountain, 2,
@@ -59,6 +74,7 @@ addElements([
 		"func": addDecoration,
 		"avoid": [
 			g_TileClasses.bluff, 2,
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.forest, 2,
 			g_TileClasses.mountain, 2,
 			g_TileClasses.player, 12,
@@ -69,12 +85,13 @@ addElements([
 		"amounts": ["normal"]
 	}
 ]);
-RMS.SetProgress(50);
+Engine.SetProgress(50);
 
 addElements(shuffleArray([
 	{
 		"func": addMetal,
 		"avoid": [
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.berries, 5,
 			g_TileClasses.forest, 3,
 			g_TileClasses.mountain, 2,
@@ -91,6 +108,7 @@ addElements(shuffleArray([
 	{
 		"func": addStone,
 		"avoid": [
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.berries, 5,
 			g_TileClasses.forest, 3,
 			g_TileClasses.mountain, 2,
@@ -108,6 +126,7 @@ addElements(shuffleArray([
 	{
 		"func": addForests,
 		"avoid": [
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.forest, 6,
 			g_TileClasses.metal, 3,
 			g_TileClasses.mountain, 5,
@@ -124,6 +143,7 @@ addElements(shuffleArray([
 	{
 		"func": addForests,
 		"avoid": [
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.bluff, 10,
 			g_TileClasses.forest, 10,
 			g_TileClasses.metal, 3,
@@ -137,13 +157,14 @@ addElements(shuffleArray([
 		"amounts": ["normal"]
 	}
 ]));
-RMS.SetProgress(70);
+Engine.SetProgress(70);
 
 addElements(shuffleArray([
 	{
 		"func": addBerries,
 		"avoid": [
 			g_TileClasses.bluff, 5,
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.forest, 5,
 			g_TileClasses.metal, 10,
 			g_TileClasses.mountain, 2,
@@ -159,6 +180,7 @@ addElements(shuffleArray([
 		"func": addAnimals,
 		"avoid": [
 			g_TileClasses.bluff, 5,
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.forest, 2,
 			g_TileClasses.metal, 2,
 			g_TileClasses.mountain, 1,
@@ -168,13 +190,14 @@ addElements(shuffleArray([
 		],
 		"sizes": ["small"],
 		"mixes": ["similar"],
-		"amounts": ["normal", "many"]
+		"amounts": ["few"]
 	},
 	{
 		"func": addStragglerTrees,
 		"avoid": [
 			g_TileClasses.berries, 5,
 			g_TileClasses.bluff, 5,
+			g_TileClasses.bluffsPassage, 4,
 			g_TileClasses.forest, 7,
 			g_TileClasses.metal, 2,
 			g_TileClasses.mountain, 1,
@@ -187,6 +210,28 @@ addElements(shuffleArray([
 		"amounts": ["many"]
 	}
 ]));
-RMS.SetProgress(90);
+Engine.SetProgress(90);
 
-ExportMap();
+if (isNomad())
+{
+	g_Map.log("Preventing units to be spawned at the map border");
+	createArea(
+		new DiskPlacer(mapSize / 2 - scaleByMapSize(15, 35), mapCenter),
+		new TileClassPainter(g_TileClasses.nomadArea));
+
+	placePlayersNomad(
+		g_TileClasses.player,
+		[
+			stayClasses(g_TileClasses.nomadArea, 0),
+			avoidClasses(
+				g_TileClasses.bluff, 2,
+				g_TileClasses.water, 4,
+				g_TileClasses.forest, 1,
+				g_TileClasses.metal, 4,
+				g_TileClasses.rock, 4,
+				g_TileClasses.mountain, 4,
+				g_TileClasses.animals, 2)
+		]);
+}
+
+g_Map.ExportMap();
