@@ -6,7 +6,7 @@ const g_EngineInfo = Engine.GetEngineInfo();
 /**
  * Needed for formatPlayerInfo to show the player civs in the details.
  */
-const g_CivData = loadCivData();
+const g_CivData = loadCivData(false, false);
 
 /**
  * Used for creating the mapsize filter.
@@ -36,7 +36,7 @@ var g_MapNames = [];
 /**
  * Sorted list of the victory conditions occuring in the replays
  */
-var g_VictoryConditions = [];
+var g_VictoryConditions = g_Settings && g_Settings.VictoryConditions;
 
 /**
  * Directory name of the currently selected replay. Used to restore the selection after changing filters.
@@ -48,6 +48,11 @@ var g_SelectedReplayDirectory = "";
  */
 var g_ReplaysLoaded = false;
 
+/**
+ * Remember last viewed summary panel and charts.
+ */
+var g_SummarySelectedData;
+
 /**
  * Initializes globals, loads replays and displays the list.
  */
@@ -69,6 +74,9 @@ function init(data)
 
 	initHotkeyTooltips();
 	displayReplayList();
+
+	if (data && data.summarySelectedData)
+		g_SummarySelectedData = data.summarySelectedData;
 }
 
 /**
@@ -100,10 +108,6 @@ function loadReplays(replaySelectionData, compareFiles)
 		if (g_MapNames.indexOf(replay.attribs.settings.Name) == -1 && replay.attribs.settings.Name != "")
 			g_MapNames.push(replay.attribs.settings.Name);
 
-		// Extract victory conditions
-		if (replay.attribs.settings.GameType && g_VictoryConditions.indexOf(replay.attribs.settings.GameType) == -1)
-			g_VictoryConditions.push(replay.attribs.settings.GameType);
-
 		// Extract playernames
 		for (let playerData of replay.attribs.settings.PlayerData)
 		{
@@ -130,7 +134,6 @@ function loadReplays(replaySelectionData, compareFiles)
 	}
 
 	g_MapNames.sort();
-	g_VictoryConditions.sort();
 
 	// Reload filters (since they depend on g_Replays and its derivatives)
 	initFilters(replaySelectionData && replaySelectionData.filters);
@@ -174,9 +177,6 @@ function sanitizeGameAttributes(attribs)
 	if (!attribs.settings.mapType)
 		attribs.settings.mapType = "skirmish";
 
-	if (!attribs.settings.GameType)
-		attribs.settings.GameType = "conquest";
-
 	// Remove gaia
 	if (attribs.settings.PlayerData.length && attribs.settings.PlayerData[0] == null)
 		attribs.settings.PlayerData.shift();
@@ -268,10 +268,11 @@ function displayReplayDetails()
 	Engine.GetGUIObjectByName("sgMapName").caption = translate(replay.attribs.settings.Name);
 	Engine.GetGUIObjectByName("sgMapSize").caption = translateMapSize(replay.attribs.settings.Size);
 	Engine.GetGUIObjectByName("sgMapType").caption = translateMapType(replay.attribs.settings.mapType);
-	Engine.GetGUIObjectByName("sgVictory").caption = translateVictoryCondition(replay.attribs.settings.GameType);
+	Engine.GetGUIObjectByName("sgVictory").caption = replay.attribs.settings.VictoryConditions.map(victoryConditionName =>
+		translateVictoryCondition(victoryConditionName)).join(translate(", "));
 	Engine.GetGUIObjectByName("sgNbPlayers").caption = sprintf(translate("Players: %(numberOfPlayers)s"),
 		{ "numberOfPlayers": replay.attribs.settings.PlayerData.length });
-	Engine.GetGUIObjectByName("replayFilename").caption = escapeText(Engine.GetReplayDirectoryName(replay.directory));
+	Engine.GetGUIObjectByName("replayFilename").caption = Engine.GetReplayDirectoryName(replay.directory);
 
 	let metadata = Engine.GetReplayMetadata(replay.directory);
 	Engine.GetGUIObjectByName("sgPlayersNames").caption =
@@ -283,7 +284,7 @@ function displayReplayDetails()
 				metadata.playerStates.map(pState => pState.state)
 		);
 
-	let mapData = getMapDescriptionAndPreview(replay.attribs.settings.mapType, replay.attribs.map);
+	let mapData = getMapDescriptionAndPreview(replay.attribs.settings.mapType, replay.attribs.map, replay.attribs);
 	Engine.GetGUIObjectByName("sgMapDescription").caption = mapData.description;
 
 	Engine.GetGUIObjectByName("summaryButton").hidden = !Engine.HasReplayMetadata(replay.directory);
@@ -344,7 +345,7 @@ function getReplayDuration(replay)
  */
 function isReplayCompatible(replay)
 {
-	return replayHasSameEngineVersion(replay) && hasSameMods(replay.attribs, g_EngineInfo);
+	return replayHasSameEngineVersion(replay) && hasSameMods(replay.attribs.mods, g_EngineInfo.mods);
 }
 
 /**
