@@ -1,13 +1,15 @@
 Trigger.prototype.CheckRegicideDefeat = function(data)
 {
 	if (data.entity == this.regicideHeroes[data.from])
-		TriggerHelper.DefeatPlayer(data.from);
+		TriggerHelper.DefeatPlayer(
+			data.from,
+			markForTranslation("%(player)s has been defeated (lost hero)."));
 };
 
 Trigger.prototype.InitRegicideGame = function(msg)
 {
 	let cmpEndGameManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_EndGameManager);
-	let regicideGarrison = cmpEndGameManager.GetGameTypeSettings().regicideGarrison;
+	let regicideGarrison = cmpEndGameManager.GetGameSettings().regicideGarrison;
 
 	let playersCivs = [];
 	for (let playerID = 1; playerID < TriggerHelper.GetNumberOfPlayers(); ++playerID)
@@ -18,7 +20,7 @@ Trigger.prototype.InitRegicideGame = function(msg)
 	let cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 	for (let templateName of cmpTemplateManager.FindAllTemplates(false))
 	{
-		if (templateName.substring(0,6) != "units/")
+		if (!templateName.startsWith("units/"))
 			continue;
 
 		let identity = cmpTemplateManager.GetTemplate(templateName).Identity;
@@ -62,11 +64,13 @@ Trigger.prototype.InitRegicideGame = function(msg)
 			getSpawnPreference(entity2) - getSpawnPreference(entity1));
 
 		// Spawn the hero on land as close as possible
-		if (!regicideGarrison && TriggerHelper.EntityHasClass(spawnPoints[0], "Ship"))
+		if (!regicideGarrison && TriggerHelper.EntityMatchesClassList(spawnPoints[0], "Ship"))
 		{
 			let shipPosition = Engine.QueryInterface(spawnPoints[0], IID_Position).GetPosition2D();
+
 			let distanceToShip = entity =>
-				Engine.QueryInterface(entity, IID_Position).GetPosition2D().distanceTo(shipPosition);
+				Engine.QueryInterface(entity, IID_Position).GetPosition2D().distanceToSquared(shipPosition);
+
 			spawnPoints = TriggerHelper.GetLandSpawnPoints().sort((entity1, entity2) =>
 				distanceToShip(entity1) - distanceToShip(entity2));
 		}
@@ -91,7 +95,7 @@ Trigger.prototype.SpawnRegicideHero = function(playerID, heroTemplates, spawnPoi
 				continue;
 
 			// Consider nomad maps where units start on a ship
-			let isShip = TriggerHelper.EntityHasClass(spawnPoint, "Ship");
+			let isShip = TriggerHelper.EntityMatchesClassList(spawnPoint, "Ship");
 			if (isShip)
 			{
 				let cmpGarrisonHolder = Engine.QueryInterface(spawnPoint, IID_GarrisonHolder);
