@@ -1,18 +1,17 @@
 /**
  * All known cheat commands.
- * @type {Object}
  */
 const g_Cheats = getCheatsData();
 
 /**
  * Number of seconds after which chatmessages will disappear.
  */
-const g_ChatTimeout = 30;
+var g_ChatTimeout = 30;
 
 /**
  * Maximum number of lines to display simultaneously.
  */
-const g_ChatLines = 20;
+var g_ChatLines = 20;
 
 /**
  * The currently displayed strings, limited by the given timeframe and limit above.
@@ -29,6 +28,21 @@ var g_ChatHistory = [];
  */
 var g_ChatTimers = [];
 
+/**
+ * Command to send to the previously selected private chat partner.
+ */
+var g_LastChatAddressee = "";
+
+/**
+ * All tutorial messages received so far.
+ */
+var g_TutorialMessages = [];
+
+/**
+ * GUI tags applied to the most recent tutorial message.
+ */
+var g_TutorialNewMessageTags = { "color": "yellow" };
+
 /**
  * Handle all netmessage types that can occur.
  */
@@ -39,12 +53,18 @@ var g_NetMessageTypes = {
 	"netwarn": msg => {
 		addNetworkWarning(msg);
 	},
+	"out-of-sync": msg => {
+		onNetworkOutOfSync(msg);
+	},
 	"players": msg => {
 		handlePlayerAssignmentsMessage(msg);
 	},
 	"paused": msg => {
 		setClientPauseState(msg.guid, msg.pause);
 	},
+	"clients-loading": msg => {
+		handleClientsLoadingMessage(msg.guids);
+	},
 	"rejoined": msg => {
 		addChatMessage({
 			"type": "rejoined",
@@ -80,17 +100,27 @@ var g_NetMessageTypes = {
 var g_FormatChatMessage = {
 	"system": msg => msg.text,
 	"connect": msg =>
-		sprintf(translate("%(player)s is starting to rejoin the game."), {
-			"player": colorizePlayernameByGUID(msg.guid)
-		}),
+		sprintf(
+			g_PlayerAssignments[msg.guid].player != -1 ?
+				// Translation: A player that left the game joins again
+				translate("%(player)s is starting to rejoin the game.") :
+				// Translation: A player joins the game for the first time
+				translate("%(player)s is starting to join the game."),
+			{ "player": colorizePlayernameByGUID(msg.guid) }
+		),
 	"disconnect": msg =>
 		sprintf(translate("%(player)s has left the game."), {
 			"player": colorizePlayernameByGUID(msg.guid)
 		}),
 	"rejoined": msg =>
-		sprintf(translate("%(player)s has rejoined the game."), {
-			"player": colorizePlayernameByGUID(msg.guid)
-		}),
+		sprintf(
+			g_PlayerAssignments[msg.guid].player != -1 ?
+				// Translation: A player that left the game joins again
+				translate("%(player)s has rejoined the game.") :
+				// Translation: A player joins the game for the first time
+				translate("%(player)s has joined the game."),
+			{ "player": colorizePlayernameByGUID(msg.guid) }
+		),
 	"kicked": msg =>
 		sprintf(
 			msg.banned ?
@@ -105,12 +135,12 @@ var g_FormatChatMessage = {
 		),
 	"clientlist": msg => getUsernameList(),
 	"message": msg => formatChatCommand(msg),
-	"defeat": msg => formatDefeatMessage(msg),
-	"won": msg => formatWinMessage(msg),
+	"defeat-victory": msg => formatDefeatVictoryMessage(msg.message, msg.players),
 	"diplomacy": msg => formatDiplomacyMessage(msg),
 	"tribute": msg => formatTributeMessage(msg),
 	"barter": msg => formatBarterMessage(msg),
-	"attack": msg => formatAttackMessage(msg)
+	"attack": msg => formatAttackMessage(msg),
+	"phase": msg => formatPhaseMessage(msg)
 };
 
 /**
@@ -124,8 +154,8 @@ var g_StatusMessageTypes = {
 		sprintf(translate("Reason: %(reason)s."), {
 			"reason": getDisconnectReason(msg.reason, true)
 		}),
-	"waiting_for_players": msg => translate("Waiting for other players to connect..."),
-	"join_syncing": msg => translate("Synchronising gameplay with other players..."),
+	"waiting_for_players": msg => translate("Waiting for players to connect:"),
+	"join_syncing": msg => translate("Synchronising gameplay with other players…"),
 	"active": msg => ""
 };
 
@@ -286,53 +316,41 @@ var g_NotificationsTypes =
 		{
 			message.translateParameters = notification.translateParameters;
 			message.parameters = notification.parameters;
-			// special case for formatting of player names which are transmitted as _player_num
-			for (let param in message.parameters)
-			{
-				if (!param.startsWith("_player_"))
-					continue;
-
-				message.parameters[param] = colorizePlayernameByID(message.parameters[param]);
-			}
+			colorizePlayernameParameters(notification.parameters);
 		}
 
 		addChatMessage(message);
 	},
 	"defeat": function(notification, player)
 	{
-		addChatMessage({
-			"type": "defeat",
-			"guid": findGuidForPlayerID(player),
-			"player": player,
-			"resign": !!notification.resign
-		});
-		playerFinished(player, false);
-		sendLobbyPlayerlistUpdate();
+		playersFinished(notification.allies, notification.message, false);
 	},
 	"won": function(notification, player)
 	{
-		addChatMessage({
-			"type": "won",
-			"guid": findGuidForPlayerID(player),
-			"player": player
-		});
-		playerFinished(player, true);
-		sendLobbyPlayerlistUpdate();
+		playersFinished(notification.allies, notification.message, true);
 	},
 	"diplomacy": function(notification, player)
 	{
+		updatePlayerData();
+		if (g_DiplomacyColorsToggle)
+			updateDisplayedPlayerColors();
+
 		addChatMessage({
 			"type": "diplomacy",
 			"sourcePlayer": player,
 			"targetPlayer": notification.targetPlayer,
 			"status": notification.status
 		});
-
-		updateDiplomacy();
 	},
-	"quit": function(notification, player)
+	"ceasefire-ended": function(notification, player)
 	{
-		Engine.Exit();
+		updatePlayerData();
+		if (g_DiplomacyColorsToggle)
+			updateDisplayedPlayerColors();
+	},
+	"tutorial": function(notification, player)
+	{
+		updateTutorial(notification);
 	},
 	"tribute": function(notification, player)
 	{
@@ -354,6 +372,17 @@ var g_NotificationsTypes =
 			"resourceBought": notification.resourceBought
 		});
 	},
+	"spy-response": function(notification, player)
+	{
+		if (g_BribeButtonsWaiting[player])
+			g_BribeButtonsWaiting[player] = g_BribeButtonsWaiting[player].filter(p => p != notification.target);
+
+		if (notification.entity && g_ViewedPlayer == player)
+		{
+			closeDiplomacy();
+			setCameraFollow(notification.entity);
+		}
+	},
 	"attack": function(notification, player)
 	{
 		if (player != g_ViewedPlayer)
@@ -369,7 +398,7 @@ var g_NotificationsTypes =
 				g_Selection.addList([notification.target]);
 		}
 
-		if (Engine.ConfigDB_GetValue("user", "gui.session.attacknotificationmessage") !== "true")
+		if (Engine.ConfigDB_GetValue("user", "gui.session.notifications.attack") !== "true")
 			return;
 
 		addChatMessage({
@@ -379,6 +408,15 @@ var g_NotificationsTypes =
 			"targetIsDomesticAnimal": notification.targetIsDomesticAnimal
 		});
 	},
+	"phase": function(notification, player)
+	{
+		addChatMessage({
+			"type": "phase",
+			"player": player,
+			"phaseName": notification.phaseName,
+			"phaseState": notification.phaseState
+		});
+	},
 	"dialog": function(notification, player)
 	{
 		if (player == Engine.GetPlayerID())
@@ -398,10 +436,11 @@ var g_NotificationsTypes =
 
 		let cmd = notification.cmd;
 
-		// Ignore boring animals
+		// Ignore rallypoint commands of trained animals
 		let entState = cmd.entities && cmd.entities[0] && GetEntityState(cmd.entities[0]);
-		if (entState && entState.identity && entState.identity.classes &&
-				entState.identity.classes.indexOf("Animal") != -1)
+		if (g_ViewedPlayer != 0 &&
+		    entState && entState.identity && entState.identity.classes &&
+		    entState.identity.classes.indexOf("Animal") != -1)
 			return;
 
 		// Focus the building to construct
@@ -411,10 +450,15 @@ var g_NotificationsTypes =
 			if (targetState)
 				Engine.CameraMoveTo(targetState.position.x, targetState.position.z);
 		}
+		else if (cmd.type == "delete-entities" && notification.position)
+			Engine.CameraMoveTo(notification.position.x, notification.position.y);
 		// Focus commanded entities, but don't lose previous focus when training units
 		else if (cmd.type != "train" && cmd.type != "research" && entState)
 			setCameraFollow(cmd.entities[0]);
 
+		if (["walk", "attack-walk", "patrol"].indexOf(cmd.type) != -1)
+			DrawTargetMarker(cmd);
+
 		// Select units affected by that command
 		let selection = [];
 		if (cmd.entities)
@@ -425,28 +469,34 @@ var g_NotificationsTypes =
 		// Allow gaia in selection when gathering
 		g_Selection.reset();
 		g_Selection.addList(selection, false, cmd.type == "gather");
+	},
+	"play-tracks": function (notification, player)
+	{
+		if (notification.lock)
+		{
+			global.music.storeTracks(notification.tracks.map(track => ({ "Type": "custom", "File": track })));
+			global.music.setState(global.music.states.CUSTOM);
+		}
+
+		global.music.setLocked(notification.lock);
 	}
 };
 
 /**
  * Loads all known cheat commands.
- *
- * @returns {Object}
  */
 function getCheatsData()
 {
 	let cheats = {};
-	for (let fileName of getJSONFileList("simulation/data/cheats/"))
+	for (let fileName of Engine.ListDirectoryFiles("simulation/data/cheats/", "*.json", false))
 	{
-		let currentCheat = Engine.ReadJSONFile("simulation/data/cheats/"+fileName+".json");
-		if (!currentCheat)
-			continue;
-		if (Object.keys(cheats).indexOf(currentCheat.Name) !== -1)
+		let currentCheat = Engine.ReadJSONFile(fileName);
+		if (cheats[currentCheat.Name])
 			warn("Cheat name '" + currentCheat.Name + "' is already present");
 		else
 			cheats[currentCheat.Name] = currentCheat.Data;
 	}
-	return cheats;
+	return deepfreeze(cheats);
 }
 
 /**
@@ -461,17 +511,17 @@ function executeCheat(text)
 		return false;
 
 	// Find the cheat code that is a prefix of the user input
-	let cheatCode = Object.keys(g_Cheats).find(cheatCode => text.indexOf(cheatCode) == 0);
+	let cheatCode = Object.keys(g_Cheats).find(code => text.indexOf(code) == 0);
 	if (!cheatCode)
 		return false;
 
 	let cheat = g_Cheats[cheatCode];
 
-	let parameter = text.substr(cheatCode.length);
+	let parameter = text.substr(cheatCode.length + 1);
 	if (cheat.isNumeric)
 		parameter = +parameter;
 
-	if (cheat.DefaultParameter && (isNaN(parameter) || parameter <= 0))
+	if (cheat.DefaultParameter && !parameter)
 		parameter = cheat.DefaultParameter;
 
 	Engine.PostNetworkCommand({
@@ -511,14 +561,44 @@ function handleNotifications()
 }
 
 /**
- * Updates playerdata cache and refresh diplomacy panel.
+ * Updates the tutorial panel when a new goal.
  */
-function updateDiplomacy()
+function updateTutorial(notification)
 {
-	g_Players = getPlayerData(g_Players);
+	// Show the tutorial panel if not yet done
+	Engine.GetGUIObjectByName("tutorialPanel").hidden = false;
 
-	if (g_IsDiplomacyOpen)
-		openDiplomacy();
+	if (notification.warning)
+	{
+		Engine.GetGUIObjectByName("tutorialWarning").caption = coloredText(translate(notification.warning), "orange");
+		return;
+	}
+
+	let notificationText =
+		notification.instructions.reduce((instructions, item) =>
+			instructions + (typeof item == "string" ? translate(item) : colorizeHotkey(translate(item.text), item.hotkey)),
+			"");
+
+	Engine.GetGUIObjectByName("tutorialText").caption = g_TutorialMessages.concat(setStringTags(notificationText, g_TutorialNewMessageTags)).join("\n");
+	g_TutorialMessages.push(notificationText);
+
+	if (notification.readyButton)
+	{
+		Engine.GetGUIObjectByName("tutorialReady").hidden = false;
+		if (notification.leave)
+		{
+			Engine.GetGUIObjectByName("tutorialWarning").caption = translate("Click to quit this tutorial.");
+			Engine.GetGUIObjectByName("tutorialReady").caption = translate("Quit");
+			Engine.GetGUIObjectByName("tutorialReady").onPress = leaveGame;
+		}
+		else
+			Engine.GetGUIObjectByName("tutorialWarning").caption = translate("Click when ready.");
+	}
+	else
+	{
+		Engine.GetGUIObjectByName("tutorialWarning").caption = translate("Follow the instructions.");
+		Engine.GetGUIObjectByName("tutorialReady").hidden = true;
+	}
 }
 
 /**
@@ -526,7 +606,7 @@ function updateDiplomacy()
  */
 function updateTimeNotifications()
 {
-	let notifications =  Engine.GuiInterfaceCall("GetTimeNotifications", g_ViewedPlayer);
+	let notifications = Engine.GuiInterfaceCall("GetTimeNotifications", g_ViewedPlayer);
 	let notificationText = "";
 	for (let n of notifications)
 	{
@@ -538,7 +618,9 @@ function updateTimeNotifications()
 		if (n.translateParameters)
 			translateObjectKeys(parameters, n.translateParameters);
 
-		parameters.time = timeToString(n.endTime - g_SimState.timeElapsed);
+		parameters.time = timeToString(n.endTime - GetSimState().timeElapsed);
+
+		colorizePlayernameParameters(parameters);
 
 		notificationText += sprintf(message, parameters) + "\n";
 	}
@@ -582,10 +664,13 @@ function handleNetStatusMessage(message)
 
 	g_IsNetworkedActive = message.status == "active";
 
-	let label = Engine.GetGUIObjectByName("netStatus");
+	let netStatus = Engine.GetGUIObjectByName("netStatus");
 	let statusMessage = g_StatusMessageTypes[message.status](message);
-	label.caption = statusMessage;
-	label.hidden = !statusMessage;
+	netStatus.caption = statusMessage;
+	netStatus.hidden = !statusMessage;
+
+	let loadingClientsText = Engine.GetGUIObjectByName("loadingClientsText");
+	loadingClientsText.hidden = message.status != "waiting_for_players";
 
 	if (message.status == "disconnected")
 	{
@@ -594,10 +679,67 @@ function handleNetStatusMessage(message)
 		Engine.SetPaused(true, false);
 
 		g_Disconnected = true;
+		updateCinemaPath();
 		closeOpenDialogs();
 	}
 }
 
+function handleClientsLoadingMessage(guids)
+{
+	let loadingClientsText = Engine.GetGUIObjectByName("loadingClientsText");
+	loadingClientsText.caption = guids.map(guid => colorizePlayernameByGUID(guid)).join(translateWithContext("Separator for a list of client loading messages", ", "));
+}
+
+function onNetworkOutOfSync(msg)
+{
+	let txt = [
+		sprintf(translate("Out-Of-Sync error on turn %(turn)s."), {
+			"turn": msg.turn
+		}),
+
+		sprintf(translateWithContext("Out-Of-Sync", "Players: %(players)s"), {
+			"players": msg.players.join(translateWithContext("Separator for a list of players", ", "))
+		}),
+
+		msg.hash == msg.expectedHash ?
+			translateWithContext("Out-Of-Sync", "Your game state is identical to the hosts game state.") :
+			translateWithContext("Out-Of-Sync", "Your game state differs from the hosts game state."),
+
+		""
+	];
+
+	if (msg.turn > 1 && g_GameAttributes.settings.PlayerData.some(pData => pData && pData.AI))
+		txt.push(translateWithContext("Out-Of-Sync", "Rejoining Multiplayer games with AIs is not supported yet!"));
+	else
+		txt.push(
+			translateWithContext("Out-Of-Sync", "Ensure all players use the same mods."),
+			translateWithContext("Out-Of-Sync", 'Click on "Report a Bug" in the main menu to help fix this.'),
+			sprintf(translateWithContext("Out-Of-Sync", "Replay saved to %(filepath)s"), {
+				"filepath": escapeText(msg.path_replay)
+			}),
+			sprintf(translateWithContext("Out-Of-Sync", "Dumping current state to %(filepath)s"), {
+				"filepath": escapeText(msg.path_oos_dump)
+			})
+		);
+
+	messageBox(
+		600, 280,
+		txt.join("\n"),
+		translate("Out of Sync")
+	);
+}
+
+function onReplayOutOfSync()
+{
+	messageBox(
+		500, 140,
+		translate("Out-Of-Sync error!") + "\n" +
+			// Translation: This is shown if replay is out of sync
+			translateWithContext("Out-Of-Sync", "The current game state is different from the original game state."),
+		translate("Out of Sync")
+	);
+}
+
 function handlePlayerAssignmentsMessage(message)
 {
 	for (let guid in g_PlayerAssignments)
@@ -612,6 +754,7 @@ function handlePlayerAssignmentsMessage(message)
 		onClientJoin(guid);
 	});
 
+	updateGUIObjects();
 	updateChatAddressees();
 	sendLobbyPlayerlistUpdate();
 }
@@ -682,7 +825,7 @@ function updateChatAddressees()
 	let guids = sortGUIDsByPlayerID();
 	for (let guid of guids)
 	{
-		if (guid == Engine.GetPlayerGUID() || guid == "local")
+		if (guid == Engine.GetPlayerGUID())
 			continue;
 
 		let playerID = g_PlayerAssignments[guid].player;
@@ -751,6 +894,10 @@ function submitChatInput()
 	if (chatAddressee.selected > 0 && (text.indexOf("/") != 0 || text.indexOf("/me ") == 0))
 		text = chatAddressee.list_data[chatAddressee.selected] + " " + text;
 
+	let selectedChat = chatAddressee.list_data[chatAddressee.selected];
+	if (selectedChat.startsWith("/msg"))
+		g_LastChatAddressee = selectedChat;
+
 	submitChatDirectly(text);
 }
 
@@ -781,7 +928,7 @@ function addChatMessage(msg)
 	let historical = {
 		"txt": formatted,
 		"timePrefix": sprintf(translate("\\[%(time)s]"), {
-			"time": Engine.FormatMillisecondsIntoDateString(new Date().getTime(), translate("HH:mm"))
+			"time": Engine.FormatMillisecondsIntoDateStringLocal(Date.now(), translate("HH:mm"))
 		}),
 		"filter": {}
 	};
@@ -824,24 +971,34 @@ function colorizePlayernameByGUID(guid)
 
 function colorizePlayernameHelper(username, playerID)
 {
-	let playerColor = playerID > -1 ? rgbToGuiColor(g_Players[playerID].color) : "white";
-	return '[color="' + playerColor + '"]' + (username || translate("Unknown Player")) + "[/color]";
+	let playerColor = playerID > -1 ? rgbToGuiColor(g_DisplayedPlayerColors[playerID]) : "white";
+	return coloredText(username || translate("Unknown Player"), playerColor);
 }
 
-function formatDefeatMessage(msg)
+/**
+ * Insert the colorized playername to chat messages sent by the AI and time notifications.
+ */
+function colorizePlayernameParameters(parameters)
 {
-	return sprintf(
-		msg.resign ?
-			translate("%(player)s has resigned.") :
-			translate("%(player)s has been defeated."),
-		{ "player": colorizePlayernameByID(msg.player) }
-	);
+	for (let param in parameters)
+		if (param.startsWith("_player_"))
+			parameters[param] = colorizePlayernameByID(parameters[param]);
 }
 
-function formatWinMessage(msg)
+function formatDefeatVictoryMessage(message, players)
 {
-	return sprintf(translate("%(player)s has won."), {
-		"player": colorizePlayernameByID(msg.player)
+	if (!message.pluralMessage)
+		return sprintf(translate(message), {
+			"player": colorizePlayernameByID(players[0])
+		});
+
+	let mPlayers = players.map(playerID => colorizePlayernameByID(playerID));
+	let lastPlayer = mPlayers.pop();
+
+	return sprintf(translatePlural(message.message, message.pluralMessage, message.pluralCount), {
+		// Translation: This comma is used for separating first to penultimate elements in an enumeration.
+		"players": mPlayers.join(translate(", ")),
+		"lastPlayer": lastPlayer
 	});
 }
 
@@ -864,14 +1021,22 @@ function formatDiplomacyMessage(msg)
 	});
 }
 
+/**
+ * Optionally show all tributes sent in observer mode and tributes sent between allied players.
+ * Otherwise, only show tributes sent directly to us, and tributes that we send.
+ */
 function formatTributeMessage(msg)
 {
-	// Check observer first, since we also want to see if the selected player in the developer-overlay has sent tributes
 	let message = "";
-	if (g_IsObserver)
-		message = translate("%(player)s has sent %(player2)s %(amounts)s.");
-	else if (msg.targetPlayer == Engine.GetPlayerID())
+	if (msg.targetPlayer == Engine.GetPlayerID())
 		message = translate("%(player)s has sent you %(amounts)s.");
+	else if (msg.sourcePlayer == Engine.GetPlayerID())
+		message = translate("You have sent %(player2)s %(amounts)s.");
+	else if (Engine.ConfigDB_GetValue("user", "gui.session.notifications.tribute") == "true" &&
+	        (g_IsObserver || g_GameAttributes.settings.LockTeams &&
+	           g_Players[msg.sourcePlayer].isMutualAlly[Engine.GetPlayerID()] &&
+	           g_Players[msg.targetPlayer].isMutualAlly[Engine.GetPlayerID()]))
+		message = translate("%(player)s has sent %(player2)s %(amounts)s.");
 
 	return sprintf(message, {
 		"player": colorizePlayernameByID(msg.sourcePlayer),
@@ -882,7 +1047,7 @@ function formatTributeMessage(msg)
 
 function formatBarterMessage(msg)
 {
-	if (!g_IsObserver)
+	if (!g_IsObserver || Engine.ConfigDB_GetValue("user", "gui.session.notifications.barter") != "true")
 		return "";
 
 	let amountsSold = {};
@@ -904,14 +1069,37 @@ function formatAttackMessage(msg)
 		return "";
 
 	let message = msg.targetIsDomesticAnimal ?
-			translate("Your livestock has been attacked by %(attacker)s!") :
-			translate("You have been attacked by %(attacker)s!");
+		translate("Your livestock has been attacked by %(attacker)s!") :
+		translate("You have been attacked by %(attacker)s!");
 
 	return sprintf(message, {
 		"attacker": colorizePlayernameByID(msg.attacker)
 	});
 }
 
+function formatPhaseMessage(msg)
+{
+	let notifyPhase = Engine.ConfigDB_GetValue("user", "gui.session.notifications.phase");
+	if (notifyPhase == "none" || msg.player != g_ViewedPlayer && !g_IsObserver && !g_Players[msg.player].isMutualAlly[g_ViewedPlayer])
+		return "";
+
+	let message = "";
+	if (notifyPhase == "all")
+	{
+		if (msg.phaseState == "started")
+			message = translate("%(player)s is advancing to the %(phaseName)s.");
+		else if (msg.phaseState == "aborted")
+			message = translate("The %(phaseName)s of %(player)s has been aborted.");
+	}
+	if (msg.phaseState == "completed")
+		message = translate("%(player)s has reached the %(phaseName)s.");
+
+	return sprintf(message, {
+		"player": colorizePlayernameByID(msg.player),
+		"phaseName": getEntityNames(GetTechnologyData(msg.phaseName, g_Players[msg.player].civ))
+	});
+}
+
 function formatChatCommand(msg)
 {
 	if (!msg.text)
@@ -943,10 +1131,10 @@ function formatChatCommand(msg)
 	{
 		msg.text = escapeText(msg.text);
 
-		let userName = g_PlayerAssignments[Engine.GetPlayerGUID() || "local"].name;
-
-		if (userName != g_PlayerAssignments[msg.guid].name)
-			notifyUser(userName, msg.text);
+		let userName = g_PlayerAssignments[Engine.GetPlayerGUID()].name;
+		if (userName != g_PlayerAssignments[msg.guid].name &&
+		    msg.text.toLowerCase().indexOf(splitRatingFromNick(userName).nick.toLowerCase()) != -1)
+			soundNotification("nick");
 	}
 
 	// GUID for players, playerID for AIs
@@ -980,7 +1168,7 @@ function parseChatAddressee(msg)
 	// Chat messages sent by the simulation (AI) come with the playerID.
 	let senderID = msg.player ? msg.player : (g_PlayerAssignments[msg.guid] || msg).player;
 
-	let isSender = msg.guid && msg.guid != "local" ?
+	let isSender = msg.guid ?
 		msg.guid == Engine.GetPlayerGUID() :
 		senderID == Engine.GetPlayerID();
 
@@ -1053,7 +1241,7 @@ function matchUsername(text)
  */
 function sendDialogAnswer(guiObject, dialogName)
 {
-	Engine.GetGUIObjectByName(dialogName+"-dialog").hidden = true;
+	Engine.GetGUIObjectByName(dialogName + "-dialog").hidden = true;
 
 	Engine.PostNetworkCommand({
 		"type": "dialog-answer",
