@@ -46,7 +46,7 @@ function updateGarrisonHealthBar(entState, selection)
 	{
 		let healthBarGarrison = Engine.GetGUIObjectByName("healthBarGarrison");
 		let healthSize = healthBarGarrison.size;
-		healthSize.rtop = 100-100*Math.max(0, Math.min(1, totalGarrisonHealth / maxGarrisonHealth));
+		healthSize.rtop = 100 - 100 * Math.max(0, Math.min(1, totalGarrisonHealth / maxGarrisonHealth));
 		healthBarGarrison.size = healthSize;
 
 		healthGarrison.tooltip = getCurrentHealthTooltip({
@@ -72,7 +72,7 @@ function displaySingle(entState)
 	let civEmblem = g_CivData[playerState.civ].Emblem;
 
 	let playerName = playerState.name;
-	let playerColor = playerState.color.r + " " + playerState.color.g + " " + playerState.color.b + " 128";
+	let playerColor = rgbToGuiColor(g_DisplayedPlayerColors[entState.player], 128);
 
 	// Indicate disconnected players by prefixing their name
 	if (g_Players[entState.player].offline)
@@ -84,7 +84,7 @@ function displaySingle(entState)
 		Engine.GetGUIObjectByName("rankIcon").tooltip = sprintf(translate("%(rank)s Rank"), {
 			"rank": translateWithContext("Rank", entState.identity.rank)
 		});
-		Engine.GetGUIObjectByName("rankIcon").sprite = getRankIconSprite(entState);
+		Engine.GetGUIObjectByName("rankIcon").sprite = "stretched:session/icons/ranks/" + entState.identity.rank + ".png";
 		Engine.GetGUIObjectByName("rankIcon").hidden = false;
 	}
 	else
@@ -109,20 +109,8 @@ function displaySingle(entState)
 	{
 		let unitHealthBar = Engine.GetGUIObjectByName("healthBar");
 		let healthSize = unitHealthBar.size;
-		healthSize.rright = 100*Math.max(0, Math.min(1, entState.hitpoints / entState.maxHitpoints));
+		healthSize.rright = 100 * Math.max(0, Math.min(1, entState.hitpoints / entState.maxHitpoints));
 		unitHealthBar.size = healthSize;
-
-		if (entState.foundation && entState.visibility == "visible" && entState.foundation.numBuilders !== 0)
-		{
-			// logic comes from Foundation component.
-			let speed = Math.pow(entState.foundation.numBuilders, 0.7);
-			let timeLeft = (1.0 - entState.foundation.progress / 100.0) * template.cost.time;
-			let timeToCompletion = Math.ceil(timeLeft/speed);
-			Engine.GetGUIObjectByName("health").tooltip = sprintf(translatePlural("This foundation will be completed in %(seconds)s second.", "This foundation will be completed in %(seconds)s seconds.", timeToCompletion), { "seconds": timeToCompletion });
-		}
-		else
-			Engine.GetGUIObjectByName("health").tooltip = "";
-
 		Engine.GetGUIObjectByName("healthStats").caption = sprintf(translate("%(hitpoints)s / %(maxHitpoints)s"), {
 			"hitpoints": Math.ceil(entState.hitpoints),
 			"maxHitpoints": Math.ceil(entState.maxHitpoints)
@@ -143,15 +131,15 @@ function displaySingle(entState)
 	if (entState.capturePoints)
 	{
 		let setCaptureBarPart = function(playerID, startSize) {
-			let unitCaptureBar = Engine.GetGUIObjectByName("captureBar["+playerID+"]");
+			let unitCaptureBar = Engine.GetGUIObjectByName("captureBar[" + playerID + "]");
 			let sizeObj = unitCaptureBar.size;
 			sizeObj.rleft = startSize;
 
-			let size = 100*Math.max(0, Math.min(1, entState.capturePoints[playerID] / entState.maxCapturePoints));
+			let size = 100 * Math.max(0, Math.min(1, entState.capturePoints[playerID] / entState.maxCapturePoints));
 			sizeObj.rright = startSize + size;
 			unitCaptureBar.size = sizeObj;
-			unitCaptureBar.sprite = "color: " + rgbToGuiColor(g_Players[playerID].color, 128);
-			unitCaptureBar.hidden=false;
+			unitCaptureBar.sprite = "color:" + rgbToGuiColor(g_DisplayedPlayerColors[playerID], 128);
+			unitCaptureBar.hidden = false;
 			return startSize + size;
 		};
 
@@ -168,7 +156,7 @@ function displaySingle(entState)
 		});
 
 		let showSmallCapture = showResource && showHealth;
-		Engine.GetGUIObjectByName("captureStats").caption = showSmallCapture ? "": captureText;
+		Engine.GetGUIObjectByName("captureStats").caption = showSmallCapture ? "" : captureText;
 		Engine.GetGUIObjectByName("capture").tooltip = showSmallCapture ? captureText : "";
 	}
 
@@ -208,7 +196,7 @@ function displaySingle(entState)
 		let resourceSize = unitResourceBar.size;
 
 		resourceSize.rright = entState.resourceSupply.isInfinite ? 100 :
-						100 * Math.max(0, Math.min(1, +entState.resourceSupply.amount / +entState.resourceSupply.max));
+			100 * Math.max(0, Math.min(1, +entState.resourceSupply.amount / +entState.resourceSupply.max));
 		unitResourceBar.size = resourceSize;
 
 		Engine.GetGUIObjectByName("resourceLabel").caption = sprintf(translate("%(resource)s:"), {
@@ -218,90 +206,82 @@ function displaySingle(entState)
 
 	}
 
+	let resourceCarryingIcon = Engine.GetGUIObjectByName("resourceCarryingIcon");
+	let resourceCarryingText = Engine.GetGUIObjectByName("resourceCarryingText");
+	resourceCarryingIcon.hidden = false;
+	resourceCarryingText.hidden = false;
+
 	// Resource carrying
 	if (entState.resourceCarrying && entState.resourceCarrying.length)
 	{
 		// We should only be carrying one resource type at once, so just display the first
 		let carried = entState.resourceCarrying[0];
-
-		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingIcon").sprite = "stretched:session/icons/resources/"+carried.type+".png";
-		Engine.GetGUIObjectByName("resourceCarryingText").caption = sprintf(translate("%(amount)s / %(max)s"), { "amount": carried.amount, "max": carried.max });
-		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = "";
+		resourceCarryingIcon.sprite = "stretched:session/icons/resources/" + carried.type + ".png";
+		resourceCarryingText.caption = sprintf(translate("%(amount)s / %(max)s"), { "amount": carried.amount, "max": carried.max });
+		resourceCarryingIcon.tooltip = "";
 	}
 	// Use the same indicators for traders
 	else if (entState.trader && entState.trader.goods.amount)
 	{
-		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingIcon").sprite = "stretched:session/icons/resources/"+entState.trader.goods.type+".png";
+		resourceCarryingIcon.sprite = "stretched:session/icons/resources/" + entState.trader.goods.type + ".png";
 		let totalGain = entState.trader.goods.amount.traderGain;
 		if (entState.trader.goods.amount.market1Gain)
 			totalGain += entState.trader.goods.amount.market1Gain;
 		if (entState.trader.goods.amount.market2Gain)
 			totalGain += entState.trader.goods.amount.market2Gain;
-		Engine.GetGUIObjectByName("resourceCarryingText").caption = totalGain;
-		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = sprintf(translate("Gain: %(gain)s"), {
+		resourceCarryingText.caption = totalGain;
+		resourceCarryingIcon.tooltip = sprintf(translate("Gain: %(gain)s"), {
 			"gain": getTradingTooltip(entState.trader.goods.amount)
 		});
 	}
 	// And for number of workers
-	else if (entState.foundation && entState.visibility == "visible")
+	else if (entState.foundation)
 	{
-		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingIcon").sprite = "stretched:session/icons/repair.png";
-		Engine.GetGUIObjectByName("resourceCarryingText").caption = entState.foundation.numBuilders + "    ";
-		if (entState.foundation.numBuilders !== 0)
-		{
-			let speedup = Math.pow((entState.foundation.numBuilders+1)/entState.foundation.numBuilders, 0.7);
-			let timeLeft = (1.0 - entState.foundation.progress / 100.0) * template.cost.time;
-			let timeSpeedup = Math.ceil(timeLeft - timeLeft/speedup);
-			Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = sprintf(translatePlural("Number of builders.\nTasking another to this foundation would speed construction up by %(speedup)s second.", "Number of builders.\nTasking another to this foundation would speed construction up by %(speedup)s seconds.", timeSpeedup), { "speedup": timeSpeedup });
-		}
-		else
-			Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = translate("Number of builders.");
+		resourceCarryingIcon.sprite = "stretched:session/icons/repair.png";
+		resourceCarryingIcon.tooltip = getBuildTimeTooltip(entState);
+		resourceCarryingText.caption = entState.foundation.numBuilders ?
+			Engine.FormatMillisecondsIntoDateStringGMT(entState.foundation.buildTime.timeRemaining * 1000, translateWithContext("countdown format", "m:ss")) : "";
 	}
-	else if (entState.repairable && entState.repairable.numBuilders > 0 && entState.visibility == "visible")
+	else if (entState.resourceSupply && (!entState.resourceSupply.killBeforeGather || !entState.hitpoints))
 	{
-		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingIcon").sprite = "stretched:session/icons/repair.png";
-		Engine.GetGUIObjectByName("resourceCarryingText").caption = entState.repairable.numBuilders + "    ";
-		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = translate("Number of builders.");
-	}
-	else if (entState.resourceSupply && (!entState.resourceSupply.killBeforeGather || !entState.hitpoints) && entState.visibility == "visible")
-	{
-		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
-		Engine.GetGUIObjectByName("resourceCarryingIcon").sprite = "stretched:session/icons/repair.png";
-		Engine.GetGUIObjectByName("resourceCarryingText").caption = sprintf(translate("%(amount)s / %(max)s"), {
+		resourceCarryingIcon.sprite = "stretched:session/icons/repair.png";
+		resourceCarryingText.caption = sprintf(translate("%(amount)s / %(max)s"), {
 			"amount": entState.resourceSupply.numGatherers,
 			"max": entState.resourceSupply.maxGatherers
-		}) + "    ";
+		});
 		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = translate("Current/max gatherers");
 	}
+	else if (entState.repairable && entState.needsRepair)
+	{
+		resourceCarryingIcon.sprite = "stretched:session/icons/repair.png";
+		resourceCarryingIcon.tooltip = getRepairTimeTooltip(entState);
+		resourceCarryingText.caption = entState.repairable.numBuilders ?
+			Engine.FormatMillisecondsIntoDateStringGMT(entState.repairable.buildTime.timeRemaining * 1000, translateWithContext("countdown format", "m:ss")) : "";
+	}
 	else
 	{
-		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = true;
-		Engine.GetGUIObjectByName("resourceCarryingText").hidden = true;
+		resourceCarryingIcon.hidden = true;
+		resourceCarryingText.hidden = true;
 	}
 
 	Engine.GetGUIObjectByName("specific").caption = specificName;
 	Engine.GetGUIObjectByName("player").caption = playerName;
-	Engine.GetGUIObjectByName("playerColorBackground").sprite = "color: " + playerColor;
+	Engine.GetGUIObjectByName("playerColorBackground").sprite = "color:" + playerColor;
 	Engine.GetGUIObjectByName("generic").caption = genericName == specificName ? "" :
 		sprintf(translate("(%(genericName)s)"), {
 			"genericName": genericName
 		});
 
-	let isGaia = "gaia" == playerState.civ;
+	let isGaia = playerState.civ == "gaia";
 	Engine.GetGUIObjectByName("playerCivIcon").sprite = isGaia ? "" : "stretched:grayscale:" + civEmblem;
 	Engine.GetGUIObjectByName("player").tooltip = isGaia ? "" : civName;
 
 	// TODO: we should require all entities to have icons
 	Engine.GetGUIObjectByName("icon").sprite = template.icon ? ("stretched:session/portraits/" + template.icon) : "BackgroundBlack";
+	if (template.icon)
+		Engine.GetGUIObjectByName("iconBorder").onPressRight = () => {
+			showTemplateDetails(entState.template);
+		};
 
 	Engine.GetGUIObjectByName("attackAndArmorStats").tooltip = [
 		getAttackTooltip,
@@ -309,8 +289,6 @@ function displaySingle(entState)
 		getHealerTooltip,
 		getArmorTooltip,
 		getGatherTooltip,
-		getRepairRateTooltip,
-		getBuildRateTooltip,
 		getSpeedTooltip,
 		getGarrisonTooltip,
 		getProjectilesTooltip,
@@ -326,7 +304,8 @@ function displaySingle(entState)
 	iconTooltips = iconTooltips.concat([
 		getVisibleEntityClassesFormatted,
 		getAurasTooltip,
-		getEntityTooltip
+		getEntityTooltip,
+		showTemplateViewerOnRightClickTooltip
 	].map(func => func(template)));
 
 	Engine.GetGUIObjectByName("iconBorder").tooltip = iconTooltips.filter(tip => tip).join("\n");
@@ -361,12 +340,13 @@ function displayMultiple(entStates)
 		}
 
 		let carrying = calculateCarriedResources(
-			entState.resourceCarrying,
+			entState.resourceCarrying || null,
 			entState.trader && entState.trader.goods
 		);
 
-		for (let type in entState.loot)
-			totalLoot[type] = (totalLoot[type] || 0) + entState.loot[type];
+		if (entState.loot)
+			for (let type in entState.loot)
+				totalLoot[type] = (totalLoot[type] || 0) + entState.loot[type];
 
 		for (let type in carrying)
 		{
@@ -380,7 +360,7 @@ function displayMultiple(entStates)
 	{
 		let unitHealthBar = Engine.GetGUIObjectByName("healthBarMultiple");
 		let healthSize = unitHealthBar.size;
-		healthSize.rtop = 100-100*Math.max(0, Math.min(1, averageHealth / maxHealth));
+		healthSize.rtop = 100 - 100 * Math.max(0, Math.min(1, averageHealth / maxHealth));
 		unitHealthBar.size = healthSize;
 
 		Engine.GetGUIObjectByName("healthMultiple").tooltip = getCurrentHealthTooltip({
@@ -392,16 +372,16 @@ function displayMultiple(entStates)
 	Engine.GetGUIObjectByName("captureMultiple").hidden = maxCapturePoints <= 0;
 	if (maxCapturePoints > 0)
 	{
-		let setCaptureBarPart = function(playerID, startSize)
+		let setCaptureBarPart = function(pID, startSize)
 		{
-			let unitCaptureBar = Engine.GetGUIObjectByName("captureBarMultiple["+playerID+"]");
+			let unitCaptureBar = Engine.GetGUIObjectByName("captureBarMultiple[" + pID + "]");
 			let sizeObj = unitCaptureBar.size;
 			sizeObj.rtop = startSize;
 
-			let size = 100*Math.max(0, Math.min(1, capturePoints[playerID] / maxCapturePoints));
+			let size = 100 * Math.max(0, Math.min(1, capturePoints[pID] / maxCapturePoints));
 			sizeObj.rbottom = startSize + size;
 			unitCaptureBar.size = sizeObj;
-			unitCaptureBar.sprite = "color: " + rgbToGuiColor(g_Players[playerID].color, 128);
+			unitCaptureBar.sprite = "color:" + rgbToGuiColor(g_DisplayedPlayerColors[pID], 128);
 			unitCaptureBar.hidden = false;
 			return startSize + size;
 		};
@@ -414,11 +394,12 @@ function displayMultiple(entStates)
 		// last handle the owner's points, to keep those points on the bottom for clarity
 		setCaptureBarPart(playerID, size);
 
-		Engine.GetGUIObjectByName("captureMultiple").tooltip = getCurrentHealthTooltip({
+		Engine.GetGUIObjectByName("captureMultiple").tooltip = getCurrentHealthTooltip(
+			{
 				"hitpoints": capturePoints[playerID],
 				"maxHitpoints": maxCapturePoints
 			},
-			translate("Capture points:"));
+			translate("Capture Points:"));
 	}
 
 	let numberOfUnits = Engine.GetGUIObjectByName("numberOfUnits");
@@ -459,7 +440,7 @@ function updateSelectionDetails()
 
 	for (let sel of g_Selection.toList())
 	{
-		let entState = GetExtendedEntityState(sel);
+		let entState = GetEntityState(sel);
 		if (!entState)
 			continue;
 		entStates.push(entState);
@@ -494,20 +475,6 @@ function updateSelectionDetails()
 		updateGarrisonHealthBar(entStates[0], g_Selection.toList());
 }
 
-function getRankIconSprite(entState)
-{
-	if (entState.identity.rank == "Elite")
-		return "stretched:session/icons/rank3.png";
-
-	if (entState.identity.rank == "Advanced")
-		return "stretched:session/icons/rank2.png";
-
-	if (entState.identity.classes.indexOf("CitizenSoldier") != -1)
-		return "stretched:session/icons/rank1.png";
-
-	return "";
-}
-
 function tradingGainString(gain, owner)
 {
 	// Translation: Used in the trading gain tooltip
