@@ -36,23 +36,11 @@ let g_SelectionPanels = {};
 g_SelectionPanels.Alert = {
 	"getMaxNumberOfItems": function()
 	{
-		return 3;
+		return 2;
 	},
-	"conflictsWith": ["Barter"],
 	"getItems": function(unitEntStates)
 	{
-		let ret = [];
-
-		if (unitEntStates.some(state => state.alertRaiser && !state.alertRaiser.hasRaisedAlert))
-			ret.push("raise");
-
-		if (unitEntStates.some(state => state.alertRaiser && state.alertRaiser.hasRaisedAlert && state.alertRaiser.canIncreaseLevel))
-			ret.push("increase");
-
-		if (unitEntStates.some(state => state.alertRaiser && state.alertRaiser.hasRaisedAlert))
-			ret.push("end");
-
-		return ret;
+		return unitEntStates.some(state => !!state.alertRaiser) ? ["raise", "end"] : [];
 	},
 	"setupButton": function(data)
 	{
@@ -62,9 +50,6 @@ g_SelectionPanels.Alert = {
 			case "raise":
 				raiseAlert();
 				return;
-			case "increase":
-				increaseAlertLevel();
-				return;
 			case "end":
 				endOfAlert();
 				return;
@@ -77,10 +62,6 @@ g_SelectionPanels.Alert = {
 			data.icon.sprite = "stretched:session/icons/bell_level1.png";
 			data.button.tooltip = translate("Raise an alert!");
 			break;
-		case "increase":
-			data.icon.sprite = "stretched:session/icons/bell_level2.png";
-			data.button.tooltip = translate("Increase the alert level to protect more units");
-			break;
 		case "end":
 			data.button.tooltip = translate("End of alert.");
 			data.icon.sprite = "stretched:session/icons/bell_level0.png";
@@ -88,7 +69,7 @@ g_SelectionPanels.Alert = {
 		}
 		data.button.enabled = controlsPlayer(data.player);
 
-		setPanelObjectPosition(data.button, this.getMaxNumberOfItems() - data.i - 1, data.rowLength);
+		setPanelObjectPosition(data.button, this.getMaxNumberOfItems() - data.i, data.rowLength);
 		return true;
 	}
 };
@@ -99,7 +80,7 @@ g_SelectionPanels.Barter = {
 		return 4;
 	},
 	"rowLength": 4,
-	"conflictsWith": ["Alert", "Garrison"],
+	"conflictsWith": ["Garrison"],
 	"getItems": function(unitEntStates)
 	{
 		// If more than `rowLength` resources, don't display icons.
@@ -163,12 +144,11 @@ g_SelectionPanels.Command = {
 		data.icon.sprite = "stretched:session/icons/" + data.item.icon;
 
 		let size = data.button.size;
-		// count on square buttons, so size.bottom is the width too
-		let spacer = size.bottom + 1;
 		// relative to the center ( = 50%)
-		size.rleft = size.rright = 50;
-		// offset from the center calculation
-		size.left = (data.i - data.numberOfItems/2) * spacer;
+		size.rleft = 50;
+		size.rright = 50;
+		// offset from the center calculation, count on square buttons, so size.bottom is the width too
+		size.left = (data.i - data.numberOfItems / 2) * (size.bottom + 1);
 		size.right = size.left + size.bottom;
 		data.button.size = size;
 		return true;
@@ -185,7 +165,6 @@ g_SelectionPanels.AllyCommand = {
 	{
 		let commands = [];
 		for (let command in g_AllyEntityCommands)
-		{
 			for (let state of unitEntStates)
 			{
 				let info = g_AllyEntityCommands[command].getInfo(state);
@@ -196,7 +175,6 @@ g_SelectionPanels.AllyCommand = {
 					break;
 				}
 			}
-		}
 		return commands;
 	},
 	"setupButton": function(data)
@@ -218,12 +196,11 @@ g_SelectionPanels.AllyCommand = {
 		data.icon.sprite = "stretched:" + grayscale + "session/icons/" + data.item.icon;
 
 		let size = data.button.size;
-		// count on square buttons, so size.bottom is the width too
-		let spacer = size.bottom + 1;
 		// relative to the center ( = 50%)
-		size.rleft = size.rright = 50;
-		// offset from the center calculation
-		size.left = (data.i - data.numberOfItems/2) * spacer;
+		size.rleft = 50;
+		size.rright = 50;
+		// offset from the center calculation, count on square buttons, so size.bottom is the width too
+		size.left = (data.i - data.numberOfItems / 2) * (size.bottom + 1);
 		size.right = size.left + size.bottom;
 		data.button.size = size;
 
@@ -258,10 +235,8 @@ g_SelectionPanels.Construction = {
 				"player": data.player
 			});
 
-		if (template.wallSet)
-			template.auras = GetTemplateData(template.wallSet.templates.long).auras;
-
-		data.button.onPress = function () { startBuildingPlacement(data.item, data.playerState); };
+		data.button.onPress = function() { startBuildingPlacement(data.item, data.playerState); };
+		data.button.onPressRight = function() { showTemplateDetails(data.item); };
 
 		let tooltips = [
 			getEntityNamesFormatted,
@@ -270,7 +245,8 @@ g_SelectionPanels.Construction = {
 			getEntityTooltip,
 			getEntityCostTooltip,
 			getGarrisonTooltip,
-			getPopulationBonusTooltip
+			getPopulationBonusTooltip,
+			showTemplateViewerOnRightClickTooltip
 		].map(func => func(template));
 
 		let limits = getEntityLimitAndCount(data.playerState, data.item);
@@ -290,7 +266,7 @@ g_SelectionPanels.Construction = {
 		else if (neededResources)
 		{
 			data.button.enabled = false;
-			modifier += resourcesToAlphaMask(neededResources) +":";
+			modifier += resourcesToAlphaMask(neededResources) + ":";
 		}
 		else
 			data.button.enabled = controlsPlayer(data.player);
@@ -298,8 +274,7 @@ g_SelectionPanels.Construction = {
 		if (template.icon)
 			data.icon.sprite = modifier + "stretched:session/portraits/" + template.icon;
 
-		let index = data.i + getNumberOfRightPanelButtons();
-		setPanelObjectPosition(data.button, index, data.rowLength);
+		setPanelObjectPosition(data.button, data.i + getNumberOfRightPanelButtons(), data.rowLength);
 		return true;
 	}
 };
@@ -334,18 +309,19 @@ g_SelectionPanels.Formation = {
 
 		let formationInfo = g_FormationsInfo.get(data.item);
 		let formationOk = canMoveSelectionIntoFormation(data.item);
+		let unitIds = data.unitEntStates.map(state => state.id);
 		let formationSelected = Engine.GuiInterfaceCall("IsFormationSelected", {
-			"ents": data.unitEntStates.map(state => state.id),
+			"ents": unitIds,
 			"formationTemplate": data.item
 		});
 
 		data.button.onPress = function() {
-			performFormation(data.unitEntStates.map(state => state.id), data.item);
+			performFormation(unitIds, data.item);
 		};
 
 		let tooltip = translate(formationInfo.name);
 		if (!formationOk && formationInfo.tooltip)
-			tooltip += "\n" + "[color=\"red\"]" + translate(formationInfo.tooltip) + "[/color]";
+			tooltip += "\n" + coloredText(translate(formationInfo.tooltip), "red");
 		data.button.tooltip = tooltip;
 
 		data.button.enabled = formationOk && controlsPlayer(data.player);
@@ -392,27 +368,21 @@ g_SelectionPanels.Garrison = {
 
 		data.countDisplay.caption = data.item.ents.length || "";
 
-		let garrisonedUnitOwner = entState.player;
-
 		let canUngarrison =
 			g_ViewedPlayer == data.player ||
-			g_ViewedPlayer == garrisonedUnitOwner;
+			g_ViewedPlayer == entState.player;
 
 		data.button.enabled = canUngarrison && controlsPlayer(g_ViewedPlayer);
 
-		let tooltip = canUngarrison || g_IsObserver ?
-			sprintf(translate("Unload %(name)s"),
-			{ "name": getEntityNames(template) }) + "\n" +
+		data.button.tooltip = (canUngarrison || g_IsObserver ?
+			sprintf(translate("Unload %(name)s"), { "name": getEntityNames(template) }) + "\n" +
 			translate("Single-click to unload 1. Shift-click to unload all of this type.") :
-			getEntityNames(template);
+			getEntityNames(template)) + "\n" +
+			sprintf(translate("Player: %(playername)s"), {
+				"playername": g_Players[entState.player].name
+			});
 
-		tooltip += "\n" + sprintf(translate("Player: %(playername)s"), {
-			"playername": g_Players[garrisonedUnitOwner].name
-		});
-
-		data.button.tooltip = tooltip;
-
-		data.guiSelection.sprite = getPlayerHighlightColor(garrisonedUnitOwner);
+		data.guiSelection.sprite = getPlayerHighlightColor(entState.player);
 		data.button.sprite_disabled = data.button.sprite;
 
 		// Selection panel buttons only appear disabled if they
@@ -434,54 +404,36 @@ g_SelectionPanels.Gate = {
 	},
 	"getItems": function(unitEntStates)
 	{
-		let gates = [];
-		for (let state of unitEntStates)
-		{
-			if (state.gate && !gates.length)
-			{
-				gates.push({
-					"gate": state.gate,
-					"tooltip": translate("Lock Gate"),
-					"locked": true,
-					"callback": function (item) { lockGate(item.locked); }
-				},
-				{
-					"gate": state.gate,
-					"tooltip": translate("Unlock Gate"),
-					"locked": false,
-					"callback": function (item) { lockGate(item.locked); }
-				});
-			}
-			// Show both 'locked' and 'unlocked' as active if the selected gates have both lock states.
-			else if (state.gate && state.gate.locked != gates[0].gate.locked)
-				for (let j = 0; j < gates.length; ++j)
-					delete gates[j].gate.locked;
-		}
+		let hideLocked = unitEntStates.every(state => !state.gate || !state.gate.locked);
+		let hideUnlocked = unitEntStates.every(state => !state.gate || state.gate.locked);
 
-		return gates;
+		if (hideLocked && hideUnlocked)
+			return [];
+
+		return [
+			{
+				"hidden": hideLocked,
+				"tooltip": translate("Lock Gate"),
+				"icon": "session/icons/lock_locked.png",
+				"locked": true
+			},
+			{
+				"hidden": hideUnlocked,
+				"tooltip": translate("Unlock Gate"),
+				"icon": "session/icons/lock_unlocked.png",
+				"locked": false
+			}
+		];
 	},
 	"setupButton": function(data)
 	{
-		data.button.onPress = function() {data.item.callback(data.item); };
-
+		data.button.onPress = function() { lockGate(data.item.locked); };
 		data.button.tooltip = data.item.tooltip;
-
 		data.button.enabled = controlsPlayer(data.player);
-		let gateIcon;
-		if (data.item.gate)
-		{
-			// show locking actions
-			gateIcon = "icons/lock_" + GATE_ACTIONS[data.item.locked ? 0 : 1] + "ed.png";
-			if (data.item.gate.locked === undefined)
-				data.guiSelection.hidden = false;
-			else
-				data.guiSelection.hidden = data.item.gate.locked != data.item.locked;
-		}
+		data.guiSelection.hidden = data.item.hidden;
+		data.icon.sprite = "stretched:" + data.item.icon;
 
-		data.icon.sprite = (data.neededResources ? resourcesToAlphaMask(data.neededResources) + ":" : "") + "stretched:session/" + gateIcon;
-
-		let index = data.i + getNumberOfRightPanelButtons();
-		setPanelObjectPosition(data.button, index, data.rowLength);
+		setPanelObjectPosition(data.button, data.i + getNumberOfRightPanelButtons(), data.rowLength);
 		return true;
 	}
 };
@@ -506,13 +458,10 @@ g_SelectionPanels.Pack = {
 				else
 					checks.packButton = true;
 			}
+			else if (state.pack.packed)
+				checks.unpackCancelButton = true;
 			else
-			{
-				if (state.pack.packed)
-					checks.unpackCancelButton = true;
-				else
-					checks.packCancelButton = true;
-			}
+				checks.packCancelButton = true;
 		}
 
 		let items = [];
@@ -565,8 +514,7 @@ g_SelectionPanels.Pack = {
 
 		data.button.enabled = controlsPlayer(data.player);
 
-		let index = data.i + getNumberOfRightPanelButtons();
-		setPanelObjectPosition(data.button, index, data.rowLength);
+		setPanelObjectPosition(data.button, data.i + getNumberOfRightPanelButtons(), data.rowLength);
 		return true;
 	}
 };
@@ -592,9 +540,10 @@ g_SelectionPanels.Queue = {
 			{
 				if (!state.production || !state.production.queue[i])
 					continue;
-				let item = state.production.queue[i];
-				item.producingEnt = state.id;
-				queue.push(item);
+				queue.push({
+					"producingEnt": state.id,
+					"queuedItem": state.production.queue[i]
+				});
 				foundNew = true;
 			}
 		}
@@ -607,45 +556,50 @@ g_SelectionPanels.Queue = {
 		let size = panel.size;
 		let buttonSize = Engine.GetGUIObjectByName("unitQueueButton[0]").size.bottom;
 		let margin = 4;
-		size.top = size.bottom - numRows*buttonSize - (numRows+2)*margin;
+		size.top = size.bottom - numRows * buttonSize - (numRows + 2) * margin;
 		panel.size = size;
 	},
 	"setupButton": function(data)
 	{
+		let queuedItem = data.item.queuedItem;
+
 		// Differentiate between units and techs
 		let template;
-		if (data.item.unitTemplate)
-			template = GetTemplateData(data.item.unitTemplate);
-		else if (data.item.technologyTemplate)
-			template = GetTechnologyData(data.item.technologyTemplate);
-
-		if (!template)
+		if (queuedItem.unitTemplate)
+			template = GetTemplateData(queuedItem.unitTemplate);
+		else if (queuedItem.technologyTemplate)
+			template = GetTechnologyData(queuedItem.technologyTemplate, GetSimState().players[data.player].civ);
+		else
+		{
+			warning("Unknown production queue template " + uneval(queuedItem));
 			return false;
+		}
 
-		data.button.onPress = function() { removeFromProductionQueue(data.item.producingEnt, data.item.id); };
+		data.button.onPress = function() { removeFromProductionQueue(data.item.producingEnt, queuedItem.id); };
 
 		let tooltip = getEntityNames(template);
-		if (data.item.neededSlots)
+		if (queuedItem.neededSlots)
 		{
-			tooltip += "\n[color=\"red\"]" + translate("Insufficient population capacity:") + "\n[/color]";
-			tooltip += sprintf(translate("%(population)s %(neededSlots)s"), {
+			tooltip += "\n" + coloredText(translate("Insufficient population capacity:"), "red");
+			tooltip += "\n" + sprintf(translate("%(population)s %(neededSlots)s"), {
 				"population": resourceIcon("population"),
-				"neededSlots": data.item.neededSlots
+				"neededSlots": queuedItem.neededSlots
 			});
 		}
 		data.button.tooltip = tooltip;
 
-		data.countDisplay.caption = data.item.count > 1 ? data.item.count : "";
+		data.countDisplay.caption = queuedItem.count > 1 ? queuedItem.count : "";
 
 		// Show the time remaining to finish the first item
 		if (data.i == 0)
-			Engine.GetGUIObjectByName("queueTimeRemaining").caption = Engine.FormatMillisecondsIntoDateStringGMT(data.item.timeRemaining, translateWithContext("countdown format", "m:ss"));
+			Engine.GetGUIObjectByName("queueTimeRemaining").caption =
+				Engine.FormatMillisecondsIntoDateStringGMT(queuedItem.timeRemaining, translateWithContext("countdown format", "m:ss"));
 
-		let guiObject = Engine.GetGUIObjectByName("unitQueueProgressSlider["+data.i+"]");
+		let guiObject = Engine.GetGUIObjectByName("unitQueueProgressSlider[" + data.i + "]");
 		let size = guiObject.size;
 
 		// Buttons are assumed to be square, so left/right offsets can be used for top/bottom.
-		size.top = size.left + Math.round(data.item.progress * (size.right - size.left));
+		size.top = size.left + Math.round(queuedItem.progress * (size.right - size.left));
 		guiObject.size = size;
 
 		if (template.icon)
@@ -689,7 +643,7 @@ g_SelectionPanels.Research = {
 							item.tech.top == tech.top) &&
 						Object.keys(item.techCostMultiplier).every(
 							k => item.techCostMultiplier[k] == state.production.techCostMultiplier[k])
-			));
+				));
 
 			if (filteredTechs.length + ret.length <= this.getMaxNumberOfItems() &&
 			    getNumberOfRightPanelButtons() <= this.getMaxNumberOfItems() * (filteredTechs.some(tech => !!tech.pair) ? 1 : 2))
@@ -715,7 +669,6 @@ g_SelectionPanels.Research = {
 			g_SelectionPanels.Research.hideItem(data.i, data.rowLength);
 			return false;
 		}
-		let techs = data.item.tech.pair ? [data.item.tech.bottom, data.item.tech.top] : [data.item.tech];
 
 		// Start position (start at the bottom)
 		let position = data.i + data.rowLength;
@@ -731,11 +684,9 @@ g_SelectionPanels.Research = {
 
 		// Handle one or two techs (tech pair)
 		let player = data.player;
-		for (let i in techs)
+		let playerState = GetSimState().players[player];
+		for (let tech of data.item.tech.pair ? [data.item.tech.bottom, data.item.tech.top] : [data.item.tech])
 		{
-			let tech = techs[i];
-			let playerState = GetSimState().players[player];
-
 			// Don't change the object returned by GetTechnologyData
 			let template = clone(GetTechnologyData(tech, playerState.civ));
 			if (!template)
@@ -760,7 +711,8 @@ g_SelectionPanels.Research = {
 			let tooltips = [
 				getEntityNamesFormatted,
 				getEntityTooltip,
-				getEntityCostTooltip
+				getEntityCostTooltip,
+				showTemplateViewerOnRightClickTooltip
 			].map(func => func(template));
 
 			if (!requirementsPassed)
@@ -807,20 +759,25 @@ g_SelectionPanels.Research = {
 			tooltips.push(getNeededResourcesTooltip(neededResources));
 			button.tooltip = tooltips.filter(tip => tip).join("\n");
 
-			button.onPress = function () {
-				addResearchToQueue(data.item.researchFacilityId, tech);
-			};
+			button.onPress = (t => function() {
+				addResearchToQueue(data.item.researchFacilityId, t);
+			})(tech);
+
+			button.onPressRight = (t => function () {
+				showTemplateDetails(
+					t,
+					GetTemplateData(data.unitEntStates.find(state => state.id == data.item.researchFacilityId).template).nativeCiv);
+			})(tech);
 
 			if (data.item.tech.pair)
 			{
 				// On mouse enter, show a cross over the other icon
-				let otherPosition = (position + data.rowLength) % (2 * data.rowLength);
-				let unchosenIcon = Engine.GetGUIObjectByName("unitResearchUnchosenIcon[" + otherPosition + "]");
+				let unchosenIcon = Engine.GetGUIObjectByName("unitResearchUnchosenIcon[" + (position + data.rowLength) % (2 * data.rowLength) + "]");
 				button.onMouseEnter = function() {
-						unchosenIcon.hidden = false;
+					unchosenIcon.hidden = false;
 				};
 				button.onMouseLeave = function() {
-						unchosenIcon.hidden = true;
+					unchosenIcon.hidden = true;
 				};
 			}
 
@@ -942,17 +899,18 @@ g_SelectionPanels.Stance = {
 		if (unitEntStates.some(state => !state.unitAI || !hasClass(state, "Unit") || hasClass(state, "Animal")))
 			return [];
 
-		return unitEntStates[0].unitAI.possibleStances;
+		return unitEntStates[0].unitAI.selectableStances;
 	},
 	"setupButton": function(data)
 	{
-		data.button.onPress = function() { performStance(data.unitEntStates.map(state => state.id), data.item); };
+		let unitIds = data.unitEntStates.map(state => state.id);
+		data.button.onPress = function() { performStance(unitIds, data.item); };
 
 		data.button.tooltip = getStanceDisplayName(data.item) + "\n" +
 			"[font=\"sans-13\"]" + getStanceTooltip(data.item) + "[/font]";
 
 		data.guiSelection.hidden = !Engine.GuiInterfaceCall("IsStanceSelected", {
-			"ents": data.unitEntStates.map(state => state.id),
+			"ents": unitIds,
 			"stance": data.item
 		});
 		data.icon.sprite = "stretched:session/icons/stances/" + data.item + ".png";
@@ -983,8 +941,9 @@ g_SelectionPanels.Training = {
 			"player": data.player
 		});
 
+		let unitIds = data.unitEntStates.map(status => status.id)
 		let [buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch] =
-			getTrainingStatus(data.playerState, data.item, data.unitEntStates.map(status => status.id));
+			getTrainingStatus(unitIds, data.item, data.playerState);
 
 		let trainNum = buildingsCountToTrainFullBatch * fullBatchSize + remainderBatch;
 
@@ -996,7 +955,11 @@ g_SelectionPanels.Training = {
 			});
 
 		data.button.onPress = function() {
-			addTrainingToQueue(data.unitEntStates.map(state => state.id), data.item, data.playerState);
+			if (!neededResources)
+				addTrainingToQueue(unitIds, data.item, data.playerState);
+		};
+		data.button.onPressRight = function() {
+			showTemplateDetails(data.item);
 		};
 
 		data.countDisplay.caption = trainNum > 1 ? trainNum : "";
@@ -1008,7 +971,7 @@ g_SelectionPanels.Training = {
 			getVisibleEntityClassesFormatted(template),
 			getAurasTooltip(template),
 			getEntityTooltip(template),
-			getEntityCostTooltip(template, data.unitEntStates[0].id, buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch)
+			getEntityCostTooltip(template, unitIds[0], buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch)
 		];
 
 		let limits = getEntityLimitAndCount(data.playerState, data.item);
@@ -1026,10 +989,9 @@ g_SelectionPanels.Training = {
 				getSpeedTooltip
 			].map(func => func(template)));
 
+		tooltips.push(showTemplateViewerOnRightClickTooltip());
 		tooltips.push(
-			"[color=\"" + g_HotkeyColor + "\"]" +
-			formatBatchTrainingString(buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch) +
-			"[/color]",
+			formatBatchTrainingString(buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch),
 			getRequiredTechnologyTooltip(technologyEnabled, template.requiredTechnology, GetSimState().players[data.player].civ),
 			getNeededResourcesTooltip(neededResources));
 
@@ -1041,16 +1003,15 @@ g_SelectionPanels.Training = {
 			data.button.enabled = false;
 			modifier = "color:0 0 0 127:grayscale:";
 		}
-		else if (neededResources)
-		{
-			data.button.enabled = false;
-			modifier = resourcesToAlphaMask(neededResources) +":";
-		}
 		else
+		{
 			data.button.enabled = controlsPlayer(data.player);
+			if (neededResources)
+				modifier = resourcesToAlphaMask(neededResources) + ":";
+		}
 
 		if (template.icon)
-		data.icon.sprite = modifier + "stretched:session/portraits/" + template.icon;
+			data.icon.sprite = modifier + "stretched:session/portraits/" + template.icon;
 
 		let index = data.i + getNumberOfRightPanelButtons();
 		setPanelObjectPosition(data.button, index, data.rowLength);
@@ -1072,7 +1033,7 @@ g_SelectionPanels.Upgrade = {
 
 		return unitEntStates[0].upgrade && unitEntStates[0].upgrade.upgrades;
 	},
-	"setupButton" : function(data)
+	"setupButton": function(data)
 	{
 		let template = GetTemplateData(data.item.entity);
 		if (!template)
@@ -1086,18 +1047,10 @@ g_SelectionPanels.Upgrade = {
 				"player": data.player
 			});
 
-		let neededResources;
-		if (data.item.cost)
-		{
-			for (let cost in data.item.cost)
-				if (cost != "time")
-					data.item.cost[cost] *= data.unitEntStates.length;
-
-			neededResources = Engine.GuiInterfaceCall("GetNeededResources", {
-				"cost": data.item.cost,
-				"player": data.player
-			});
-		}
+		let neededResources = data.item.cost && Engine.GuiInterfaceCall("GetNeededResources", {
+			"cost": multiplyEntityCosts(data.item, data.unitEntStates.length),
+			"player": data.player
+		});
 
 		let limits = getEntityLimitAndCount(data.playerState, data.item.entity);
 		let progress = data.unitEntStates[0].upgrade.progress || 0;
@@ -1118,10 +1071,11 @@ g_SelectionPanels.Upgrade = {
 				}));
 
 			tooltips.push(
-				getEntityCostTooltip(data.item),
+				getEntityCostComponentsTooltipString(data.item, undefined, data.unitEntStates.length),
 				formatLimitString(limits.entLimit, limits.entCount, limits.entLimitChangers),
 				getRequiredTechnologyTooltip(technologyEnabled, data.item.requiredTechnology, GetSimState().players[data.player].civ),
-				getNeededResourcesTooltip(neededResources));
+				getNeededResourcesTooltip(neededResources),
+				showTemplateViewerOnRightClickTooltip());
 
 			tooltip = tooltips.filter(tip => tip).join("\n");
 
@@ -1140,9 +1094,12 @@ g_SelectionPanels.Upgrade = {
 		data.button.enabled = controlsPlayer(data.player);
 		data.button.tooltip = tooltip;
 
+		data.button.onPressRight = function() {
+			showTemplateDetails(data.item.entity);
+		};
+
 		let modifier = "";
 		if (!isUpgrading)
-		{
 			if (progress || !technologyEnabled || limits.canBeAddedCount == 0 &&
 				!hasSameRestrictionCategory(data.item.entity, data.unitEntStates[0].template))
 			{
@@ -1154,7 +1111,6 @@ g_SelectionPanels.Upgrade = {
 				data.button.enabled = false;
 				modifier = resourcesToAlphaMask(neededResources) + ":";
 			}
-		}
 
 		data.icon.sprite = modifier + "stretched:session/" +
 			(data.item.icon || "portraits/" + template.icon);
@@ -1170,12 +1126,30 @@ g_SelectionPanels.Upgrade = {
 		}
 		progressOverlay.hidden = !isUpgrading;
 
-		let index = data.i + getNumberOfRightPanelButtons();
-		setPanelObjectPosition(data.button, index, data.rowLength);
+		setPanelObjectPosition(data.button, data.i + getNumberOfRightPanelButtons(), data.rowLength);
 		return true;
 	}
 };
 
+/**
+ * Pauses game and opens the template details viewer for a selected entity or technology.
+ *
+ * Technologies don't have a set civ, so we pass along the native civ of
+ * the template of the entity that's researching it.
+ *
+ * @param {string} [civCode] - The template name of the entity that researches the selected technology.
+ */
+function showTemplateDetails(templateName, civCode)
+{
+	pauseGame();
+
+	Engine.PushGuiPage("page_viewer.xml", {
+		"templateName": templateName,
+		"callback": "resumeGame",
+		"civ": civCode
+	});
+}
+
 /**
  * If two panels need the same space, so they collide,
  * the one appearing first in the order is rendered.
