@@ -287,16 +287,6 @@ function init(initData, hotloadData)
 	sendLobbyPlayerlistUpdate();
 	onSimulationUpdate();
 	setTimeout(displayGamestateNotifications, 1000);
-
-	// Report the performance after 5 seconds (when we're still near
-	// the initial camera view) and a minute (when the profiler will
-	// have settled down if framerates as very low), to give some
-	// extremely rough indications of performance
-	//
-	// DISABLED: this information isn't currently useful for anything much,
-	// and it generates a massive amount of data to transmit and store
-	// setTimeout(function() { reportPerformance(5); }, 5000);
-	// setTimeout(function() { reportPerformance(60); }, 60000);
 }
 
 function initGUIObjects()
@@ -713,18 +703,6 @@ function updateTopPanel()
 	Engine.GetGUIObjectByName("lobbyButton").enabled = Engine.HasXmppClient();
 }
 
-function reportPerformance(time)
-{
-	let settings = g_GameAttributes.settings;
-	Engine.SubmitUserReport("profile", 3, JSON.stringify({
-		"time": time,
-		"map": settings.Name,
-		"seed": settings.Seed, // only defined for random maps
-		"size": settings.Size, // only defined for random maps
-		"profiler": Engine.GetProfilerState()
-	}));
-}
-
 /**
  * Resign a player.
  * @param leaveGameAfterResign If player is quitting after resignation.
@@ -1589,6 +1567,7 @@ function sendLobbyPlayerlistUpdate()
 
 /**
  * Send a report on the gamestatus to the lobby.
+ * Keep in sync with source/tools/XpartaMuPP/LobbyRanking.py
  */
 function reportGame()
 {
