@@ -1,20 +1,20 @@
-const g_MaxHeadingTitle= 9;
+const g_CivData = loadCivData(false, false);
 
-// const for filtering long collective headings
-const g_LongHeadingWidth = 250;
+var g_ScorePanelsData;
 
-const g_PlayerBoxYSize = 40;
-const g_PlayerBoxGap = 2;
-const g_PlayerBoxAlpha = " 50";
-const g_PlayerColorBoxAlpha = " 255";
-const g_TeamsBoxYStart = 40;
+var g_MaxHeadingTitle = 9;
+var g_LongHeadingWidth = 250;
+var g_PlayerBoxYSize = 40;
+var g_PlayerBoxGap = 2;
+var g_PlayerBoxAlpha = 50;
+var g_TeamsBoxYStart = 40;
 
-const g_TypeColors = {
+var g_TypeColors = {
 	"blue": "196 198 255",
 	"green": "201 255 200",
 	"red": "255 213 213",
 	"yellow": "255 255 157"
-}
+};
 
 /**
  * Colors, captions and format used for units, buildings, etc. types
@@ -60,6 +60,11 @@ var g_SummaryTypes = {
 		"caption": translate("Captured"),
 		"postfix": " / "
 	},
+	"succeeded": {
+		"color": g_TypeColors.green,
+		"caption": translate("Succeeded"),
+		"postfix": " / "
+	},
 	"destroyed": {
 		"color": g_TypeColors.blue,
 		"caption": translate("Destroyed"),
@@ -73,59 +78,128 @@ var g_SummaryTypes = {
 	"lost": {
 		"color": g_TypeColors.red,
 		"caption": translate("Lost"),
-		"postfix": "\n"
+		"postfix": ""
 	},
 	"used": {
 		"color": g_TypeColors.red,
 		"caption": translate("Used"),
-		"postfix": "\n"
+		"postfix": ""
 	},
 	"received": {
 		"color": g_TypeColors.red,
 		"caption": translate("Received"),
-		"postfix": "\n"
+		"postfix": ""
 	},
 	"sold": {
 		"color": g_TypeColors.red,
 		"caption": translate("Sold"),
-		"postfix": "\n"
+		"postfix": ""
 	},
 	"outcome": {
 		"color": g_TypeColors.red,
 		"caption": translate("Outcome"),
-		"postfix": "\n"
+		"postfix": ""
+	},
+	"failed": {
+		"color": g_TypeColors.red,
+		"caption": translate("Failed"),
+		"postfix": ""
 	}
 };
 
-const g_InfinitySymbol = "\u221E";
+// Translation: Unicode encoded infinity symbol indicating a division by zero in the summary screen.
+var g_InfinitySymbol = translate("\u221E");
 
-var g_CivData = loadCivData();
 var g_Teams = [];
 
 // TODO set g_PlayerCount as playerCounters.length
 var g_PlayerCount = 0;
 
-// Count players without team (or all if teams are not displayed)
-var g_WithoutTeam = 0;
 var g_GameData;
 var g_ResourceData = new Resources();
 
-// Selected chart indexes
+/**
+ * Selected chart indexes.
+ */
 var g_SelectedChart = {
 	"category": [0, 0],
-	"value": [0, 0],
+	"value": [0, 1],
 	"type": [0, 0]
 };
 
+/**
+ * Array of the panel button names.
+ */
+var g_PanelButtons = [];
+
+/**
+ * Remember the name of the currently opened view panel.
+ */
+var g_SelectedPanel = "";
+
+function init(data)
+{
+	// Fill globals
+	g_GameData = data;
+	g_ScorePanelsData = getScorePanelsData();
+	g_PanelButtons = Object.keys(g_ScorePanelsData).concat(["charts"]).map(panel => panel + "PanelButton");
+
+	g_SelectedPanel = g_PanelButtons[0];
+	if (data && data.selectedData)
+	{
+		g_SelectedPanel = data.selectedData.panel;
+		g_SelectedChart = data.selectedData.charts;
+	}
+
+	initTeamData();
+	calculateTeamCounterDataHelper();
+
+	// Output globals
+	initGUIWindow();
+	initPlayerBoxPositions();
+	initGUICharts();
+	initGUILabels();
+	initGUIButtons();
+
+	selectPanel(Engine.GetGUIObjectByName(g_SelectedPanel));
+	for (let button of g_PanelButtons)
+	{
+		let tab = Engine.GetGUIObjectByName(button);
+		tab.onMouseWheelUp = () => selectNextTab(1);
+		tab.onMouseWheelDown = () => selectNextTab(-1);
+	}
+}
+
+/**
+ * Sets the style and title of the page.
+ */
+function initGUIWindow()
+{
+	let summaryWindow = Engine.GetGUIObjectByName("summaryWindow");
+	summaryWindow.sprite = g_GameData.gui.dialog ? "ModernDialog" : "ModernWindow";
+	summaryWindow.size = g_GameData.gui.dialog ? "16 24 100%-16 100%-24" : "0 0 100% 100%";
+	Engine.GetGUIObjectByName("summaryWindowTitle").size = g_GameData.gui.dialog ? "50%-128 -16 50%+128 16" : "50%-128 4 50%+128 36";
+}
+
+/**
+ * Show next/previous panel.
+ * @param direction - 1/-1 forward, backward panel.
+ */
+function selectNextTab(direction)
+{
+	selectPanel(Engine.GetGUIObjectByName(g_PanelButtons[
+		(g_PanelButtons.indexOf(g_SelectedPanel) + direction + g_PanelButtons.length) % g_PanelButtons.length]));
+}
+
 function selectPanel(panel)
 {
 	// TODO: move panel buttons to a custom parent object
 
 	for (let button of Engine.GetGUIObjectByName("summaryWindow").children)
 		if (button.name.endsWith("PanelButton"))
-			button.sprite = "BackgroundTab";
+			button.sprite = "ModernTabHorizontalBackground";
 
-	panel.sprite = "ForegroundTab";
+	panel.sprite = "ModernTabHorizontalForeground";
 
 	adjustTabDividers(panel.size);
 
@@ -138,9 +212,11 @@ function selectPanel(panel)
 		updatePanelData(g_ScorePanelsData[panel.name.substr(0, panel.name.length - "PanelButton".length)]);
 	else
 		[0, 1].forEach(updateCategoryDropdown);
+
+	g_SelectedPanel = panel.name;
 }
 
-function initCharts()
+function initGUICharts()
 {
 	let player_colors = [];
 	for (let i = 1; i <= g_PlayerCount; ++i)
@@ -153,12 +229,13 @@ function initCharts()
 		);
 	}
 
-	[0, 1].forEach(i => Engine.GetGUIObjectByName("chart[" + i + "]").series_color = player_colors);
+	for (let i = 0; i < 2; ++i)
+		Engine.GetGUIObjectByName("chart[" + i + "]").series_color = player_colors;
 
 	let chartLegend = Engine.GetGUIObjectByName("chartLegend");
 	chartLegend.caption = g_GameData.sim.playerStates.slice(1).map(
-		(state, index) => '[color="' + player_colors[index] + '"]■[/color] ' + state.name
-	).join("  ");
+		(state, index) => coloredText("■", player_colors[index]) + " " + state.name
+	).join("   ");
 
 	let chart1Part = Engine.GetGUIObjectByName("chart[1]Part");
 	let chart1PartSize = chart1Part.size;
@@ -192,7 +269,7 @@ function updateCategoryDropdown(number)
 			g_SelectedChart.value[number] = 0;
 			g_SelectedChart.type[number] = 0;
 		}
-		
+
 		resizeDropdown(this);
 		updateValueDropdown(number, this.list_data[this.selected]);
 	};
@@ -254,6 +331,8 @@ function updateChart(number, category, item, itemNumber, type)
 	if (!g_ScorePanelsData[category].counters[itemNumber].fn)
 		return;
 	let chart = Engine.GetGUIObjectByName("chart[" + number + "]");
+	chart.format_y = g_ScorePanelsData[category].headings[itemNumber + 1].format || "INTEGER";
+	Engine.GetGUIObjectByName("chart[" + number + "]XAxisLabel").caption = translate("Time elapsed");
 	let series = [];
 	for (let j = 1; j <= g_PlayerCount; ++j)
 	{
@@ -294,23 +373,23 @@ function adjustTabDividers(tabSize)
 function updatePanelData(panelInfo)
 {
 	resetGeneralPanel();
-	resetDataHelpers();
 	updateGeneralPanelHeadings(panelInfo.headings);
 	updateGeneralPanelTitles(panelInfo.titleHeadings);
 	let rowPlayerObjectWidth = updateGeneralPanelCounter(panelInfo.counters);
 	updateGeneralPanelTeams();
 
-	let playerBoxesCounts = [ ];
+	let index = g_GameData.sim.playerStates[1].sequences.time.length - 1;
+	let playerBoxesCounts = [];
 	for (let i = 0; i < g_PlayerCount; ++i)
 	{
-		let playerState = g_GameData.sim.playerStates[i+1];
+		let playerState = g_GameData.sim.playerStates[i + 1];
 
-		if (!playerBoxesCounts[playerState.team+1])
-			playerBoxesCounts[playerState.team+1] = 1;
+		if (!playerBoxesCounts[playerState.team + 1])
+			playerBoxesCounts[playerState.team + 1] = 1;
 		else
-			playerBoxesCounts[playerState.team+1] += 1;
+			playerBoxesCounts[playerState.team + 1] += 1;
 
-		let positionObject = playerBoxesCounts[playerState.team+1] - 1;
+		let positionObject = playerBoxesCounts[playerState.team + 1] - 1;
 		let rowPlayer = "playerBox[" + positionObject + "]";
 		let playerOutcome = "playerOutcome[" + positionObject + "]";
 		let playerNameColumn = "playerName[" + positionObject + "]";
@@ -333,7 +412,7 @@ function updatePanelData(panelInfo)
 
 		let rowPlayerObject = Engine.GetGUIObjectByName(rowPlayer);
 		rowPlayerObject.hidden = false;
-		rowPlayerObject.sprite = colorString + g_PlayerBoxAlpha;
+		rowPlayerObject.sprite = colorString + " " + g_PlayerBoxAlpha;
 
 		let boxSize = rowPlayerObject.size;
 		boxSize.right = rowPlayerObjectWidth;
@@ -341,55 +420,48 @@ function updatePanelData(panelInfo)
 
 		setOutcomeIcon(playerState.state, playerOutcome);
 
-		Engine.GetGUIObjectByName(playerNameColumn).caption = g_GameData.sim.playerStates[i+1].name;
+		playerNameColumn = Engine.GetGUIObjectByName(playerNameColumn);
+		playerNameColumn.caption = g_GameData.sim.playerStates[i + 1].name;
+		playerNameColumn.tooltip = translateAISettings(g_GameData.sim.mapSettings.PlayerData[i + 1]);
 
 		let civIcon = Engine.GetGUIObjectByName(playerCivicBoxColumn);
 		civIcon.sprite = "stretched:" + g_CivData[playerState.civ].Emblem;
 		civIcon.tooltip = g_CivData[playerState.civ].Name;
 
-		updateCountersPlayer(playerState, panelInfo.counters, panelInfo.headings, playerCounterValue);
-
-		calculateTeamCounters(playerState);
+		updateCountersPlayer(playerState, panelInfo.counters, panelInfo.headings, playerCounterValue, index);
 	}
 
 	let teamCounterFn = panelInfo.teamCounterFn;
 	if (g_Teams && teamCounterFn)
-		teamCounterFn(panelInfo.counters, g_GameData.sim.playerStates[1].sequences.time.length - 1);
-}
-
-function confirmStartReplay()
-{
-	if (Engine.HasXmppClient())
-		messageBox(
-			400, 200,
-			translate("Are you sure you want to quit the lobby?"),
-			translate("Confirmation"),
-			[translate("No"), translate("Yes")],
-			[null, startReplay]
-		);
-	else
-		startReplay();
+		updateCountersTeam(teamCounterFn, panelInfo.counters, panelInfo.headings, index);
 }
 
 function continueButton()
 {
+	let summarySelectedData = {
+		"panel": g_SelectedPanel,
+		"charts": g_SelectedChart
+	};
 	if (g_GameData.gui.isInGame)
-		Engine.PopGuiPageCB(0);
+		Engine.PopGuiPageCB({
+			"explicitResume": 0,
+			"summarySelectedData": summarySelectedData
+		});
+	else if (g_GameData.gui.dialog)
+		Engine.PopGuiPage();
+	else if (Engine.HasXmppClient())
+		Engine.SwitchGuiPage("page_lobby.xml", { "dialog": false });
 	else if (g_GameData.gui.isReplay)
 		Engine.SwitchGuiPage("page_replaymenu.xml", {
-			"replaySelectionData": g_GameData.gui.replaySelectionData
+			"replaySelectionData": g_GameData.gui.replaySelectionData,
+			"summarySelectedData": summarySelectedData
 		});
-	else if (Engine.HasXmppClient())
-		Engine.SwitchGuiPage("page_lobby.xml");
 	else
 		Engine.SwitchGuiPage("page_pregame.xml");
 }
 
 function startReplay()
 {
-	if (Engine.HasXmppClient())
-		Engine.StopXmppClient();
-
 	if (!Engine.StartVisualReplay(g_GameData.gui.replayDirectory))
 	{
 		warn("Replay file not found!");
@@ -398,7 +470,6 @@ function startReplay()
 
 	Engine.SwitchGuiPage("page_loading.xml", {
 		"attribs": Engine.GetReplayAttributes(g_GameData.gui.replayDirectory),
-		"isNetworked": false,
 		"playerAssignments": {
 			"local": {
 				"name": singleplayerName(),
@@ -411,9 +482,8 @@ function startReplay()
 	});
 }
 
-function init(data)
+function initGUILabels()
 {
-	g_GameData = data;
 	let assignedState = g_GameData.sim.playerStates[g_GameData.gui.assignedPlayer || -1];
 
 	Engine.GetGUIObjectByName("summaryText").caption =
@@ -428,17 +498,15 @@ function init(data)
 		assignedState.state == "won" ?
 			translate("You have won the battle!") :
 		assignedState.state == "defeated" ?
-			translate("You have been defeated...") :
+			translate("You have been defeated…") :
 			translate("You have abandoned the game.");
 
-	initPlayerBoxPositions();
-
 	Engine.GetGUIObjectByName("timeElapsed").caption = sprintf(
 		translate("Game time elapsed: %(time)s"), {
 			"time": timeToString(g_GameData.sim.timeElapsed)
 	});
 
-	let mapType = g_Settings.MapTypes.find(mapType => mapType.Name == g_GameData.sim.mapSettings.mapType);
+	let mapType = g_Settings.MapTypes.find(type => type.Name == g_GameData.sim.mapSettings.mapType);
 	let mapSize = g_Settings.MapSizes.find(size => size.Tiles == g_GameData.sim.mapSettings.Size || 0);
 
 	Engine.GetGUIObjectByName("mapName").caption = sprintf(
@@ -446,22 +514,42 @@ function init(data)
 			"mapName": translate(g_GameData.sim.mapSettings.Name),
 			"mapType": mapSize ? mapSize.Name : (mapType ? mapType.Title : "")
 		});
+}
 
-	Engine.GetGUIObjectByName("replayButton").hidden = g_GameData.gui.isInGame || !g_GameData.gui.replayDirectory;
+function initGUIButtons()
+{
+	let replayButton = Engine.GetGUIObjectByName("replayButton");
+	replayButton.hidden = g_GameData.gui.isInGame || !g_GameData.gui.replayDirectory;
 
+	let lobbyButton = Engine.GetGUIObjectByName("lobbyButton");
+	lobbyButton.tooltip = colorizeHotkey(translate("%(hotkey)s: Toggle the multiplayer lobby in a dialog window."), "lobby");
+	lobbyButton.hidden = g_GameData.gui.isInGame || !Engine.HasXmppClient();
+
+	// Right-align lobby button
+	let lobbyButtonSize = lobbyButton.size;
+	let lobbyButtonWidth = lobbyButtonSize.right - lobbyButtonSize.left;
+	lobbyButtonSize.right = (replayButton.hidden ? Engine.GetGUIObjectByName("continueButton").size.left : replayButton.size.left) - 10;
+	lobbyButtonSize.left = lobbyButtonSize.right - lobbyButtonWidth;
+	lobbyButton.size = lobbyButtonSize;
+}
+
+function initTeamData()
+{
 	// Panels
 	g_PlayerCount = g_GameData.sim.playerStates.length - 1;
 
 	if (g_GameData.sim.mapSettings.LockTeams)
 	{
 		// Count teams
-		for (let t = 0; t < g_PlayerCount; ++t)
+		for (let player = 1; player <= g_PlayerCount; ++player)
 		{
-			let playerTeam = g_GameData.sim.playerStates[t+1].team;
-			g_Teams[playerTeam] = (g_Teams[playerTeam] || 0) + 1;
+			let playerTeam = g_GameData.sim.playerStates[player].team;
+			if (!g_Teams[playerTeam])
+				g_Teams[playerTeam] = [];
+			g_Teams[playerTeam].push(player);
 		}
 
-		if (g_Teams.length == g_PlayerCount)
+		if (g_Teams.every(team => team && team.length < 2))
 			g_Teams = false;	// Each player has his own team. Displaying teams makes no sense.
 	}
 	else
@@ -469,19 +557,6 @@ function init(data)
 
 	// Erase teams data if teams are not displayed
 	if (!g_Teams)
-	{
 		for (let p = 0; p < g_PlayerCount; ++p)
 			g_GameData.sim.playerStates[p+1].team = -1;
-	}
-
-	g_WithoutTeam = g_PlayerCount;
-	if (g_Teams)
-	{
-		// Count players without team (or all if teams are not displayed)
-		for (let i = 0; i < g_Teams.length; ++i)
-			g_WithoutTeam -= g_Teams[i] ? g_Teams[i] : 0;
-	}
-
-	initCharts();
-	selectPanel(Engine.GetGUIObjectByName("scorePanelButton"));
 }
