@@ -8,14 +8,16 @@
 // TODO: Type errors if v not instanceof Vector classes
 // TODO: Possibly implement in C++
 
-function Vector2D(x, y)
+function Vector2D(x = 0, y = 0)
 {
-	if (arguments.length == 2)
-		this.set(x, y);
-	else
-		this.set(0, 0);
+	this.set(x, y);
 }
 
+Vector2D.prototype.clone = function()
+{
+	return new Vector2D(this.x, this.y);
+};
+
 // Mutating 2D functions
 //
 // These functions modify the current object,
@@ -58,37 +60,79 @@ Vector2D.prototype.div = function(f)
 
 Vector2D.prototype.normalize = function()
 {
-	var mag = this.length();
-	if (!mag)
+	let magnitude = this.length();
+	if (!magnitude)
 		return this;
 
-	return this.div(mag);
+	return this.div(magnitude);
 };
 
 /**
  * Rotate a radians anti-clockwise
  */
-Vector2D.prototype.rotate = function(a)
+Vector2D.prototype.rotate = function(angle)
 {
-	var sin = Math.sin(a);
-	var cos = Math.cos(a);
-	var x = this.x * cos + this.y * sin;
-	var y = this.y * cos - this.x * sin;
-	this.x = x;
-	this.y = y;
-	return this;
+	let sin = Math.sin(angle);
+	let cos = Math.cos(angle);
+
+	return this.set(
+		this.x * cos + this.y * sin,
+		-this.x * sin + this.y * cos);
+};
+
+/**
+ * Rotate radians anti-clockwise around the specified rotation center.
+ */
+Vector2D.prototype.rotateAround = function(angle, center)
+{
+	return this.sub(center).rotate(angle).add(center);
+};
+
+/**
+ * Convert to integer coordinates.
+ */
+Vector2D.prototype.round = function()
+{
+	return this.set(Math.round(this.x), Math.round(this.y));
+};
+
+Vector2D.prototype.floor = function()
+{
+	return this.set(Math.floor(this.x), Math.floor(this.y));
+};
+
+Vector2D.prototype.toFixed = function(digits)
+{
+	return this.set(this.x.toFixed(digits), this.y.toFixed(digits));
 };
 
 // Numeric 2D info functions (non-mutating)
 //
 // These methods serve to get numeric info on the vector, they don't modify the vector
 
+/**
+ * Returns a vector that forms a right angle with this one.
+ */
+Vector2D.prototype.perpendicular = function()
+{
+	return new Vector2D(-this.y, this.x);
+};
+
+/**
+ * Computes the scalar product of the two vectors.
+ * Geometrically, this is the product of the length of the two vectors and the cosine of the angle between them.
+ * If the vectors are orthogonal, the product is zero.
+ */
 Vector2D.prototype.dot = function(v)
 {
 	return this.x * v.x + this.y * v.y;
 };
 
-// get the non-zero coordinate of the vector cross
+/**
+ * Computes the non-zero coordinate of the cross product of the two vectors.
+ * Geometrically, the cross of the vectors is a 3D vector perpendicular to the two 2D vectors.
+ * The returned number corresponds to the area of the parallelogram with the vectors for sides.
+ */
 Vector2D.prototype.cross = function(v)
 {
 	return this.x * v.y - this.y * v.x;
@@ -105,7 +149,7 @@ Vector2D.prototype.length = function()
 };
 
 /**
- * Compare this length to the length of v,
+ * Compare this length to the length of v.
  * @return 0 if the lengths are equal
  * @return 1 if this is longer than v
  * @return -1 if this is shorter than v
@@ -113,22 +157,26 @@ Vector2D.prototype.length = function()
  */
 Vector2D.prototype.compareLength = function(v)
 {
-	var dDist = this.lengthSquared() - v.lengthSquared();
-	if (!dDist)
-		return dDist == 0 ? 0 : NaN;
-	return dDist < 0 ? -1 : 1;
+	return Math.sign(this.lengthSquared() - v.lengthSquared());
 };
 
 Vector2D.prototype.distanceToSquared = function(v)
 {
-	var dx = this.x - v.x;
-	var dy = this.y - v.y;
-	return dx * dx + dy * dy;
+	return Math.euclidDistance2DSquared(this.x, this.y, v.x, v.y);
 };
 
 Vector2D.prototype.distanceTo = function(v)
 {
-	return Math.sqrt(this.distanceToSquared(v));
+	return Math.euclidDistance2D(this.x, this.y, v.x, v.y);
+};
+
+/**
+ * Returns the angle going from this position to v.
+ * Angles are between -PI and PI. E.g., north is 0, east is PI/2.
+ */
+Vector2D.prototype.angleTo = function(v)
+{
+	return Math.atan2(v.x - this.x, v.y - this.y);
 };
 
 // Static 2D functions
@@ -136,11 +184,6 @@ Vector2D.prototype.distanceTo = function(v)
 // Static functions that return a new vector object.
 // Note that object creation is slow in JS, so use them only when necessary
 
-Vector2D.clone = function(v)
-{
-	return new Vector2D(v.x, v.y);
-};
-
 Vector2D.from3D = function(v)
 {
 	return new Vector2D(v.x, v.z);
@@ -156,6 +199,11 @@ Vector2D.sub = function(v1, v2)
 	return new Vector2D(v1.x - v2.x, v1.y - v2.y);
 };
 
+Vector2D.isEqualTo = function(v1, v2)
+{
+	return v1.x == v2.x && v1.y == v2.y;
+};
+
 Vector2D.mult = function(v, f)
 {
 	return new Vector2D(v.x * f, v.y * f);
@@ -166,15 +214,29 @@ Vector2D.div = function(v, f)
 	return new Vector2D(v.x / f, v.y / f);
 };
 
-Vector2D.avg = function(vectorList)
+Vector2D.min = function(v1, v2)
+{
+	return new Vector2D(Math.min(v1.x, v2.x), Math.min(v1.y, v2.y));
+};
+
+Vector2D.max = function(v1, v2)
+{
+	return new Vector2D(Math.max(v1.x, v2.x), Math.max(v1.y, v2.y));
+};
+
+Vector2D.average = function(vectorList)
 {
 	return Vector2D.sum(vectorList).div(vectorList.length);
 };
 
 Vector2D.sum = function(vectorList)
 {
-	var sum = new Vector2D();
-	vectorList.forEach(function(v) {sum.add(v);});
+	// Do not use for...of nor array functions for performance
+	let sum = new Vector2D();
+
+	for (let i = 0; i < vectorList.length; ++i)
+		sum.add(vectorList[i]);
+
 	return sum;
 };
 
@@ -185,14 +247,16 @@ Vector2D.sum = function(vectorList)
 //
 /////////////////////////////////////////////////////////////////////
 
-function Vector3D(x, y, z)
+function Vector3D(x = 0, y = 0, z = 0)
 {
-	if (arguments.length == 3)
-		this.set(x, y, z);
-	else
-		this.set(0, 0, 0);
+	this.set(x, y, z);
 }
 
+Vector3D.prototype.clone = function()
+{
+	return new Vector3D(this.x, this.y, this.z);
+};
+
 // Mutating 3D functions
 //
 // These functions modify the current object,
@@ -240,11 +304,29 @@ Vector3D.prototype.div = function(f)
 
 Vector3D.prototype.normalize = function()
 {
-	var mag = this.length();
-	if (!mag)
+	let magnitude = this.length();
+	if (!magnitude)
 		return this;
 
-	return this.div(mag);
+	return this.div(magnitude);
+};
+
+/**
+ * Convert to integer coordinates.
+ */
+Vector3D.prototype.round = function()
+{
+	return this.set(Math.round(this.x), Math.round(this.y), Math.round(this.z));
+};
+
+Vector3D.prototype.floor = function()
+{
+	return this.set(Math.floor(this.x), Math.floor(this.y), Math.floor(this.z));
+};
+
+Vector3D.prototype.toFixed = function(digits)
+{
+	return this.set(this.x.toFixed(digits), this.y.toFixed(digits), this.z.toFixed(digits));
 };
 
 // Numeric 3D info functions (non-mutating)
@@ -256,6 +338,18 @@ Vector3D.prototype.dot = function(v)
 	return this.x * v.x + this.y * v.y + this.z * v.z;
 };
 
+/**
+ * Returns a vector perpendicular to the two given vectors.
+ * The length of the returned vector corresponds to the area of the parallelogram with the vectors for sides.
+ */
+Vector3D.prototype.cross = function(v)
+{
+	return new Vector3D(
+		this.y * v.z - this.z * v.y,
+		this.z * v.x - this.x * v.z,
+		this.x * v.y - this.y * v.x);
+};
+
 Vector3D.prototype.lengthSquared = function()
 {
 	return this.dot(this);
@@ -275,30 +369,22 @@ Vector3D.prototype.length = function()
  */
 Vector3D.prototype.compareLength = function(v)
 {
-	var dDist = this.lengthSquared() - v.lengthSquared();
-	if (!dDist)
-		return dDist == 0 ? 0 : NaN;
-	return dDist < 0 ? -1 : 1;
+	return Math.sign(this.lengthSquared() - v.lengthSquared());
 };
 
 Vector3D.prototype.distanceToSquared = function(v)
 {
-	var dx = this.x - v.x;
-	var dy = this.y - v.y;
-	var dz = this.z - v.z;
-	return dx * dx + dy * dy + dz * dz;
+	return Math.euclidDistance3DSquared(this.x, this.y, this.z, v.x, v.y, v.z);
 };
 
 Vector3D.prototype.distanceTo = function(v)
 {
-	return Math.sqrt(this.distanceToSquared(v));
+	return Math.euclidDistance3D(this.x, this.y, this.z, v.x, v.y, v.z);
 };
 
 Vector3D.prototype.horizDistanceToSquared = function(v)
 {
-	var dx = this.x - v.x;
-	var dz = this.z - v.z;
-	return dx * dx + dz * dz;
+	return Math.euclidDistance2DSquared(this.x, this.z, v.x, v.z);
 };
 
 Vector3D.prototype.horizDistanceTo = function(v)
@@ -306,16 +392,19 @@ Vector3D.prototype.horizDistanceTo = function(v)
 	return Math.sqrt(this.horizDistanceToSquared(v));
 };
 
+/**
+ * Returns the angle going from this position to v.
+ */
+Vector3D.prototype.horizAngleTo = function(v)
+{
+	return Math.atan2(v.x - this.x, v.z - this.z);
+};
+
 // Static 3D functions
 //
 // Static functions that return a new vector object.
 // Note that object creation is slow in JS, so use them only when really necessary
 
-Vector3D.clone = function(v)
-{
-	return new Vector3D(v.x, v.y, v.z);
-};
-
 Vector3D.add = function(v1, v2)
 {
 	return new Vector3D(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
@@ -326,6 +415,11 @@ Vector3D.sub = function(v1, v2)
 	return new Vector3D(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
 };
 
+Vector3D.isEqualTo = function(v1, v2)
+{
+	return v1.x == v2.x && v1.y == v2.y && v1.z == v2.z;
+};
+
 Vector3D.mult = function(v, f)
 {
 	return new Vector3D(v.x * f, v.y * f, v.z * f);
